package dbridge;

import java.io.*;
import java.util.*;
import javax.mail.*;
import javax.mail.internet.*;
import javax.activation.*;
import java.sql.Timestamp;

abstract class CBaseMessageBuilder {

  public CBaseMessageBuilder(String sComments, String sxTag,
                             String sPublicKey) {
    mPublicKey = sPublicKey;
    mComments = sComments;
    smXTag = sxTag;

    mSecretKey = "";
    mPass = "";

    mID = "";
    mPart = 0;
    mTotal = 0;
    mMethodDN = DBConsts.MDN_NONE;
    mMethodSign = DBConsts.MDN_NONE;
    mReplayToDN = "";
    mKeyIDDN = "";
    mFilesToDel = new Vector();
  }

  protected void SetDNHeaders(Part pMessagePart, boolean isMessage) throws MessagingException {
    if (mMethodDN != DBConsts.MDN_NONE) {
      if (mMethodDN == DBConsts.MDN_PGP || mMethodDN == DBConsts.MDN_MIX) {
        //pMessagePart.setContentID(FileTools.makeUniqName()+"@builplaintxtmessage.de");
        pMessagePart.setHeader("ContentId","<"+FileTools.makeUniqName()+"@dbridge.de"+">");
        pMessagePart.setHeader(DBConsts.XTAG_ID, mID);
        pMessagePart.setHeader(DBConsts.XTAG_PART, new Integer(mPart).toString());
        pMessagePart.setHeader(DBConsts.XTAG_TOTAL,
                               new Integer(mTotal).toString());
        if (!isMessage) {
          pMessagePart.setHeader(DBConsts.XTAG_DNKEYID, mKeyIDDN);
        }
      }
      // disposition notification to
      if (isMessage) {
       pMessagePart.setHeader(DBConsts.DN_RFC, this.mReplayToDN);
       /* It is not clear why,when and where we need this header...
                 else
                 pMessagePart.setHeader(DBConsts.XTAG_DNTO, mReplayToDN);*/
      }
    }
  }

  protected MimeBodyPart MakeTempBodyPart(MimeMultipart mmpPart) throws
      MessagingException {
    MimeBodyPart tmpPart = new MimeBodyPart();
    tmpPart.setContent(mmpPart);
    tmpPart.setHeader("MIME-Version", "1.0");
    tmpPart.addHeader("Content-Type", mmpPart.getContentType());
    SetDNHeaders(tmpPart, false);
    return tmpPart;
  }

  abstract protected MimeMultipart BuildMimePgpMultipart(MimeBodyPart mbpPart,
      String sFileNameGpg) throws MessagingException, IOException,
      wrapperException;

  protected MimeBodyPart BuildPlainCommentMessage() throws MessagingException {
    MimeBodyPart mbp = new MimeBodyPart();
    if ( (mComments != null) && (mComments.length() != 0)) {
      mbp.setContent(mComments, "text/plain");
      mbp.setHeader(DBConsts.XTAG_STUDYID, smXTag);

      String saveTo = FileTools.makeUniqName();
      saveTo += ".";
      saveTo += BridgeApp.getMimer().getExtension("text/plain");
      mbp.setFileName(saveTo);
      mbp.setDisposition(Part.ATTACHMENT);
    }
    return mbp;
  }

  protected MimeBodyPart BuildPlainDataMessage(String attachedFileName) throws
      MessagingException, FileNotFoundException, IOException {
    File attachedFile = new File(attachedFileName);
    if (!attachedFile.exists()) {
      String errorStr = "File " + attachedFileName +
          " does not exists...";
      throw new RuntimeException(errorStr);
    }

    FileDataSource fds = new FileDataSource(attachedFile);
    ////MimetypesFileTypeMap ftm = new MimetypesFileTypeMap();
    ////ftm.addMimeTypes("application/dicom dcm dicm");
    ////fds.setFileTypeMap(ftm);

    // classify the file type
    fds.setFileTypeMap(BridgeApp.getMimer().getMimeTypesMap());

    // create the second message part
    MimeBodyPart mbp = new MimeBodyPart();

    // attach the file to the message
    mbp.setDataHandler(new DataHandler(fds));
    String fileName = fds.getName();
    mbp.setFileName(fileName); //+"\""
        
    //
    // make classification of dicom files stronger
    // other types are classified automatically using file extensions
    //
    
    /**
     * Experimental code. In according with RFC3240, the dicom attachment would be 
     * specified as base64. Due to modification of base64 parsers in many mail clients,
     * we add this information too.
     */
    if (FileTools.isDICMfile(fds.getInputStream())) {
      mbp.setHeader("Content-Type",
                    "application/dicom;\n\tname=\"" + fileName + "\"");
      // experiment 9.07.08
      mbp.setHeader("Content-Transfer-Encoding","base64");
    }
    else {
    	mbp.setHeader("Content-Type",mbp.getDataHandler().getContentType());
        // experiment 9.07.08
    	mbp.setHeader("Content-Transfer-Encoding","base64");
      mbp.setHeader(DBConsts.XTAG_STUDYID, smXTag);
    }

    // create the Multipart and its parts to it
    //Multipart mp = new MimeMultipart();
    //mp.addBodyPart(mbp);

    return mbp;
  }

   
  public static void UpdateDBRecord(String MessageID_old, String MessageID_new){
	  
      if (dbQuery.setQueryUpdate("update dmt set id=\'" + MessageID_new + "\' where id like \'" +
    		  MessageID_old + "\'") == null) {
    	  BridgeApp.LogError(Logs.Srvcs.SetText("Updating record id = " + MessageID_old + " failed"));
      }
      
      if (dbQuery.setQueryUpdate("update dmt set sent=1 where id like \'" +
    		  MessageID_new + "\'") == null) {
    	  BridgeApp.LogError(Logs.Srvcs.SetText("Updating record send with id " + MessageID_new + " failed"));
      }
  }
  
  
  public MimeMessage BuildMessage(String sAttachedFile) {
    boolean bOk = false;
    MimeMessage msgPGP = null;
    MimeBodyPart mbp = null;
    String sFileNameGpg = BridgeApp.getRoot() + FileTools.makeUniqName() +
        "." + DBConsts.ASC_TMP_EXT;
    vfilesRecord v = new vfilesRecord(); // db record
    dmtRecord r = new dmtRecord(); // db record
    try {
      //
      // Part 1: build main part
      if ( (sAttachedFile != null) && (sAttachedFile.length() != 0)) {
        v.m_name = new String(sAttachedFile);
        v.m_number = 1;
        mbp = BuildPlainDataMessage(sAttachedFile);
      }
      else {
        mbp = BuildPlainCommentMessage();
      }
      //
      //Part 2: Build MIME/PGP Message
      MimeMultipart tmpRoot = new MimeMultipart();
      tmpRoot.addBodyPart(mbp);
      MimeBodyPart tmpPart = MakeTempBodyPart(tmpRoot);
      MimeMultipart mpRoot = BuildMimePgpMultipart(tmpPart, sFileNameGpg);

      Properties props = System.getProperties();
      Session session = Session.getInstance(props, null);
      msgPGP = new MimeMessage(session);
      msgPGP.setContent(mpRoot);
      SetDNHeaders(msgPGP, true);

      msgPGP.saveChanges();
//        --------------------------------------------------------------------
      FileOutputStream fosPGP = new FileOutputStream("EncryptedMail1.eml",false);
       msgPGP.writeTo(fosPGP);
       fosPGP.close();
//          --------------------------------------------------------------------
      bOk = true;
    }
    catch (Exception ex) {
      BridgeApp.LogError(Logs.Srvcs.SetText("Message building error: " +
                                            ex));
    }
    mFilesToDel.add(sFileNameGpg);

    if (bOk) {
      // Disposition Notification
      // Two methods are currently supported: RFC and MIME/PGP
      if (this.mMethodDN != DBConsts.MDN_NONE) {
        // RFC Disposition Notification
        // here a record in DB tables has to be created
        // Message-Id is used as a transaction Id
        // One record is created in both tables per the file
        String message_id = "null";
        try {
          message_id = msgPGP.getMessageID();
        }
        catch (MessagingException mex) {
          System.out.println(mex);
        }

        if (this.mMethodDN == DBConsts.MDN_RFC) {
          r.m_id = new String(message_id); // Transaction id
          v.m_id = new String(message_id); // Transaction id
          r.m_start_time = new Timestamp(System.currentTimeMillis());
          dbQuery query = new dbQuery();
          if (query.setQueryInsertDmt(r.toString()) == null) {
            // error insert
          }
          if (query.setQueryInsertFiles(v.toString()) == null) {
            // error insert
          }

        }
        // MIMEPGP Disposition Notification
        // here a record in DB tables has to be created
        // X-Telelemedicine-Id is used as a transaction Id
        // One record in the dmt belongs to a whole sending collection
        // One record in the vfiles belongs to the sending file
        if (this.mMethodDN == DBConsts.MDN_PGP ||
            this.mMethodDN == DBConsts.MDN_MIX) {
          r.m_id = new String(this.mID); // Transaction id
          r.m_sent = 1;
          r.m_total = this.mTotal;

          r.m_start_time = new Timestamp(System.currentTimeMillis());
          v.m_id = new String(this.mID); // Transaction id
          v.m_number = this.mPart;
          v.m_status = true;
          v.m_mid = message_id; // ToDo
          v.m_report = "sent";

          dbQuery query = new dbQuery();
          String r_str = new String("");
          // update dmt or insert?
          r_str = query.setQueryDmt("select * from dmt where id like \'" +
                                    r.m_id + "\'");
          if (r_str == null) {
            System.out.println("Builder: The record with id =" + r.m_id +
                               " was not found, inserting ...");
            // insert the record here
            if (query.setQueryInsertDmt(r.toString()) == null) {
              // error insert
              System.out.println("Builder: The record dmt was not inserted");
            }
            if (query.setQueryInsertFiles(v.toString()) == null) {
              // error insert
              System.out.println("Builder: The record vfiles was not inserted");
            }
          }
          else {
            // update
            // DMT: ch
            dmtRecord r_tmp = new dmtRecord(r_str);
            System.out.println("Builder: Try to update sent field by " +
                               r_tmp.m_sent + 1);
            //r.m_sent = r_tmp.m_sent++; // increase 1
            int index = r_tmp.m_dn_index; // table key
            String v_str = new String("");
            System.out.println("Builder: The record with id =" + r.m_id +
                               " index=" + index + " will be updated");
            
            // Functionality is moved to UpdateDBRecord()
            // update sent
            //if (query.setQueryUpdate("update dmt set sent=" + r.m_sent +
            //                         " where dn_index=" + index) == null) {
            //  System.out.println("Builder: update sent failed");
            //}

            // check either the attached file has been sent already or not
            v_str = query.setQueryInsertFiles(v.toString());
            if (v_str == null) {
              // insert a new record
              if (query.setQueryInsertFiles(v.toString()) == null) {
                // error insert
                System.out.println(
                    "Builder: The record vfiles was not inserted");
              }
            }
            else {
              // increase resent
              r.m_resent = r_tmp.m_resent++;
              // RESERVED for future
              if (query.setQueryUpdate("update dmt set resent=" + r.m_resent +
                                       " where dn_index=" + index) == null) {
                System.out.println("Builder: update resent failed");
              }

            } // end if exist
          }
        }

      } // end if NONE

      return msgPGP;
    }
    else {
      throw (new RuntimeException("Build message failed\n"));
    }
  }

  public void ClearResources() {
    for (int fileCount = 0; fileCount < mFilesToDel.size(); fileCount++) {
      String sFileToDel = (String) mFilesToDel.get(fileCount);
      if ( (sFileToDel != null) && (sFileToDel.length() != 0)) {
        (new File(sFileToDel)).delete();
      }
    }
  }

  protected MimeMultipart BuildPlainReportRoot(String body, String recipient,
                                               String disposition_type,
                                               String sOrigMessageID,
                                               String sContentID,
                                               boolean isRFC) throws
      MessagingException {
    MimeMultipart mpRoot;
    if (isRFC) {
      mpRoot = new MimeMultipart(
          "report;\n\treport-type=disposition-notification");
    }
    else {
      mpRoot = new MimeMultipart(
          "report;\n\treport-type=" + DBConsts.XTAG_REPORT);
    }

    MimeBodyPart freePart = new MimeBodyPart();
    freePart.setText(
        "This report was generated by DBridge communication software.");

    MimeBodyPart reportPart = new MimeBodyPart();
    reportPart.setText(body);
    reportPart.setHeader("Content-Type", "message/disposition-notification");
    reportPart.removeHeader("Content-Transfer-Encoding");
    reportPart.setHeader("Reporting-UA", "DBridge 3.0"); // add version here
    reportPart.setHeader("Final-Recipient", "rfc822; " + recipient);
    reportPart.setHeader(DBConsts.OMID_RFC, sOrigMessageID);
    if (!isRFC) {
      reportPart.setHeader(
          DBConsts.XTAG_CONTENTID, sContentID);
    }
    //Disposition: manual-action/MDN-sent-manually; displayed
    reportPart.setHeader("Disposition", disposition_type);

    mpRoot.addBodyPart(freePart);
    mpRoot.addBodyPart(reportPart);

    return mpRoot;
  }

  public MimeMessage BuildReportMessage(String body, String recipient,
                                        String disposition_type,
                                        String sOrigMessageID,
                                        String sContentID,
                                        boolean isRFC, boolean needMimePgp) {
    boolean bOk = false;
    MimeMessage msgReport = null;
    MimeMultipart tmpRoot = null;
    MimeMultipart mpRoot = null;
    String sFileNameGpg = BridgeApp.getRoot() + FileTools.makeUniqName() +
        "." + DBConsts.ASC_TMP_EXT;
    try {
      tmpRoot = BuildPlainReportRoot(body, recipient, disposition_type,
                                     sOrigMessageID, sContentID, isRFC);
      if (needMimePgp) {
        //build encrypted...
        MimeBodyPart tmpPart = MakeTempBodyPart(tmpRoot);
        mpRoot = BuildMimePgpMultipart(tmpPart, sFileNameGpg);
      }
      else {
        mpRoot = tmpRoot;
      }
      Properties props = System.getProperties();
      Session session = Session.getInstance(props, null);
      msgReport = new MimeMessage(session);
      msgReport.setContent(mpRoot);
      msgReport.saveChanges();
//    --------------------------------------------------------------------
//FileOutputStream fosPGP = new FileOutputStream("outs"+File.separator+"ReportMail.eml",false);
//msgReport.writeTo(fosPGP);
//fosPGP.close();
//      --------------------------------------------------------------------
      bOk = true;
    }
    catch (Exception ex) {
      BridgeApp.LogError(Logs.Srvcs.SetText("Message building error: " +
                                            ex));
    }
    mFilesToDel.add(sFileNameGpg);

    if (bOk) {
      return msgReport;
    }
    else {
      throw (new RuntimeException("Build message failed\n"));
    }
  }

//--------------------------Properties-------------------------------------------
  public String getSecretKey() {
    return mSecretKey;
  }

  public void setSecretKey(String sKeyID) {
    this.mSecretKey = sKeyID;
  }

  public String getPass() {
    return mPass;
  }

  public void setPass(String sPass) {
    this.mPass = sPass;
  }

  public String getID() {
    return mID;
  }

  public void setID(String sID) {
    this.mID = sID;
  }

  public int getPart() {
    return mPart;
  }

  public void setPart(int iPart) {
    this.mPart = iPart;
  }

  public int getTotal() {
    return mTotal;
  }

  public void setTotal(int iTotal) {
    this.mTotal = iTotal;
  }

  public int getMethodDN() {
    return mMethodDN;
  }

  public void setMethodDN(int iMethodDN) {
    this.mMethodDN = iMethodDN;
  }

  public int getMethodSign() {
    return mMethodSign;
  }

  public void setMethodSign(int iMethodSign) {
    this.mMethodSign = iMethodSign;
  }

  public String getReplayToDN() {
    return mReplayToDN;
  }

  public void setReplayToDN(String sReplayToDN) {
    this.mReplayToDN = sReplayToDN;
  }

  public String getKeyIDDN() {
    return mKeyIDDN;
  }

  public void setKeyIDDN(String sKeyIDDN) {
    this.mKeyIDDN = sKeyIDDN;
  }

  protected Vector mFilesToDel;
  protected String smXTag;
  protected String mPublicKey;
  protected String mComments;

  protected String mSecretKey;
  protected String mPass;

  protected String mID;
  protected int mPart;
  protected int mTotal;
  protected int mMethodDN;
  protected int mMethodSign;
  protected String mReplayToDN;
  protected String mKeyIDDN;

}
