package dbridge;



/**

 * <p>Title: DicomBridge</p>

 * <p>Description: Transfer  Images over e-mail</p>

 * <p>Copyright: Copyright (c) 2003</p>

 * <p>Company: a</p>

 * @version 1.0

 */

/*
 * Modified 22.06.04
 * Modified 28.03.07
 */

import java.util.*;

//
// This Interface should be implemented by gui to use these FolderCheckers
//

interface IGuiWithFolderChecker
{
    public void UpdateFiles(Vector vFiles, String sFolder);
}

//
// OnTimer task. Perform Notification of gui, about files in directory
//
class CGuiUpdateTask extends TimerTask
{
    public CGuiUpdateTask(CFolderChecker director, IGuiWithFolderChecker gui)
	{
		mDirector = director;
		mGui = gui;
	}
    
    public void run()
    {
        UpdateGui();
    }

    private void UpdateGui()
    {
        mGui.UpdateFiles(mDirector.GetFilesVector(), mDirector.GetFolderName());
    }

    private CFolderChecker mDirector;

    private IGuiWithFolderChecker mGui;

}

//
// CFolderChecker. works accordingly to its settings:
// - OnTimer performs mTimerTask->Run()
// - ProcessFiles  mFilesProcessor.ProcessFiles(vFiles);
//

class CFolderChecker
{
    protected CFolderChecker(String sFolder)
    {
        mFolderName = sFolder;
        mTimer = new Timer();
        mTimerTask = null;
    }

    public void StartChecking(int iSek)
    {
        if (mTimerTask != null)
        {
            mTimer.schedule(mTimerTask, 0, iSek * 1000);
        }
        else
        {
            System.out.println(
                    "Checking is not started. No TimerTask object for this timer.");
        }
    }

    public void StopChecking()
    {
        mTimer.cancel();
    }

    public Vector GetFilesVector()
    {
        //return FileTools.GetFilesFromDir(mFolderName);
        // 28.03.07 return FileTools.GetNotHiddenFilesFromDir(mFolderName);
        return FileTools.GetNotHiddenFilesWithinSubdirs(mFolderName);
    }

    public String GetFolderName()
    {
        return mFolderName;
    }

    protected void SetTimerTask(TimerTask task)
    {
        mTimerTask = task;
    }

    protected TimerTask GetTimerTask()
    {
        return mTimerTask;
    }

    //Period in sec.

    private long      checkPeriod;
    private Timer     mTimer;
    private TimerTask mTimerTask;
    private String    mFolderName;
}


//
// CFolderChecker with settings: Task=CGuiUpdateTask, FilesProcessor=Null
//
class CGuiFolderChecker extends CFolderChecker
{
    public CGuiFolderChecker(String sFolder, IGuiWithFolderChecker gui)
    {
        super(sFolder);
        //
        // it is necessary to install this params
        //
        SetTimerTask(new CGuiUpdateTask(this, gui));
    }
}



