/*
* Created: 19.02.2006 by Bob
* (Moved from file GpgWrapper)
*/


package dbridge;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;

/**
 * Containes the GPG key information
 */
public class GpgKeyInfo implements Cloneable{
	
		  public GpgKeyInfo () {
			init_labels();  
		    keyWithColon = new String();
		    fingerPrint = new String();
		    subKey=null;
		  }

		  public GpgKeyInfo (String key) {
			init_labels();  
		    keyWithColon = new String(key);
		    fingerPrint = new String();
		    subKey=null;
		  }

		  public GpgKeyInfo (String key, String fpr) {
			init_labels();  
		    keyWithColon = new String(key);
		    fingerPrint = new String(fpr);
		    if(fpr.startsWith("fpr")){
		      fingerPrint=getTokenN(fpr,USERIDANDFPRPOSITION);
		    }
		    subKey=null;
		  }
		  
		  /**
		   * Copy Constructor
		   * @param other
		   */
		  public GpgKeyInfo (GpgKeyInfo other){
			  init_labels();
			  this.copy(other);
		  }
		  
		  public Object clone(){
			    return new GpgKeyInfo(this);
			  }

		  /**
		   * Copy the other object
		   * @param obj
		   */
		  public void copy(GpgKeyInfo obj){
			  this.fingerPrint = new String(obj.fingerPrint);
			  this.keyWithColon = new String(obj.keyWithColon);
			  this.subKey.copy(obj.subKey);
			  this.passwd = (char []) obj.passwd.clone();
		  }
		  
		  public void setSubKey(String subKeyStr) {
		    subKey=new GpgKeyInfo(subKeyStr);
		  }

		  public void setSubKey(String subKeyStr, String subKeyFpr) {
		    subKey=new GpgKeyInfo(subKeyStr,subKeyFpr);
		  }

		  public void setSubKeyFingerprint(String skFpr) {
		    if(subKey!=null)
		     subKey.setFingerPrint(skFpr);
		  }

		  public GpgKeyInfo getSubKey() {
		    return subKey;
		  }

		  public void setFingerPrint(String fpr) {
		    fingerPrint = new String(fpr);
		    if(fpr.startsWith("fpr")){
		      fingerPrint=getTokenN(fpr,USERIDANDFPRPOSITION);
		    }
		  }

		  public String getFingerPrint() {
		      return fingerPrint;
		  }

		  /**
		   * Return last 8 symbols of the fingerprint
		   * @return String
		   */
		  public String getKeyID() {
		    int n = fingerPrint.length();
		    if ( n<8 ) return labels.getString("bad_id");
		    return fingerPrint.substring(n-8,n);
		  }
		  
		  /**
		   * Check if keyID matches with ours
		   * @param keyID
		   * @return
		   */
		  public boolean checkKeyID(String keyID){
			  boolean result = false;
			  if ((this.getKeyID().compareTo(keyID)==0) && 
					  (keyID.compareTo(labels.getString("bad_id"))!=0)) result=true;
			  return result;
		  }
		  
		  /**
		   * Set GPG private key password
		   * @deprecated
		   * @param pwd
		   * @return void
		   */
		  public void setPasswd(char[] pwd){
			  passwd=pwd;
		  }
		  
		  /**
		   * Get GPG private key password
		   * @deprecated
		   * @return String
		   */
		  public char[] getPasswd(){
			  return passwd;
		  }

		  public String getKeyType() {
		    return getTokenN(keyWithColon,KEYTYPEPOSITION);
		  }

		/**
		 * Get the Key Trust level
		 * @param needFullDescr
		 * @return String
		 */
		  public String getTrust(boolean needFullDescr) {
			String trust = getTokenN(keyWithColon, TRUSTPOSITION);
			String result = labels.getString("undefined");
			if (trust.length() > 0) {
				result = new String(trust);
				if (needFullDescr) {
					switch (trust.charAt(0)) {
					case 'o':
						result = labels.getString("unknown");
						break;
					case 'i':
						result = labels.getString("invalid");
						break;
					case 'r':
						result = labels.getString("revoked");
						break;
					case 'e':
						result = labels.getString("expired");
						break;
					case '-':
						result = labels.getString("unk_trust");
						break;
					case 'q':
						result = labels.getString("und_trust");
						break;
					case 'n':
						result = labels.getString("no_trust");
						break;
					case 'm':
						result = labels.getString("marg_trust");
						break;
					case 'f':
						result = labels.getString("full_trust");
						break;
					case 'u':
						result = labels.getString("ult_trust");
						break;
					case 'd':
						result = labels.getString("disabled");
						break;
					default:
						result = labels.getString("unkn_val") + trust;
					}
				}
			}
			return result;
		}

		  public String getKeySize() {
		    return getTokenN(keyWithColon,KEYSIZEPOSITION);
		  }
		  
		  public String getAlgorithm(boolean needAlgoName) {
		    String algNumber=getTokenN(keyWithColon,ALGORITHMPOSITION);
		    String result = new String(algNumber);
		    if (needAlgoName) {
		      int algoId = Integer.parseInt(algNumber);
		      switch (algoId) {
		        case 1:  result = labels.getString("alg_RSA");
		                 break;
				    case 16: result = labels.getString("alg_ELG_so");
				             break;
				    case 17: result = labels.getString("alg_DSA_so");
				             break;
				    case 20: result = labels.getString("alg_ELG_se");
				             break;
		  		  default: result = labels.getString("alg_unk") + algNumber;
		      }
		    }
		    return result;
		  }

		  public String getCreationDate(){
		    return getTokenN(keyWithColon,CREATIONDATEPOSITION);
		  }

		  public String getExpirationDate() {
		    String expDate=getTokenN(keyWithColon,EXPIRATIONDATEPOSITION);
		    if (expDate.length()==0)
		      expDate=labels.getString("exp_never");
		    return expDate;
		  }

		  public String getUserID() {
		    return getTokenN(keyWithColon,USERIDANDFPRPOSITION);
		  }

		  public String toString() {
		    //return keyWithColon +" "+fingerPrint;
			  return this.getKeyID();
		  }

		  private void init_labels(){
			    Locale currentLocale = BridgeApp.getCurrentLocale();
			    labels = ResourceBundle.getBundle("dbridge.prop.GpgKeyInfoBundle",currentLocale);

		  }
		  
		  private static String getTokenN (String source, int tokenNumber) {
		    String result=new String();
		    String [] parsed=source.split(":");
		    if (tokenNumber<parsed.length) {
		      result = parsed [tokenNumber];
		    }
		    return result;
		  }
		  
		  private final static int KEYTYPEPOSITION = 0;
		  private final static int TRUSTPOSITION = 1;
		  private final static int KEYSIZEPOSITION = 2;
		  private final static int ALGORITHMPOSITION = 3;
		  private final static int CREATIONDATEPOSITION = 5;
		  private final static int EXPIRATIONDATEPOSITION = 6;
		  private final static int USERIDANDFPRPOSITION = 9;

		  private String keyWithColon;
		  private String fingerPrint;
		  private char[] passwd;
		  private GpgKeyInfo subKey;
		  private ResourceBundle labels;

		  //----- DEBUG ------------------------------------------------------
		  public static void main(String[] args){
		    GpgWrapper.setGpgPath("/usr/bin");
		   try {
//		    GpgWrapper.noChangeSign(args[0],args[1],"RIPEMD160","2FD1061A","12345",false);
/*		    String[] recip = {"50399B82"};
	        GpgWrapper.signEncrypt(args[0],recip,args[1],"2FD1061A","12345");*/
		    Vector gpgPubKeys = GpgWrapper.getKeys(false);
		    Vector gpgSecKeys = GpgWrapper.getKeys(true);
		    System.out.println("Public keys:");
		    System.out.println("========================================================");
		    for (int keycount=0; keycount<gpgPubKeys.size();keycount++) {
		      GpgKeyInfo gpgKey = (GpgKeyInfo)(gpgPubKeys.get(keycount));
		      //System.out.println("Key: "+gpgKey.toString());
		      System.out.println(gpgKey.getKeyType()+" key from "+gpgKey.getUserID()+" Created: "+gpgKey.getCreationDate()+" Expired: "+gpgKey.getExpirationDate());
		      System.out.println("Algo: "+gpgKey.getAlgorithm(true)+" Size: "+gpgKey.getKeySize());
		      System.out.println(gpgKey.getTrust(true));
		      System.out.println("Fingerprint: "+gpgKey.getFingerPrint());
		      System.out.println("--------------------------------------------------------");
		    }

		    System.out.println("\nSecret keys:");
		    System.out.println("========================================================");
		    for (int keycount=0; keycount<gpgSecKeys.size();keycount++) {
		      GpgKeyInfo gpgKey = (GpgKeyInfo)(gpgSecKeys.get(keycount));
		      System.out.println("Key from "+gpgKey.getUserID()+" Created: "+gpgKey.getCreationDate()+" Expired: "+gpgKey.getExpirationDate());
		      System.out.println("Algo: "+gpgKey.getAlgorithm(true)+" Size: "+gpgKey.getKeySize());
		      System.out.println(gpgKey.getTrust(true));
		      System.out.println("Fingerprint: "+gpgKey.getFingerPrint());
		      System.out.println("--------------------------------------------------------");
		    }
		    //mytst.deleteKey("6124687E",true);
		   }catch (Exception e) {
		      System.out.println("Exception: "+e);
		      e.printStackTrace();
		   }
		  }// end main
}
