/* Created: 12.31.03 18:11:25
* Modifies 16.02.06 getKeyID
**
Set of functions for using GnuPG from java programms.
Supported some key menegment functons and encrypt/decrypt.
In case of errors wrapperException will be thrown
**
*/

package dbridge;

import java.io.*;
import java.util.Vector;

public class GpgWrapper {

  public static void setGpgPath(String gpgPath) {
    Init(gpgPath);
  }

  public static String getGpgPath() {
    return gpgPath;
  }

 static void Init(String inGpgPath){
  if (inGpgPath.length() != 0) {
    if (inGpgPath.charAt(inGpgPath.length()-1) != File.separatorChar)
      gpgPath=inGpgPath + File.separator;
    else gpgPath=inGpgPath;
  } else {
    gpgPath=new String();
  }
 }

/** Export key
  Input:
    String keyID: id of key (it is good idea to use key fingerprint as id)
    String outFileName: file name where to store exported key
    boolean isSecret: which key should be exported (true - for secret key)
    boolean needArmor: if true key will be exported in ASCII else in binary format
*/
  public static void exportKey(String keyID, String outFileName,
                       boolean isSecret, boolean needArmor )
                       throws wrapperException {
    boolean errorFlag=false;
//--------------New command generation--------------------
    Vector cmdVec=new Vector(8,1);
    cmdVec.add(gpgPath+"gpg");
    cmdVec.add("--batch");
    cmdVec.add("--no-tty");
    cmdVec.add("--yes");
    if (needArmor) {
      //create ACII Armored file
      cmdVec.add("--armor");
    }
    cmdVec.add("--output");
    cmdVec.add(outFileName);
    if (!isSecret) {
      cmdVec.add("--export");
    } else {
      cmdVec.add("--export-secret-keys");
    }
    cmdVec.add(keyID);
    String[] command=getCommand(cmdVec);
//----------------------------------------------------------

    Runtime currentRT = Runtime.getRuntime();
    try {
      Process gpgProc = currentRT.exec(command);
      if (gpgProc.waitFor()!=0) errorFlag=true;
    } catch (Exception e) {
      throw new wrapperException("ExportKey Exeception: "+e.getMessage());
    }
    if (errorFlag) {
      throw new wrapperException("Error when running GnuPG.\n Command " + getCommandString(command));
    }
  } //end exportKey()

/** Import key
  Input:
    String keyFileName: file name where key is stored
*/
  public static void importKey(String keyFileName)
              throws wrapperException{
    boolean errorFlag=false;
    String[] command= {gpgPath+"gpg", "--batch", "--no-tty", "--status-fd", "1", "--import", keyFileName};

    Runtime currentRT = Runtime.getRuntime();
    try {
      Process gpgProc = currentRT.exec(command);
// ::::::::::::::Output streams::::::::::::::::
//TODO: need parser for status-fd
        BufferedReader in  = new BufferedReader(new InputStreamReader(gpgProc.getInputStream()));
        CStreamReader inputReader = new CStreamReader("ImportKeyOutput",in);
        new Thread(inputReader).start();
//::::::::::::::::::::::::::::::::::::::::
      if (gpgProc.waitFor()!=0) errorFlag=true;
    } catch (Exception e) {
      throw new wrapperException("ImportKey Exeception: "+e.getMessage());
    }
    if (errorFlag) {
      throw new wrapperException("Error when running GnuPG.\n Command "+getCommandString(command));
    }
  } //end importKey()

/** Delete key
  Input:
    String keyID: id of key (it is good idea to use key fingerprint as id
                  for secret key you MUST use fingerprint as id)
    boolean isSecret: which key should be exported (true - for secret key)
*/
  public static void deleteKey(String keyID, boolean isSecret)
              throws wrapperException {
    boolean errorFlag=false;
//--------------New command generation--------------------
    String[] command={gpgPath+"gpg", "--batch", "--no-tty", "--status-fd", "1", "--delete-key", keyID};
    if (isSecret) {
      command[5]="--delete-secret-key";
    }
//--------------------------------------------------------

    Runtime currentRT = Runtime.getRuntime();
    try {
      Process gpgProc = currentRT.exec(command);
// ::::::::::::::Output streams::::::::::::::::
//TODO: need parser for status-fd
        BufferedReader in  = new BufferedReader(new InputStreamReader(gpgProc.getInputStream()));
        CStreamReader inputReader = new CStreamReader("DeleteKeyOutput",in);
        new Thread(inputReader).start();
//:::::::::::::::::::::::::::::::::::::::::::::
      if (gpgProc.waitFor()!=0) errorFlag=true;
    } catch (Exception e) {
      throw new wrapperException("DeleteKey Exeception: "+e.getMessage());
    }
    if (errorFlag) {
      throw new wrapperException("Error when running GnuPG.\n Command "+ getCommandString(command));
    }
  } //end deleteKey()

/** Delete public and secret key
  Input:
    String keyID: id of key. The key MUST be specified by fingerprint.
*/
  public static void deleteUser(String keyID)
              throws wrapperException {
    boolean errorFlag=false;
//--------------New command generation--------------------
    String[] command={gpgPath+"gpg", "--batch", "--no-tty", "--status-fd", "1", "--delete-secret-and-public-key", keyID};
//--------------------------------------------------------

    Runtime currentRT = Runtime.getRuntime();
    try {
      Process gpgProc = currentRT.exec(command);
// ::::::::::::::Output streams::::::::::::::::
//need parser for status-fd
        BufferedReader in  = new BufferedReader(new InputStreamReader(gpgProc.getInputStream()));
        CStreamReader inputReader = new CStreamReader("DeleteUserOutput",in);
        new Thread(inputReader).start();
//::::::::::::::::::::::::::::::::::::::::
      if (gpgProc.waitFor()!=0) errorFlag=true;
    } catch (Exception e) {
      throw new wrapperException("DeleteUser Exeception: "+e.getMessage());
    }
    if (errorFlag) {
      throw new wrapperException("Error when running GnuPG.\nCommand "+getCommandString(command));
    }
  } //end deleteUser()

/** Sign key
  Input:
    String keyID: id of keyid of key (it is good idea to use key fingerprint as id).
    int checkLevel: the check level when signing a key (see gpg manual for details).
    String pass: secret key protection passphrase
*/
  public static void signKey(String secKeyID, String keyID, int checkLevel, String pass)
              throws wrapperException {
    boolean errorFlag=false;
//--------------New command generation--------------------
    Vector cmdVec=new Vector(18,2);
    cmdVec.add(gpgPath+"gpg");
    cmdVec.add("--batch");
    cmdVec.add("--no-tty");
    cmdVec.add("--default-cert-check-level");
    cmdVec.add((Integer.toString(checkLevel)));
    if (secKeyID.length()>0) {
      cmdVec.add("-u");
      cmdVec.add(secKeyID);
    }
    cmdVec.add("--status-fd");
    cmdVec.add("1");
    cmdVec.add("--passphrase-fd");
    cmdVec.add("0");
    cmdVec.add("--yes");
    cmdVec.add("--edit-key");
    cmdVec.add(keyID);
    cmdVec.add("sign");
    cmdVec.add("save");
    cmdVec.add("--default-cert-check-level");
    cmdVec.add("0");
    String[] command=getCommand(cmdVec);
//--------------------------------------------------------

    Runtime currentRT = Runtime.getRuntime();
    try {
      Process gpgProc = currentRT.exec(command);

      PrintWriter writer = new PrintWriter(new OutputStreamWriter(gpgProc.getOutputStream()),true);
	    writer.println(pass);
	    writer.flush();
	    writer.close();

// ::::::::::::::Output streams::::::::::::::::
//TODO: need parser for status-fd
        BufferedReader in  = new BufferedReader(new InputStreamReader(gpgProc.getInputStream()));
        CStreamReader inputReader = new CStreamReader("signKeyOutput",in);
        new Thread(inputReader).start();
//::::::::::::::::::::::::::::::::::::::::
      if (gpgProc.waitFor()!=0) errorFlag=true;
    } catch (Exception e) {
      throw new wrapperException("SignKey Exeception: "+e.getMessage());
    }
    if (errorFlag) {
      throw new wrapperException("Error when running GnuPG.\nCommand "+getCommandString(command));
    }
  }//end signKey

/** Set ownertrust for key
  Input:
    String keyID: id of keyid of key (it is good idea to use key fingerprint as id).
    int trustLevel: the trust level for key owner (see gpg manual for details).
*/
  public static void setOwnertrust(String keyID, int trustLevel)
              throws wrapperException {
    boolean errorFlag=false;
//--------------New command generation--------------------
    String[] command={gpgPath+"gpg", "--batch", "--no-tty", "--status-fd", "1",
                    "--command-fd", "0", "--yes", "--edit-key", keyID, "trust"};
//--------------------------------------------------------

    Runtime currentRT = Runtime.getRuntime();
    try {
      Process gpgProc = currentRT.exec(command);

      PrintWriter writer = new PrintWriter(new OutputStreamWriter(gpgProc.getOutputStream()),true);
	    writer.println(trustLevel);
	    writer.flush();
	    writer.close();

// ::::::::::::::Output streams::::::::::::::::
//TODO: need parser for status-fd
        BufferedReader in  = new BufferedReader(new InputStreamReader(gpgProc.getInputStream()));
        CStreamReader inputReader = new CStreamReader("setOwnertrustOutput",in);
        new Thread(inputReader).start();
//::::::::::::::::::::::::::::::::::::::::
      if (gpgProc.waitFor()!=0) errorFlag=true;
    } catch (Exception e) {
      throw new wrapperException("SetOwnertrust Exeception: "+e.getMessage());
    }
    if (errorFlag) {
      throw new wrapperException("Error when running GnuPG.\nCommand "+getCommandString(command));
    }
  }//end setOwnertrust

/** Encryption. Multiple recipient
    Encrypt file for several (1 and more) recipients.
   Input:
      String inFileName: file for encryption.
      String[] recipients: array with recipients id's (it is good idea to use key fingerprint as id).
      String outFileName: file name for encrypted information
*/
  public static void encrypt(ByteArrayOutputStream osDataStream, String[] recipients, String outFileName)
              throws wrapperException {
    boolean errorFlag=false;
//--------------New command generation--------------------
    Vector cmdVec=new Vector(13);
    cmdVec.add(gpgPath+"gpg");
    cmdVec.add("--batch");
    cmdVec.add("--no-tty");
    cmdVec.add("--yes");
    cmdVec.add("--status-fd");
    cmdVec.add("1");
    cmdVec.add("--armor");
    cmdVec.add("--trust-model");
    cmdVec.add("always");
    cmdVec.add("--output");
    cmdVec.add(outFileName);
    cmdVec.add("--encrypt");
    for(int recCount=0;recCount<recipients.length;recCount++) {
      cmdVec.add("--recipient");
      cmdVec.add(recipients[recCount]);
    }

    //cmdVec.add(inFileName);
    String[] command=getCommand(cmdVec);
//--------------------------------------------------------
      Runtime currentRT = Runtime.getRuntime();
    try{
      Process gpgProc = currentRT.exec(command);
//-----------------------------------------------------------
      PrintWriter writer = new PrintWriter(new OutputStreamWriter(gpgProc.getOutputStream()),true);
      osDataStream.writeTo(gpgProc.getOutputStream());
      writer.close();
//-----------------------------------------------------------      
// ::::::::::::::Output streams::::::::::::::::
        BufferedReader in  = new BufferedReader(new InputStreamReader(gpgProc.getInputStream()));
        CStreamReader inputReader = new CStreamReader("EncryptOutput",in);
        new Thread(inputReader).start();
//::::::::::::::::::::::::::::::::::::::::
      if (gpgProc.waitFor()!=0) errorFlag=true;
    } catch(Exception e){
        throw new wrapperException("Encrypt Exeception: "+e.getMessage());
    }
    if (errorFlag) {
      throw new wrapperException("Error when running GnuPG.\nCommand "+getCommandString(command));
    }
  } //end encrypt()

 /**
  * 
  * @param inFileName - name of file with encrypted message
  * @return Vector of GpgKeyInfo
  * @throws wrapperException
  * 
  * The message in inFileName is encrypted to returned keys 
  */ 
  public static Vector getEncryptedTo (String inFileName) throws wrapperException{
//  --------------New command generation--------------------
    String[] command={gpgPath+"gpg", "--batch", "--no-tty", "--yes", "--status-fd",
                     "1", "--decrypt", "--list-only", inFileName};
//  --------------------------------------------------------
    boolean errorFlag=false;
    BufferedReader in;
    String currentLine;
    Vector keyIDs = new Vector();
    Vector results = new Vector();
    
    Runtime currentRT = Runtime.getRuntime();
    try {
      Process gpgProc=currentRT.exec(command);
      in = new BufferedReader(new InputStreamReader(gpgProc.getInputStream()));
      while ((currentLine=in.readLine())!=null) {
        if (currentLine.contains("ENC_TO")) {
          String keyID = (currentLine.split("\\s"))[2];
          keyIDs.add(keyID);
        }
      }
      if (gpgProc.waitFor()!=0) errorFlag=true;
      else {
        for (int keyIndex=0; keyIndex<keyIDs.size(); ++keyIndex){
          String [] fprCommand = {gpgPath+"gpg", "--batch", "--no-tty", "--fingerprint", 
                               "--with-colon",(String)keyIDs.elementAt(keyIndex)};
          Process gpg1Proc=currentRT.exec(fprCommand);
          in = new BufferedReader(new InputStreamReader(gpg1Proc.getInputStream()));
          while ((currentLine=in.readLine())!=null) {
            if (currentLine.startsWith("pub")) {
              GpgKeyInfo gpgKey=new GpgKeyInfo(currentLine);
              currentLine=in.readLine();
              gpgKey.setFingerPrint(currentLine);
              results.add(gpgKey);
            }   
          }
          if (gpg1Proc.waitFor()!=0) errorFlag=true;
        }
      }    
    }  catch(Exception e){
        throw new wrapperException("getKeys Exception: "+e.getMessage());
    }
    if (errorFlag) {
      throw new wrapperException("Error when running GnuPG.\nCommand "+getCommandString(command));
    }
    return results;
  }//end of getEncryptedTo()
  
  
  /**
  Decryption:
   Input:
      String inFileName: file for decryption.
      String outFileName: file name for decrypted information
      String pass: secret key protection passphrase
  */
  public static void decrypt(String inFileName, String outFileName, String passPhrase)
              throws wrapperException,badSignatureException {
     boolean errorFlag=false;
     int gpgExitStatus=0;
//--------------New command generation--------------------
   String[] command={gpgPath+"gpg", "--batch", "--no-tty", "--yes", "--status-fd",
                    "1", "--passphrase-fd", "0", "--output", outFileName,
                    "--decrypt", inFileName};
//--------------------------------------------------------

     Runtime currentRT = Runtime.getRuntime();
     try{
    	Process gpgProc=currentRT.exec(command);
    	PrintWriter writer = new PrintWriter(new OutputStreamWriter(gpgProc.getOutputStream()),true);
    	writer.println(passPhrase);
    	writer.close();
// ::::::::::::::Output streams::::::::::::::::
          BufferedReader in  = new BufferedReader(new InputStreamReader(gpgProc.getInputStream()));
          CStreamReader inputReader = new CStreamReader("DecryptOutput",in);
          new Thread(inputReader).start();
//::::::::::::::::::::::::::::::::::::::::
      if ((gpgExitStatus = gpgProc.waitFor())!=0) errorFlag=true;
     }catch(Exception ex) {
       throw new wrapperException("Decrypt Exception: "+ex.getMessage());
    }
    if (errorFlag) {
      if (gpgExitStatus==1)
        throw new badSignatureException("File "+inFileName+" has a bad Signature!");
      else 
        throw new wrapperException("Error when running GnuPG.\nCommand "+getCommandString(command));
    }
  }//end decrypt()

//gpg --batch --no-tty --yes --status-fd 1 --passphrase-fd 0 --armor
//    --digest-algo RIPEMD160 --default-key 50399B82
//    --output tst.signed --sign test.txt
//gpg --batch --no-tty --yes --status-fd 1 --passphrase-fd 0 --armor
//    --digest-algo RIPEMD160 --default-key 50399B82 --recipient 50B82
//    --output tst.signed.asc --sign --encrypt test.txt

  /**
  Sign without changing input information (--clearsign or --detach-sign)
   Parameters:
          ByteArrayOutputStream osDataStream: DataStream with data for signing;
	  String outFileName: file to store results:
	     information from input file + signature in case of --clearsign or
	     signature only in case of --detach-sign.
	  String digestAlgo: name of HASH function for signature generation (currenly not in use)
	  String signer: keyid for signature generation.
	  String passPhrase: secret key protection passphrase.
	  boolean isDetached: create detached signature if TRUE.
  */
  public static void noChangeSign(ByteArrayOutputStream osDataStream, String outFileName,
           /*String digestAlgo,*/ String signer, String passPhrase, boolean isDetached)
           throws wrapperException {
    boolean errorFlag=false;

    Vector cmdVec=new Vector(18);
    cmdVec.add(gpgPath+"gpg");
    cmdVec.add("--batch");
    cmdVec.add("--no-tty");
    cmdVec.add("--yes");
    cmdVec.add("--status-fd");
    cmdVec.add("1");
    cmdVec.add("--passphrase-fd");
    cmdVec.add("0");
    cmdVec.add("--armor");
    //cmdVec.add("--digest-algo");
    //cmdVec.add(digestAlgo);
    cmdVec.add("--default-key");
    cmdVec.add(signer);
    cmdVec.add("--output");
    cmdVec.add(outFileName);
    if (isDetached) {
      cmdVec.add("--detach-sign");
    } else {
      cmdVec.add("--clearsign");
    }
//cmdVec.add(inFileName);
    String[] command=getCommand(cmdVec);
    Runtime currentRT = Runtime.getRuntime();
    try{
      Process gpgProc=currentRT.exec(command);
      PrintWriter writer = new PrintWriter(new OutputStreamWriter(gpgProc.getOutputStream()),true);
      writer.println(passPhrase);
      //-----------------------------------------------------------
      osDataStream.writeTo(gpgProc.getOutputStream());
      //writer.println();
      //-----------------------------------------------------------
      writer.close();
      //::::::::::::::Output streams::::::::::::::::
      BufferedReader in  = new BufferedReader(new InputStreamReader(gpgProc.getInputStream()));
      CStreamReader inputReader = new CStreamReader("SignOutput",in);
      new Thread(inputReader).start();
      //::::::::::::::::::::::::::::::::::::::::
      if (gpgProc.waitFor()!=0) errorFlag=true;
    } catch(Exception ex) {
      throw new wrapperException("Sign Exception: "+ex.getMessage());
    }
    if (errorFlag) {
      throw new wrapperException("Error when running GnuPG.\nCommand "+getCommandString(command));
    }
  }//end of noChangeSign

  /**
  Sign and encrypt
   Parameters:
    ByteArrayOutputStream osDataStream: Data to be signed.
    String[] recipients: list of recipeients (public keys for file encryption)
	  String outFileName: file to store results;
	  String digestAlgo: name of HASH function for signature generation (currenly not in use)
	  String signer: keyid for signature generation.
	  String passPhrase: secret key protection passphrase.
  */
  public static void signEncrypt(ByteArrayOutputStream osDataStream, String[] recipients, String outFileName,
                                /*String digestAlgo,*/ String signer, String passPhrase)
                     throws wrapperException {
    boolean errorFlag=false;

    Vector cmdVec=new Vector(18);
    cmdVec.add(gpgPath+"gpg");
    cmdVec.add("--batch");
    cmdVec.add("--no-tty");
    cmdVec.add("--yes");
    cmdVec.add("--status-fd");
    cmdVec.add("1");
    cmdVec.add("--passphrase-fd");
    cmdVec.add("0");
    cmdVec.add("--armor");
    cmdVec.add("--trust-model");
    cmdVec.add("always");   
    //cmdVec.add("--digest-algo");
    //cmdVec.add(digestAlgo);
    cmdVec.add("--default-key");
    cmdVec.add(signer);
    cmdVec.add("--output");
    cmdVec.add(outFileName);
    for(int recCount=0;recCount<recipients.length;recCount++) {
      cmdVec.add("--recipient");
      cmdVec.add(recipients[recCount]);
    }
    cmdVec.add("--sign");
    cmdVec.add("--encrypt");
    //cmdVec.add(inFileName);
    String[] command=getCommand(cmdVec);

    Runtime currentRT = Runtime.getRuntime();
    try{
      Process gpgProc=currentRT.exec(command);
      PrintWriter writer = new PrintWriter(new OutputStreamWriter(gpgProc.getOutputStream()),true);
      writer.println(passPhrase);
      osDataStream.writeTo(gpgProc.getOutputStream());
      writer.close();
      //::::::::::::::Output streams::::::::::::::::
      BufferedReader in  = new BufferedReader(new InputStreamReader(gpgProc.getInputStream()));
      CStreamReader inputReader = new CStreamReader("SignEncryptOutput",in);
      new Thread(inputReader).start();
      //::::::::::::::::::::::::::::::::::::::::
      if (gpgProc.waitFor()!=0) errorFlag=true;
    }catch(Exception ex) {
      throw new wrapperException("Sign and Encrypt Exception: "+ex.getMessage());
    }
    if (errorFlag) {
      throw new wrapperException("Error when running GnuPG.\nCommand "+getCommandString(command));
    }
  }//end of signEncrypt

  /**
   * 
   * @param signatureFileName - name of file with detached signature
   * @param signedFileName - name of signed file
   * @return true if signature is good.
   * @throws wrapperException
   */
  public static boolean isDetachedSignatureGood(String signatureFileName, String signedFileName) throws wrapperException {
    boolean errorFlag=false;
    int gpgExitStatus=0;
     Vector cmdVec=new Vector(10);
     cmdVec.add(gpgPath+"gpg");
     cmdVec.add("--batch");
     cmdVec.add("--no-tty");
     cmdVec.add("--yes");
     cmdVec.add("--status-fd");
     cmdVec.add("1");
     cmdVec.add("--verify");
     cmdVec.add(signatureFileName);
     cmdVec.add(signedFileName);
     String[] command=getCommand(cmdVec);
//--------------------------------------------------------
      Runtime currentRT = Runtime.getRuntime();
    try{
      Process gpgProc = currentRT.exec(command);
// ::::::::::::::Output streams::::::::::::::::
        BufferedReader in  = new BufferedReader(new InputStreamReader(gpgProc.getInputStream()));
        CStreamReader inputReader = new CStreamReader("VerifyOut",in);
        new Thread(inputReader).start();
//::::::::::::::::::::::::::::::::::::::::
      if ((gpgExitStatus=gpgProc.waitFor())!=0) {
         errorFlag=true;
       }
    } catch(Exception e){
        throw new wrapperException("Verify Exeception: "+e.getMessage());
    }
    if (errorFlag) {
      if (gpgExitStatus==1) {
         return false;
      } else {
       throw new wrapperException("Error when running GnuPG.\nCommand "+getCommandString(command));
      }
    }
    return !errorFlag;
  }//end of isDetachedSignatureGood();
  
/**
   GetKeys:
   	Returns information about available public and secret (if isSecret=true) keys
   	in vector of GpgKeyInfo.
*/
  public static Vector getKeys(boolean isSecret)
                throws wrapperException {
    boolean errorFlag=false;
    BufferedReader in;
    String currentLine;
    String [] command = {gpgPath+"gpg", "--batch", "--no-tty", "--fingerprint", "--with-colon","--list-key"  };
    Vector results = new Vector();
    String keyStart="pub";
    if (isSecret) {
      command[5] = "--list-secret-key";
      keyStart="sec";
    }

    Runtime currentRT = Runtime.getRuntime();
    try {
      Process gpgProc=currentRT.exec(command);
      in = new BufferedReader(new InputStreamReader(gpgProc.getInputStream()));
	    while ((currentLine=in.readLine())!=null) {
        if (currentLine.startsWith(keyStart)) {
          GpgKeyInfo gpgKey=new GpgKeyInfo(currentLine);
          currentLine=in.readLine();
          gpgKey.setFingerPrint(currentLine);
          results.add(gpgKey);
        }
	    }
	    if (gpgProc.waitFor()!=0) errorFlag=true;
    }  catch(Exception e){
        throw new wrapperException("getKeys Exception: "+e.getMessage());
    }
    if (errorFlag) {
      throw new wrapperException("Error when running GnuPG.\nCommand "+getCommandString(command));
    }

    return results;
  }//end getKeys()

 /////////////////////////////Key generation\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
/** Generate key helpers functions*/

/**
	Helper functoin for key pair generation.
	Defines the expiration date for the key (and the subkey).  It
	may be entered as	number of days, weeks, month or years.
*/
  private static String getExpDate	(int expTime, char timeSlice)
             throws wrapperException {
    String result = "Expire-Date: ";
    if ((timeSlice!='d')&&(timeSlice!='w')&&(timeSlice!='m')&&(timeSlice!='y'))
		  throw new wrapperException("Invalid time slice for expiration date.");
    if (expTime==0) {
      result+=0;
    } else {
      result+=expTime;
      result+=timeSlice;
    }
    return result;
  }

/**
	Helper functoin for key pair generation.
	Defines type and size of generated kei pair.
	type:
	    0:  Generate DSA key for signing and ElGamal subkey for encryption
	          keySize - length of ElGamal subKey in bits (768 is smallest value allowed).
	          DSA keypair will have 1024 bits.

	    1: Generate DSA key pair for sign only.
	         keysize - length of the key in bits (can be in between 768 and 1024)

	    2: Generate RSA key pair for sign only.
	         keysize - length of the key in bits (1024 is smallest value allowed).
*/
  private static String getKeyTypeAndLength(int type, int keySize)
                       throws wrapperException{

    String keyType="Key-Type: DSA";
    String subType=new String();
    String keyLength="Key-Length: ";
    String subLength=new String();
    String result=new String();

    switch (type) {
	   	case 0:
        subType="Subkey-Type: ELG-E";
        if (keySize<768) {
          throw new wrapperException("768 is smallest value allowed.");
        }
//         else if (keySize>2048) {
//          String tmpStr="Keysizes larger than 2048 are not suggested because \ncomputations take REALLY long!\nAre you sure that you want this keysize?";
//          if(JOptionPane.showConfirmDialog(this,tmpStr,"Key too long",JOptionPane.YES_NO_OPTION)==1)
//        	throw new genDialogException("Key too long");
//        }
        keyLength+=1024;
        subLength="Subkey-Length: "+keySize;
        result = keyType+"\n"+keyLength +"\n" + subType + "\n" + subLength;
        break;
      case 1:
        if ((keySize<768) || (keySize>1024)) {
        	  throw new wrapperException("DSA only allows keysizes from 768 to 1024");
        }
        keyLength+=keySize;
        result= keyType+"\n"+keyLength;
        break;
      case 2:
        keyType= "Key-Type: RSA";
        if (keySize<1024) {
          throw new wrapperException("1024 is smallest value allowed for RSA.");
        }
        keyLength+=keySize;
        result= keyType+"\n"+keyLength;
        break;
      default:
        throw new wrapperException("Invalid type of key");
    }
    return result;
  }

/** Generate  a  new  key pair.
    You should provide following information:
	keyType:
	    0:  Generate DSA key for signing and ElGamal subkey for encryption
	          keySize - length of ElGamal subKey in bits (768 is smallest value allowed).
	          DSA keypair will have 1024 bits.

	    1: Generate DSA key pair for sign only.
	         keysize - length of the key in bits (can be in between 768 and 1024)

	    2: Generate RSA key pair for sign only.
	         keysize - length of the key in bits (1024 is smallest value allowed).

	userName: name of key owner
	userMail: e-mail of key owner
	comments: comments about key owner
	expTime and timeSlice: when key will be expared (number of days, weeks, month or years).
	pass: passphrase for secret key protection.

*/
    public static void generateKey(int keysType, int keySize,
                           String userName, String comments, String userMail,
                           int expTime, int timeSlice,//String timeSlice,
                           String pass) throws wrapperException {

    boolean errorFlag=false;
    String command[] = {gpgPath + "gpg", "--quiet", "--batch", "--no-tty", "--status-fd", "1", "--gen-key"};
    char timeSliceChar='0';
    try {
     	String keys=getKeyTypeAndLength(keysType,keySize);

     	String name = new String();
      if (userName.length()>4) {
        name="Name-Real: "+userName;
	    } else {
		    throw new wrapperException("Name must be at least 5 characters long");
      }

      String comment  = new String();
      if (comments.length()!=0) {
      	comment  = "Name-Comment: "+comments;
     	}

      String mail     = new String();
      int atPos=userMail.indexOf("@");
      if ((atPos!=-1)&&(atPos!=0)&&(atPos!=userMail.length()-1)) {
	     mail = "Name-Email: "+userMail;
      } else {
	     throw new wrapperException("Not a valid email address");
      }

      switch(timeSlice) {
      case 0:
        timeSliceChar = 'd';
        break;
      case 1:
        timeSliceChar = 'w';
        break;
      case 2:
        timeSliceChar = 'm';
        break;
      case 3:
        timeSliceChar = 'y';
        break;
      default:
        timeSliceChar = '0';
        break;
      }
      String expdate  =getExpDate(expTime,timeSliceChar);//.charAt(0));

      String passStr = new String();
      if (pass.length()!=0) {
        passStr ="Passphrase: "+pass;
      }

      Process gpgProc=Runtime.getRuntime().exec(command);

 	PrintWriter writer = new PrintWriter(new OutputStreamWriter(gpgProc.getOutputStream()),true);
// ::::::::::::::Output streams::::::::::::::::
            BufferedReader in  = new BufferedReader(new InputStreamReader(gpgProc.getInputStream()));
            CStreamReader inputReader = new CStreamReader("GenerateKeyOutput",in);
            new Thread(inputReader).start();
//::::::::::::::::::::::::::::::::::::::::

	    writer.println(keys);
	    writer.println(name);
	    if (comment.length()!=0) {
			writer.println(comment);
	    }
	    writer.println(mail);
	    writer.println(expdate);
           if (passStr.length()!=0) {
		   	writer.println(passStr);
           }
	    writer.println("%commit");

	    writer.flush();
	    writer.close();
      if (gpgProc.waitFor()!=0) errorFlag=true;

  	} catch (Exception e){
        throw new wrapperException("Key generation error: "+e.getMessage());
    }
    if (errorFlag) {
      throw new wrapperException("Error when running GnuPG.\nCommand "+getCommandString(command));
    }
 }
/////////////////////////End of key generation\\\\\\\\\\\\\\\\\\\\\\\\\\\\

  private static String[] getCommand(Vector cmdVec) {
   String[] command = new String[cmdVec.size()];
   for (int argCount=0;argCount<command.length;argCount++) {
     command[argCount]=(String)cmdVec.get(argCount);
   }
   return command;
  }

/*  private static String getCommandString(Vector cmdVec) {
    String command = "";//new String;
    for (int argCount=0;argCount<cmdVec.size();argCount++) {
      command+=(String)cmdVec.get(argCount);
      command+=" ";
    }
    return command;
   }
*/
   private static String getCommandString(String[] cmdStrings) {
    String command = "";//new String;
    for (int argCount=0;argCount<cmdStrings.length;argCount++) {
      command+=cmdStrings[argCount];
      command+=" ";
    }
    return command;
   }



  private static String gpgPath;
}

class wrapperException extends Exception{
	public wrapperException(String errStr) {
		super(errStr);
	}
}

class badSignatureException extends Exception{
  public badSignatureException(String errStr) {
    super(errStr);
  }
}
