package dbridge;

import java.util.*;
import java.text.*;
import java.io.*;

import com.pixelmed.dicom.*;

/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: </p>
 * @author unascribed
 * @version 1.0
 */

/*
 * Created: 10.12.03
 * Modified 31.12.03
 * Modified 25.01.04
 * Modified 6.03.04
 */
public class Logger
{
    public Logger(Integer level, String logDir)
    {
        Init(level, logDir, "");
    }

    public Logger(Integer level, String logDir, String prefix_name)
    {
        Init(level, logDir, prefix_name);
    }

    private void Init(Integer level, String logDir, String prefix_name)
    {
        mLevel = level;
        mLogDir = logDir;
        mPrefixName = prefix_name;
//System.out.println("Init logger, preffic is "+mPrefixName);
        mTypeStrings = new HashMap();
        mTypeStrings.put(State.ERROR, new String("Error"));
        mTypeStrings.put(State.WARNING, new String("Warning"));
        mTypeStrings.put(State.INFO, new String("Info"));
        mTypeStrings.put(State.DEBUG, new String("DEBUG"));
        mLocked = false;
    }

    public void setLevel(Integer level)
    {
        mLevel = level;
    }

    public Integer getLevel()
    {
        return mLevel;
    }

    public void printMsg(final CLogMsg msg)
    {
        if (msg.Type.compareTo(mLevel)<0)
        {
            return;
        }
        
        int count=0;
        // synchronization delay
        while ( mLocked ){
        	count ++;
        	if ( count > 50000 ) mLocked=false;
        }

        String sFileName = mLogDir + mPrefixName;

        sFileName += "current.log";
        
        // if flie exists, check its size
        File logfile = new File(sFileName);
		if (logfile.exists() && logfile.length() > 1024*1024*2) {
System.out.println(sFileName+" has length "+logfile.length()+" and it will be renamed");
			// if size > 2 Mb, copy to archive
			// File (or directory) with new name
			Calendar rightNow = Calendar.getInstance();
			int dayOfMonth = rightNow.get(Calendar.DAY_OF_MONTH);
			int month = rightNow.get(Calendar.MONTH) + 1;
			int year = rightNow.get(Calendar.YEAR);
			int hour = rightNow.get(Calendar.HOUR_OF_DAY);
			int minute = rightNow.get(Calendar.MINUTE);
			String newname=mLogDir + mPrefixName+"current" + dayOfMonth + "-" + month + "-"
			+ year + "-" + hour + "-" + minute + ".log";
			File file2 = new File(newname);
			//System.out.println("New Name "+file2.getName());
			// Rename file (or directory)
			if ( !file2.exists() ){
				boolean success = logfile.renameTo(file2);
				if (!success) {
					// error message
					System.out.println("Cannot rename log to "+file2.getName());
				}
			}// end rename
		}

        // String sTime = (new Date()).toString();
        try
        {
            BufferedWriter out = new BufferedWriter(new OutputStreamWriter(new
                    FileOutputStream(sFileName, true)));
            DateFormat df = new SimpleDateFormat();
            out.write("<--msg-->;\"" + df.format(new Date()) +
                      "\";" + msg.Type + ";" + mTypeStrings.get(msg.Type) + ";" +
                      msg.Module + ";" + "[" + Thread.currentThread().getName() +
                      "];" +
                      msg.Text);
            out.newLine();
            out.close();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }
    public static void main(String args[])
    {
    }

    private Integer mLevel;
    private String mLogDir;
    private String mPrefixName;
    private boolean mLocked;

    private Map mTypeStrings;
}

/**
 * Objects of this class represent Logger before application defines its own
 * @author mcdi
 *
 */
class EmptyLogger extends Logger
{
	public EmptyLogger()
	{
		super(new Integer(0),"");
	}
    public void printMsg(final CLogMsg msg)
    {
        DateFormat df = new SimpleDateFormat();
        System.out.println("<Not_Initialised_Logger>;\"" + df.format(new Date()) +
                "\";" + msg.Type + ";" + msg.Text);
   }
};

class State
{
    public final static Integer ERROR = new Integer(3);
    public final static Integer WARNING = new Integer(2);
    public final static Integer INFO = new Integer(1);
    public final static Integer DEBUG = new Integer(0);
}


class CStateMonitor
{
    CStateMonitor()
    {
        mStateMap = new HashMap();
    }

    public void UpdateState(String sModule, Integer iState)
    {
        Integer updt = iState;

        Integer old = (Integer) mStateMap.get(sModule);
        if (old != null && old.intValue() >= iState.intValue())
        {
            updt = null;
        }

        if (updt != null)
        {
            mStateMap.put(sModule, updt);
        }
    }

    public void SetState(String sModule, Integer iState)
    {
        mStateMap.put(sModule, iState);
    }

    public Integer GetState(String sModule)
    {
        Integer ret = State.INFO;
        Integer old = (Integer) mStateMap.get(sModule);
        if (old != null)
        {
            ret = old;
        }

        return ret;
    }

    private Map mStateMap;
}


class CLogMsg
{
    public CLogMsg()
    {
        Init(DefaultType, DefaultModule, DefaultTxt);
    }

    public CLogMsg(String txt)
    {
        Init(DefaultType, DefaultModule, txt);
    }

    public CLogMsg(String module, String txt)
    {
        Init(DefaultType, module, txt);
    }

    public CLogMsg(Integer type, String module, String txt)
    {
        Init(type, module, txt);
    }

    private void Init(Integer type, String module, String txt)
    {
        Type = type;
        Module = module;
        Text = txt;
    }

    public CLogMsg SetText(String s)
    {
        Text = s;
        return this;
    }

    public CLogMsg SetType(Integer type)
    {
        Type = type;
        return this;
    }

    public CLogMsg SetModule(String mod)
    {
        Module = mod;
        return this;
    }

    public Integer Type;
    public String Module;
    public String Text;

    public static String DefaultModule = new String("common");
    public static String DefaultTxt = new String("");
    public static Integer DefaultType = State.INFO;
}


class Logs
{
    public final static String sBank = new String("Bank");
    public final static String sServices = new String("Services");
    public final static String sGUI = new String("GUI");

    public final static CLogMsg Bank = new CLogMsg(State.INFO, sBank, "");
    public final static CLogMsg Srvcs = new CLogMsg(State.INFO, sServices, "");
    public final static CLogMsg GUI = new CLogMsg(State.INFO, sGUI, "");
}


class CStatistician
{
    public CStatistician(String sFile)
    {
        mStatsFilePath = sFile;
    }

    public void SaveStatsForFile(String sDcmFile, String sTail)
    {
        try
        {
            //
            // 1. Open dicom file, read it to AttrList
            //
            AttributeList list = new AttributeList();
            list.read(sDcmFile, null, true, true);

            //
            // 2. Save stats for list
            //
            SaveStatsForAttrList(list, sTail);

        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    public void SaveStatsForAttrList(AttributeList list, String sTail)
    {
        try
        {
            Vector v = new Vector();
            //
            // 2. Read required data from dicom file
            //
            String sInfo;
            sInfo = Attribute.getSingleStringValueOrDefault(list,
                    TagFromName.StudyDate, "");
            v.add(sInfo);
            sInfo = Attribute.getSingleStringValueOrDefault(list,
                    TagFromName.Modality, "");
            v.add(sInfo);
            sInfo = Attribute.getSingleStringValueOrDefault(list,
                    TagFromName.PatientName, "");
            v.add(sInfo);
            sInfo = Attribute.getSingleStringValueOrDefault(list,
                    TagFromName.InstitutionName, "");
            v.add(sInfo);
            sInfo = Attribute.getSingleStringValueOrDefault(list,
                    TagFromName.ManufacturerModelName, "");
            v.add(sInfo);

            // TODO Add file size and Date
            v.add(sTail);
            //
            // 3. Prepare output string
            //
            String sFull = "";
            for (int i = 0; i < v.size(); i++)
            {
                String s = "\"" + v.get(i) + "\";";
                sFull += s;
            }
            //
            // 3. Save info in file
            //
            BufferedWriter out = new BufferedWriter(new OutputStreamWriter(new
                    FileOutputStream(mStatsFilePath, true)));
            DateFormat df = new SimpleDateFormat();
            out.write(sFull);
            out.newLine();
            out.close();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }


    private String mStatsFilePath;
}

/**
 * Objects of this class represent CStatistician before application defines its own
 * @author mcdi
 *
 */
class EmptyStatistician extends CStatistician
{
	public EmptyStatistician()
	{
		super("");
	}
    public void SaveStatsForFile(String sDcmFile, String sTail)
    {
    	System.out.println("Not_initialised_statistician: "+ sDcmFile+sTail);
    }
    public void SaveStatsForAttrList(AttributeList list, String sTail)
    {
    	System.out.println("Not_initialised_statistician: "+ list+sTail);
    }
}