/**
 *
 */
package dbridge;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.sql.Timestamp;
import java.util.Vector;

import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.Part;

import com.pixelmed.dicom.Attribute;
import com.pixelmed.dicom.AttributeList;
import com.pixelmed.dicom.TagFromName;


/**
 * @author McDi Modyfied Bob
 *
 */

enum HEADER_TYPE{
	HEADER_NO_DN,
	HEADER_RFC,
	HEADER_XSETID,
	HEADER_UNCKN
};

enum MESSAGE_TYPE{
	MES_ENCRYPTED,
	MES_SIGNED,
	MES_REPORT,
	MES_DEFAULT
};

/**
 * Mail Parser main class
 */
public class MailParser {

	  public MailParser(IParseController oContr) {
	    mController = oContr;
	  }

	  public void setParseController(IParseController oContr) {
	    mController = oContr;
	  }

	  //*************************************** PARSE MESSAGE *************************************
	   public void parse(Message message) throws Exception {

		   //BridgeApp.LogDebug(Logs.Srvcs.SetText(
           //"mController.Begin ..."));
		 mController.Begin();

		 //BridgeApp.LogDebug(Logs.Srvcs.SetText(
         //"DefineCtrlType ..."));
		 mController.DefineCtrlType(message);
		 //BridgeApp.LogDebug(Logs.Srvcs.SetText(
         //"get realController ..."));
		 IParseController realController = mController.getRealController();

		 //BridgeApp.LogDebug(Logs.Srvcs.SetText(
         //"parse additional headers ..."));
		 realController.ParseAdditionalHeaders(message);

		 //highest priority to sign check
		 if (message.isMimeType("multipart/signed")){
			 //BridgeApp.LogDebug(Logs.Srvcs.SetText(
	         //"IsSignedMultipartGood ..."));
			 realController.IsSignedMultipartGood((Multipart) message.getContent());
   		 }

		 try{

			 //BridgeApp.LogDebug(Logs.Srvcs.SetText(
	         //"Header analysing ..."));
			 //BridgeApp.LogDebug(Logs.Srvcs.SetText(
	         //"Type ="+realController.AnalyzeHeader(message)));
			 switch(realController.AnalyzeHeader(message)){
			 case HEADER_UNCKN:
				 //BridgeApp.LogDebug(Logs.Srvcs.SetText(
		         //"HEADER_UNCKN realController.NewUncknownHeader..."));
				 realController.NewUncknownHeader(message);
				 break;
			 case HEADER_RFC:
				 //BridgeApp.LogDebug(Logs.Srvcs.SetText(
		         //"HEADER_RFC realController.NewRFCHeader..."));
				 realController.NewRFCHeader(message);
				 break;
			 case HEADER_XSETID:
				 //BridgeApp.LogDebug(Logs.Srvcs.SetText(
		         //"HEADER_XSETID realController.NewXHeader..."));
				 realController.NewXHeader(message);
				 break;
			 }
			 //BridgeApp.LogDebug(Logs.Srvcs.SetText(
			 //"End analysing, if multipart/* ..."));
			 
			 if (message.isMimeType("multipart/*")) {
				 //BridgeApp.LogDebug(Logs.Srvcs.SetText(
				 //"multipart getContent ..."));
		    	 Multipart mp = (Multipart) message.getContent();

		    	 //BridgeApp.LogDebug(Logs.Srvcs.SetText(
		    	 //"if multipart/report ..."));
		    	 if (message.isMimeType("multipart/report")){
		    		 //BridgeApp.LogDebug(Logs.Srvcs.SetText(
		    		 //"report, end ..."));
		    		 realController.NewReport(mp.getBodyPart(1));
		    		 realController.End();
		    		 //BridgeApp.LogDebug(Logs.Srvcs.SetText(
		    		 //"return ..."));
			    	 return;
		         }

		    	 //BridgeApp.LogDebug(Logs.Srvcs.SetText(
		    	 //"For each multi part ..."));
		       int np = mp.getCount();
		       for (int j = 0; j < np; j++) {
		         Part part = mp.getBodyPart(j);

		         //BridgeApp.LogDebug(Logs.Srvcs.SetText(
		         //"multipart/encrypted? ..."));
		         if (message.isMimeType("multipart/encrypted")) {
		        	 //BridgeApp.LogDebug(Logs.Srvcs.SetText(
		        	 //"Yes! multipart/pgp-encrypted? ..."));
		           if (part.isMimeType("application/pgp-encrypted")) {
		        	   //BridgeApp.LogDebug(Logs.Srvcs.SetText(
		        	   //  "Yes, do nothing ..."));
		           }
		           else{
		        	   //BridgeApp.LogDebug(Logs.Srvcs.SetText(
		        	   //  "NO! application/octet-stream? ..."));
		        	   if (part.isMimeType("application/octet-stream")) {
		        		   //BridgeApp.LogDebug(Logs.Srvcs.SetText(
		        		   //  "Yes! NewEncryptedFile(part) ..."));
		        	       realController.NewEncryptedFile(part);
		        	   } else{
		        		   BridgeApp.LogDebug(Logs.Srvcs.SetText(
					         "Unknown mime part ..."));
		        	   }
		           }
		         }else {
		        	 //BridgeApp.LogDebug(Logs.Srvcs.SetText(
		        	 //"NO! Just parse as part ..."));
		           parsePart(part);
		         }
		       }
		     }
		     else {
		    	 //BridgeApp.LogDebug(Logs.Srvcs.SetText(
		    	 //"Else parse Part ..."));
		       parsePart(message);
		     }

			 //BridgeApp.LogDebug(Logs.Srvcs.SetText(
			 //"Parsing OK ..."));
		 }catch (BadSyntaxException e){
			 //do nothing
			 BridgeApp.LogError(Logs.Srvcs.SetText(
             "Bad syntax exception appeared during the message parsing..."));
		 }catch (badSignatureException e){
			 //do nothing
			 BridgeApp.LogError(Logs.Srvcs.SetText(
             "Bad signature exception appeared during the message parsing..."));
		 }catch (Exception e){
			 BridgeApp.LogError(Logs.Srvcs.SetText(
             "Unknown exception appeared during the message parsing..."));
			 throw e;
		 }


	     mController.End();
	   }




	  private void parsePart(Part part) throws Exception {

		IParseController realController = mController.getRealController();

       if (part.isMimeType("multipart/*")){
  			Multipart mp = (Multipart) part.getContent();
  	       int np = mp.getCount();
	       for (int j = 0; j < np; j++) {
	    	   Part _part = mp.getBodyPart(j);
	    	   parsePart(_part);
	       }
  		}else{
  		    if (part.isMimeType("text/plain")) {
  		      BridgeApp.LogInfo(Logs.Srvcs.SetText("Text saving"));
  		    realController.NewTxtFile(part);
  		    }
  		    else if (part.isMimeType("application/dicom")) {
  		      BridgeApp.LogInfo(Logs.Srvcs.SetText("Dicom saving"));
  		    realController.NewDicomFile(part);
  		    }else if (part.isMimeType("application/pgp-signature")) {
  		    	//do nothing
  		    } else{
  		      if (part.getFileName()==null){
  		    	BridgeApp.LogWarning(Logs.Srvcs.SetText("Parser: Unknown Message part: "+part.getContentType()));
  		      }else{
  	  		      BridgeApp.LogInfo(Logs.Srvcs.SetText("Unknown saving"));
  	  		  realController.NewUnknFile(part);
  		      }
  		    }
  		}


	  }

	  IParseController mController;

	}

//////////////////////////////////////////////////////////////////////////
//MailParser and Parse control classes
//////////////////////////////////////////////////////////////////////////
class IParseController {

IParseController(dbUser oUser) { mUser = oUser; }

public void Begin() {}

public IParseController getRealController(){return null;}

public void DefineCtrlType(Message oMessage) throws Exception{}

public void ParseAdditionalHeaders(Message oMessage) throws Exception{}

public void NewRFCHeader(Message oMessage) throws Exception{}

public void NewXHeader(Message oMessage) throws Exception{}

public void NewUncknownHeader(Message oMessage) throws Exception{}

public void NewEncryptedFile(Part oPart) throws Exception {}

public void NewDicomFile(Part oPart) throws Exception {}

public void NewUnknFile(Part oPart) throws Exception {}

public void NewTxtFile(Part oPart) throws Exception {}

public void NewReport(Part oPart) throws Exception {}

public void End() {}

public static MESSAGE_TYPE getMessageType(Message msg) throws Exception{
	MESSAGE_TYPE res = MESSAGE_TYPE.MES_DEFAULT;

	if (msg.isMimeType("multipart/encrypted")){
		res = MESSAGE_TYPE.MES_ENCRYPTED;
	}else if (msg.isMimeType("multipart/signed")){
		res = MESSAGE_TYPE.MES_SIGNED;
	}else if (msg.isMimeType("multipart/report")){
		res = MESSAGE_TYPE.MES_REPORT;
	}
	return res;
}

//Some implemented methods

public HEADER_TYPE AnalyzeHeader(Message oMessage) throws Exception{

	HEADER_TYPE res = HEADER_TYPE.HEADER_NO_DN;
	mXTAG_ID = getMessageHeader(oMessage, DBConsts.XTAG_ID);
	mXTAG_PART = getMessageHeader(oMessage, DBConsts.XTAG_PART);
	mXTAG_TOTAL = getMessageHeader(oMessage, DBConsts.XTAG_TOTAL);
	mDN_To = getMessageHeader(oMessage, DBConsts.DN_RFC);


		if (mXTAG_ID == null && mXTAG_PART == null && mXTAG_TOTAL == null){
			res = HEADER_TYPE.HEADER_RFC;
		}else if(mXTAG_ID != null && mXTAG_PART != null){
			res = HEADER_TYPE.HEADER_XSETID;
		}else if (mDN_To != null){
			res = HEADER_TYPE.HEADER_UNCKN;
		}

	return res;

}

public boolean  IsSignedMultipartGood(Multipart _mp) throws Exception{

	  boolean result = false;
	  	   int _np = _mp.getCount();
	       //Assume that signed part has only 2 parts data and sign
	       if (_np == 2 && _mp.getBodyPart(1).isMimeType("application/pgp-signature")){
	    	   if (isSignatureGood(_mp.getBodyPart(0), _mp.getBodyPart(1))){
	    		    result = true;
	    		    mSignedOk = true;
	    	   }else{
	    		   mSignedOk = false;
	    		   BridgeApp.LogDebug(Logs.Srvcs.SetText("Bad Signature detected!!!"));
	    	   }
	       }
	       return result;
}

private boolean isSignatureGood(Part part_data, Part part_sign ) throws Exception {

		boolean res = false;
		String temp_dir = mUser.getWaitDirPath();
		String s_signed = FileTools.makeUniqName();
		String s_sign = s_signed + "_sign";
		s_signed += "_data";

		s_sign = temp_dir + File.separator + s_sign;
		s_signed = temp_dir + File.separator + s_signed;

		FileOutputStream fos1 = new FileOutputStream(s_sign, false);
		part_sign.writeTo(fos1);
		fos1.close();

		FileOutputStream fos2 = new FileOutputStream(s_signed, false);
		part_data.writeTo(fos2);
		fos2.close();

		if (GpgWrapper.isDetachedSignatureGood(s_sign, s_signed)) {
			res = true;
		}

		(new File(s_sign)).delete();
		(new File(s_signed)).delete();

		return res;
	}



/**
 * Get the left part of the header reparated by ":" or null
 *
 * @param msg
 *            Message
 * @param header
 *            String
 * @return String
 * @throws MessagingException
 */
protected static String getMessageHeader(Message msg, String header) throws MessagingException{
  String res = null;
	String [] h =msg.getHeader(header);
	   if ( (h != null) && (h.length > 0)) {
		      res = h[0];
		      BridgeApp.LogDebug(Logs.Srvcs.SetText("Header: "+header+" "+res));
		    }
	   return res;


}

protected void sendDNReport(String sFrom, String sTo, String sBody,
			String report_type, String sMessageID, String sXTagID, String sKeyID) {

		dbSmtpRecord smtp_rec = mUser.getSmtpTable().getDefaultRecord();
		CSmtpDNSender report_sender = new CSmtpDNSender(sFrom, sTo, smtp_rec
				.getIP(), smtp_rec.getLogin(), smtp_rec.getPassword(), String
				.valueOf(smtp_rec.getPort()), smtp_rec.isSSL());

		if ((sKeyID == null) || (sKeyID.length() == 0)) {
			report_sender.SendReport(sBody, sTo, report_type, sMessageID,
					sXTagID, "", DBConsts.MDN_RFC, false);
			BridgeApp
					.LogDebug(Logs.Srvcs
							.SetText("RFC Disposition Notification Report was generated"));

		} else {
			report_sender.SendReport(sBody, sTo, report_type, sMessageID,
					sXTagID, sKeyID, DBConsts.MDN_PGP, true);
			BridgeApp
					.LogDebug(Logs.Srvcs
							.SetText("X Disposition Notification Report was generated"));
		}

	}


/* parser local dbase */
protected String mXTAG_ID = null;
protected String mXTAG_PART = null;
protected String mXTAG_TOTAL = null;
protected String mDN_To = null;
protected dbUser mUser;
protected boolean mSignedOk = true;

}

/**
 * Parse controller
 */
class CParseMsgContr extends IParseController{
	CParseMsgContr(dbUser oUser) { super(oUser); }

	public void DefineCtrlType(Message oMessage) throws Exception{
		switch (getMessageType(oMessage)){

		case MES_ENCRYPTED:
			mRealController = new CEncryptedMsgContr(mUser);
			break;

		case MES_REPORT:
			mRealController = new CReportMsgContr(mUser);
			break;

		default:
			mRealController = new CDefaultMsgContr(mUser);
			break;

		}
	}

	public IParseController getRealController(){return mRealController;}

	private IParseController mRealController = null;
}

/**
 * Encrypted controller
 */
class CEncryptedMsgContr extends IParseController {

	String mTransID = null;
	String mFrom = null;
	String mMessageID = null;

	CEncryptedMsgContr(dbUser oUser) { super(oUser); }

	public void ParseAdditionalHeaders(Message oMessage) throws Exception{

        mMessageID   = getMessageHeader(oMessage, DBConsts.MID_RFC);
        if (mMessageID==null){
            BridgeApp.LogError(Logs.Srvcs.SetText(
            "No Message ID found in Email!"));
        }
        if (oMessage.getAllRecipients().length > 0) {
            mFrom = oMessage.getAllRecipients()[0].toString();
          }
          else {
            BridgeApp.LogError(Logs.Srvcs.SetText(
                "No Recipients found in Email!"));
           }
	}
	//-------------------------------------------------------------------------------

	//New RFC Header in Encrypted Message
	public void NewRFCHeader(Message oMessage) throws Exception{
        //Auto-Notification is enabled
        if (mUser.getAutoNotifyFlag()) {

          mTransID = FileTools.makeUniqName();

          dmtRecord r = new dmtRecord();
          // create a new DMT record and file
          r.m_name       = mDN_To;
          r.m_id         = mTransID + DBConsts.PARSER;
          r.m_report     = mFrom;
          r.m_memo1      = mMessageID;
          r.m_memo2      = mUser.getLogin();
          r.m_start_time = new Timestamp(System.currentTimeMillis());

          // Save all into DB
          if (dbQuery.setQueryInsertDmt(r.toString()) == null) {
            BridgeApp
                .LogError(Logs.Srvcs
                          .SetText("Error: 2.1 cannot insert a dmt record"));
          }

          BridgeApp.LogInfo(Logs.Srvcs.SetText("RFC message received. (ID: "
                                               + mMessageID +
                                               ") Waiting for decryption."));

          // Auto-Notification disabled
        }else {
          BridgeApp.LogInfo(Logs.Srvcs.SetText(
              "RFC Disposition Notification Report requested ! (Check Auto-Notify option)"));
         }
	}

	//-----------------------------------------------------------------------------------------------

	public void NewXHeader(Message oMessage) throws Exception{

		  BridgeApp.LogDebug(Logs.Srvcs.SetText(
            "Encrypted controller: try search in db memo1 and XTagId ..."));
		  dmtRecord r = DBridgeServer.Base().getDmtRecord("memo1", mXTAG_ID);
		  BridgeApp.LogDebug(Logs.Srvcs.SetText(
          "dmt Record ok ..."));
	      //search if we have a record in DB

	      //-----------------------------------------------
	      //         2.1. New XTAG_ID
	      //-----------------------------------------------
	      if (r == null) {
	    	BridgeApp.LogDebug(Logs.Srvcs
                      .SetText("empty dmt Record ..."));
	        r = new dmtRecord();

	        if (mUser.getAutoNotifyFlag()) {
	          if (oMessage.getAllRecipients().length > 0) {
	            mFrom = oMessage.getAllRecipients()[0].toString();
	          }
	          else {
	            BridgeApp.LogError(Logs.Srvcs
	                               .SetText("No Recipients in new PGP message!"));
	            return;
	          }

	          mTransID = FileTools.makeUniqName();

	          // create a new DMT record and file
	          r.m_name = mDN_To;
	          // add some ending to differ it from sender string
	          r.m_id = mTransID + DBConsts.PARSER;
	          if (mXTAG_TOTAL != null) {
	            r.m_total = Integer.parseInt(mXTAG_TOTAL);
	          }
	          r.m_received   = 1;
	          r.m_report     = mFrom;
	          r.m_start_time = new Timestamp(System.currentTimeMillis());
	          r.m_memo1      = mXTAG_ID;
	          r.m_memo2      = mUser.getLogin();
	          r.m_status     = true;

	          // if we have just 1 part
	          if (r.m_received == r.m_total) {
	            r.m_end_time = new Timestamp(System.currentTimeMillis());
	          }

	          vfilesRecord v = new vfilesRecord();
	          v.m_report = dbReportCode.print(dbReportCode.ERROR,
	                                          dbReportCode.GPG_CORRUPT_ENCRYPTION);
	          //must be equal to dmt transaction ID (used in decryptor)
	          v.m_id = r.m_id;
	          v.m_number = Integer.parseInt(mXTAG_PART);
	          v.m_mid = mMessageID;

	          // Save all into DB
	          if (dbQuery.setQueryInsertDmt(r.toString()) == null) {
	            BridgeApp
	                .LogError(Logs.Srvcs
	                          .SetText("Error: 2.1 cannot insert a dmt record"));
	          }

	          if (dbQuery.setQueryInsertFiles(v.toString()) == null) {
	            BridgeApp
	                .LogError(Logs.Srvcs
	                          .SetText("Error: 2.1 cannot insert a vfiles record"));
	          }
	        }

	        // -----------------------------------------------
	        // 2.2. Old XTAG_ID
	        // -----------------------------------------------
	      }
	      else {
	    	  BridgeApp.LogDebug(Logs.Srvcs
                      .SetText("The XTAG_ID has been detected ..."));
	          if (mUser.getAutoNotifyFlag()) {
					mTransID = r.m_id;
					// we have a new part
					vfilesRecord v = new vfilesRecord();
					v.m_report = dbReportCode.print(dbReportCode.ERROR,
							dbReportCode.GPG_CORRUPT_ENCRYPTION);
					v.m_id = r.m_id;
					v.m_number = Integer.parseInt(mXTAG_PART);
					v.m_mid = mMessageID;

			        if (mXTAG_TOTAL != null) {
				            r.m_total = Integer.parseInt(mXTAG_TOTAL);
				          }

					if (dbQuery.setQueryInsertFiles(v.toString()) == null) {
						BridgeApp.LogError(Logs.Srvcs
								.SetText("Error: 2.2.2 cannot insert vfiles"));
					}
					r.m_received++;
					if (dbQuery.setQueryUpdate("update dmt set received="
							+ Integer.toString(r.m_received) + " where id like \'"
							+ r.m_id + "\'") == null) {
						BridgeApp
								.LogError(Logs.Srvcs
										.SetText("Error: 2.2.2 cannot update received field in vfiles"));
					}
					// we have got the last part
					if (r.m_received == r.m_total && mXTAG_TOTAL != null) {
						r.m_end_time = new Timestamp(System.currentTimeMillis());
						r.m_status = false;

						// update timestamp
						if (dbQuery
								.setQueryUpdate("update dmt set end_time=to_timestamp('"
										+ r.m_end_time
										+ "')"
										+ " where dn_index="
										+ r.m_dn_index) == null) {
							// System.out.println("Error: 2.2.2 cannot update end_time
							// field in vfiles");
							BridgeApp
									.LogError(Logs.Srvcs
											.SetText("Error: 2.2.2 cannot update end_time field in dmt"));
						}

						// update status
						if (dbQuery
								.setQueryUpdate("update dmt set status=false where dn_index="
										+ r.m_dn_index) == null) {
							// System.out.println("Error: 2.2.2 cannot update end_time
							// field in vfiles");
							BridgeApp
									.LogError(Logs.Srvcs
											.SetText("Error: 2.2.2 cannot update status field in dmt"));
						}

					}


				}
			}
	}

	public void NewUncknownHeader(Message oMessage) throws Exception{
		BridgeApp.LogError(Logs.Srvcs.SetText("Unknown XTAG(s) in message: "
				+ mMessageID));
		if (mDN_To != null && mMessageID != null) {
			if (oMessage.getAllRecipients().length > 0) {
				mFrom = oMessage.getAllRecipients()[0].toString();
			} else {
				BridgeApp.LogError(Logs.Srvcs
						.SetText("No Recipients when generating RFC report!"));
				return;
			}

			sendDNReport(mFrom, mDN_To, dbReportCode.print(dbReportCode.ERROR,
					dbReportCode.MAIL_CORRUPT_SYNTAX), DBConsts.MDN_ERROR,
					mMessageID, "", "");

			throw new BadSyntaxException("Bad Syntax when parsing XSETID message with ID:" + mMessageID);
		}
	}

	public void NewEncryptedFile(Part oPart) throws Exception {

		String sEncryptedDir = mUser.getExtDirPath();
		String saveTo = "";
		if (mTransID!=null){
			if (mXTAG_PART==null) mXTAG_PART="";
			saveTo = mTransID + "_part" + mXTAG_PART + "_part";
			//System.out.println("File name generated: " + mTransID);
		} else{
			saveTo = FileTools.makeUniqName();
		}
	    saveTo += ("_crp.asc");
	    saveTo = sEncryptedDir + File.separator + saveTo;

	    FileOutputStream fos = new FileOutputStream(saveTo, false);
	    oPart.writeTo(fos);
	    fos.close();

	}
}
/*--------------------------- Default controller ----------------------------*/
class CDefaultMsgContr extends IParseController {

	String mTransID = null;
	String mFrom = null;
	String mMessageID = null;



	CDefaultMsgContr(dbUser oUser) { super(oUser); }

	public void ParseAdditionalHeaders(Message oMessage) throws Exception{

		mMessageID = getMessageHeader(oMessage, DBConsts.MID_RFC);
		mTransID = getMessageHeader(oMessage, "PARSER_TransID");
		mFrom = null;
		if ((oMessage.getAllRecipients()!=null) && (oMessage.getAllRecipients().length > 0)) {
			mFrom = oMessage.getAllRecipients()[0].toString();
		}

	}

	//------------------------------------------------------------------------------------------


	public void NewRFCHeader(Message oMessage) throws Exception{
		if (mUser.getAutoNotifyFlag()) {
			if (mTransID != null) {

				dmtRecord r = new dmtRecord();
				// search if we have a record in DB
				String s_rec = dbQuery.getDmtRecord(mTransID + DBConsts.PARSER);
				if (s_rec != null) {
					r.fromString(s_rec);

					mMessageID = r.m_memo1;
					if (mSignedOk){
						sendDNReport(r.m_report, r.m_name, "", DBConsts.MDN_OK,
							mMessageID, "", "");
					}else{
			            sendDNReport(r.m_report, r.m_name, dbReportCode.print(dbReportCode.ERROR, dbReportCode.GPG_CORRUPT_SIGN),
			                      DBConsts.MDN_ERROR, mMessageID, "", "");
						dbQuery
						.setQueryUpdate("update dmt set report=\'" +  dbReportCode.print(dbReportCode.ERROR, dbReportCode.GPG_CORRUPT_SIGN)
								+ "\' where dn_index="	+ r.m_dn_index);

						throw new badSignatureException("Bad Signature when parsing RFC message with ID:" + mMessageID);
					}

					dbQuery
							.setQueryUpdate("update dmt set status=false where dn_index="
									+ r.m_dn_index);
				}
			}
			// We have unecrypted RFC message with DN enabled
			else if ((mDN_To != null) && (mMessageID != null)
					&& (mFrom != null)) {
				sendDNReport(mFrom, mDN_To, "", DBConsts.MDN_OK, mMessageID,
						"", "");
			}
		}
	}

	//--------------------------------------------------------------------------------------------


	public void NewXHeader(Message oMessage) throws Exception{
	  BridgeApp.LogDebug(Logs.Srvcs.SetText("Default controller: try get dcm record"));

	  dmtRecord r = DBridgeServer.Base().getDmtRecord("memo1", mXTAG_ID);
		
      if ( r != null){
		BridgeApp.LogDebug(Logs.Srvcs.SetText("next ... get name"));
        String name = dbQuery.getFileName(r.m_id, Integer.parseInt(mXTAG_PART));
        
        //..................................
        // 	      2.2.1. Decrypted Part
        //..................................
        //if ((r != null) && (name != null)) {
        if ( name != null ) {

          //we have got decrypted message here
          // Change the file record
          String n_new = "decrypted ok";
          if (dbQuery.setQueryUpdate("update vfiles set report=\'"
                                     + n_new + "\' where id like \'" + r.m_id
                                     + "\' and number=" + mXTAG_PART) == null) {
            BridgeApp.LogError(Logs.Srvcs.SetText(
                "Error: 2.2.1 cannot update vfiles.name"));
          }

          // SEND REPORT HERE ...
          // Auto-Notification is enabled
          if (mUser.getAutoNotifyFlag()) {

            mFrom = r.m_report;
            mDN_To = r.m_name;
            String s_vfiles = dbQuery.setQueryFiles("select * from vfiles where vfiles.id like \'"
            		+r.m_id +"\' and vfiles.number="+Integer.parseInt(mXTAG_PART));

            mMessageID = (new vfilesRecord(s_vfiles)).m_mid;
            String sKeyID = getMessageHeader(oMessage, DBConsts.XTAG_DNKEYID);

            if (sKeyID == null) {
              BridgeApp.LogError(Logs.Srvcs.SetText("Header" +
                  DBConsts.XTAG_DNKEYID + " was not found"));
            }


            if (!mSignedOk){
	            sendDNReport(mFrom, mDN_To, dbReportCode.print(dbReportCode.ERROR, dbReportCode.GPG_CORRUPT_SIGN),
	                      DBConsts.MDN_ERROR, mMessageID, mXTAG_ID, sKeyID);

	            if (dbQuery.setQueryUpdate("update vfiles set report=\'"
							+ dbReportCode.print(dbReportCode.ERROR, dbReportCode.GPG_CORRUPT_SIGN) +
							"\' where id like \'" + r.m_id	+ "\' and number=" + mXTAG_PART) == null) {
						BridgeApp
								.LogError(Logs.Srvcs
										.SetText("Error: 2.2.1 cannot update vfiles.name"));
					}

	            throw new badSignatureException("Bad Signature when parsing XSETID message with ID:" + mMessageID);

            }else if (!r.m_memo1.equalsIgnoreCase(mXTAG_ID)) {
              sendDNReport(mFrom, mDN_To,
                           dbReportCode.print(dbReportCode.ERROR,
                                              dbReportCode.MAIL_CORRUPT_SYNTAX),
                           DBConsts.MDN_ERROR, mMessageID, mXTAG_ID, sKeyID);

              if (dbQuery.setQueryUpdate("update vfiles set report=\'"
						+ dbReportCode.print(dbReportCode.ERROR, dbReportCode.MAIL_CORRUPT_SYNTAX) +
						"\' where id like \'" + r.m_id	+ "\' and number=" + mXTAG_PART) == null) {
					BridgeApp
							.LogError(Logs.Srvcs
									.SetText("Error: 2.2.1 cannot update vfiles.name"));
				}

              throw new BadSyntaxException("Bad Syntax when parsing XSETID message with ID:" + mMessageID);

            }else {
              sendDNReport(mFrom, mDN_To, "", DBConsts.MDN_OK,
                           mMessageID, mXTAG_ID, sKeyID);
            }

            // Auto-Notification disabled
          }
          else {
            BridgeApp.LogInfo(Logs.Srvcs.SetText(
                "X Disposition Notification Report requested ! (Check Auto-Notify option)"));
            System.out.println(
                "X Disposition Notification Report requested ! (Check Auto-Notify option)");
          }
        }else{
        	BridgeApp.LogError(Logs.Srvcs.SetText("DMT record with id filename is not presented in database"));
        }
        }else{
        	BridgeApp.LogError(Logs.Srvcs.SetText("No DB entry for memo1 like " + mXTAG_ID + " file " + mXTAG_PART));
        }
	}


	//--------------------------------------------------------------------------------------------------


	public void NewUncknownHeader(Message oMessage) throws Exception{
		BridgeApp.LogError(Logs.Srvcs.SetText("Unknown XTAG(s) in message: "
				+ mMessageID));

		if (mDN_To != null && mMessageID != null && mUser.getAutoNotifyFlag()) {
			if (oMessage.getAllRecipients().length > 0) {
				mFrom = oMessage.getAllRecipients()[0].toString();
			} else {
				BridgeApp.LogError(Logs.Srvcs
						.SetText("No Recipients when generating XSETID report!"));
				return;
			}

			sendDNReport(mFrom, mDN_To, dbReportCode.print(dbReportCode.ERROR,
					dbReportCode.MAIL_CORRUPT_SYNTAX), DBConsts.MDN_ERROR,
					mMessageID, "", "");

			throw new BadSyntaxException("Bad Syntax when parsing XSETID message with ID:" + mMessageID);
		}
	}


	public void NewUnknFile(Part oPart) throws Exception {

		String mUnknownDir = mUser.getWaitDirPath();


		  FileOutputStream fos = new FileOutputStream("report_part_unckn.txt", false);
		  oPart.writeTo(fos);
		  fos.close();
		  System.out.println("filename = " + oPart.getFileName());


		  	XTag sXTag = GetXTag_StudyID(oPart);
		      //
		      // Create file name
		      //
		      BridgeApp.LogInfo(Logs.Srvcs.SetText("Create a file name (unknown type) with the X-Tag " +
		                                           sXTag.toString()));
		      String saveTo = CreateSubDirs(mUnknownDir, sXTag.toString());
		      saveTo += File.separator;
		      //saveTo += FileTools.makeUniqName();
		      //saveTo += "_";
		      saveTo += RetriveFileShortName(oPart.getFileName());
		      if ( BridgeApp.getMimer().isExtensionRequired(saveTo) ){
		        saveTo += ".";
		        saveTo += FindExtension(oPart.getContentType());
		      }
		      //
		      // Save part info into file
		      //
		      SaveInFile(oPart.getInputStream(), saveTo);
		    }

		    public void NewDicomFile(Part oPart) throws Exception {

		       String mDicomDir = mUser.getDecryptedDirPath();
		      //
		      // 1. Save file into temp file
		      //
		      String saveTo = mDicomDir;
		      saveTo += File.separator;
		      saveTo += FileTools.makeUniqName();
		      SaveInFile(oPart.getInputStream(), saveTo);

		      //
		      // 1. Read study_uid, file_uid
		      //
		      AttributeList list = new AttributeList();
		      list.read(saveTo, null, true, true);
		      // Read required data from dicom file
		      XTag sStudyUid = new XTag((String)Attribute.getSingleStringValueOrDefault(list,
		          TagFromName.StudyInstanceUID, ""));
		      String sFileUid = Attribute.getSingleStringValueOrDefault(list,
		          TagFromName.SOPInstanceUID, "");

		      //
		      // Rename file to  "study_uid\SOP_uid"
		      //
		      String renameTo = CreateSubDirs(mDicomDir, sStudyUid.toString());
		      renameTo += File.separator;
		      renameTo += sFileUid;
		      renameTo += ".";
		      renameTo += FindExtension(oPart.getContentType());
		      MoveFile(saveTo, renameTo);
		      //
		      // save stats
		      //
		      BridgeApp.Stats().SaveStatsForAttrList(list, "recieve");
		    }

		    public void NewTxtFile(Part oPart) throws Exception {

		      String mUnknownDir = mUser.getWaitDirPath();
		      XTag sXTag = GetXTag_StudyID(oPart);
		      //
		      // Create file name
		      //
		      BridgeApp.LogInfo(Logs.Srvcs.SetText("Create a file name (text type) with the X-Tag " +
		                                           sXTag.toString()));

		      String saveTo = CreateSubDirs(mUnknownDir, sXTag.toString());
		      saveTo += File.separator;
		      String sFileName = "";
		      try {
		        sFileName = oPart.getFileName();
		      } catch (MessagingException ex) {
		      }
		      if ((sFileName == null)||(sFileName.length() == 0))
		                           sFileName = FileTools.makeUniqName();
		      saveTo += RetriveFileShortName(sFileName);
		      if ( BridgeApp.getMimer().isExtensionRequired(saveTo) ){
//		      saveTo += FileTools.makeUniqName();
		        saveTo += ".";
		        saveTo += BridgeApp.getMimer().getExtension("text/plain");
		      }
		      //
		      // Save part info into file
		      //
		      SaveInFile(oPart.getInputStream(), saveTo);
		    }

		    private void MoveFile(String sFrom, String sTo) {
		      File oFrom = new File(sFrom);
		      File oTo = new File(sTo);
		      if (!oFrom.renameTo(oTo)) {
		        oTo.delete();
		        oFrom.renameTo(oTo);
		      }
		    }

		    private String CreateSubDirs(String sRootDir, String sSubDir) {
		      String sDir = sRootDir;
		      sDir += File.separator;
		      sDir += sSubDir;  // TODO thest this code if XTAG does not exist
		      File oFile = new File(sDir);
		      oFile.mkdirs();  // TODO message about report

		      return (oFile.getPath());
		    }

		    private XTag GetXTag_StudyID(Part oPart) throws MessagingException {
		        String s = "";
		        String[] vXTags = oPart.getHeader(DBConsts.XTAG_STUDYID);
		        if ( (vXTags != null) && (vXTags.length > 0)) {
		          s = vXTags[0];
		        }
		        return new XTag(s);
		      }

		      private XTag GetXTag_DNKeyID(Part oPart) throws MessagingException {
		    	    String s = "";
		    	    String[] vXTags = oPart.getHeader(DBConsts.XTAG_DNKEYID);
		    	    if ( (vXTags != null) && (vXTags.length > 0)) {
		    	      s = vXTags[0];
		    	    }
		    	    return new XTag(s);
		    	  }

		      private String RetriveFileShortName(String sFileName) {
		    	    String tmpnames[] = (sFileName).split("\"");
		    	    // TODO: delete old extension
		    	    return tmpnames[tmpnames.length - 1];
		    	  }

		      private String FindExtension(String content_type) {
		    	    int index = content_type.indexOf(";");
		    	    String mime_type;
		    	    if (index != -1) {
		    	      mime_type = new String(content_type.substring(0, index));
		    	    }
		    	    else {
		    	      mime_type = new String(content_type);
		    	    }

		    	    return BridgeApp.getMimer().getExtension(mime_type);
		    	  }

		      private void SaveInFile(InputStream is, String saveTo) throws IOException {
		    	    FileOutputStream fos = new FileOutputStream(saveTo, false);
		    	    //TODO: Change byte reading for read/write some buffer...
		    	    int b;
		    	    while ( (b = is.read()) != -1) {
		    	      fos.write(b);
		    	    }
		    	    fos.close();
		    	    is.close();
		    	  }

}

class CReportMsgContr extends IParseController {

	CReportMsgContr(dbUser oUser) { super(oUser); }

	public void NewReport(Part oPart) throws Exception{

		   boolean isRFC=true;
		    String[] h = oPart.getHeader("Disposition");
		    String disp = null;
		    String report_id=new String("");
		    if ( (h != null) && (h.length > 0)) {
		      disp = h[0];
		    }

		    h = oPart.getHeader(DBConsts.XTAG_CONTENTID);
		    String sID = null;
		    if ( (h != null) && (h.length > 0)) {
		      sID = h[0];
		      report_id=sID;
		      isRFC=false;
		    }

		    h = oPart.getHeader(DBConsts.OMID_RFC);
		    String sOrigMesID = null;
		    if ( (h != null) && (h.length > 0)) {
		      sOrigMesID = h[0];
		      report_id=sOrigMesID;
		    }

	  	    if (disp.equalsIgnoreCase(DBConsts.MDN_OK)) {
			      BridgeApp.LogInfo(Logs.Srvcs.SetText(
			          "Disposition Notification Report recieved " + sOrigMesID));

			  //RFC case
		      //Original MID is used to identify the record into the DB
		      if ( isRFC ){
		        // end set fields: status=false
		        // end_time=current time
		        if (sOrigMesID != null) {
		          String s_rec = dbQuery.getDmtRecord(sOrigMesID);
		          if (s_rec == null) {
		            BridgeApp.LogDebug(Logs.Srvcs.SetText(
		                "Cannot find specifyed record with ID " + sOrigMesID + " in DB"));
		            BridgeApp.LogError(Logs.Srvcs.SetText("1.2 corrupt mail syntax." +
		                                                  sOrigMesID));
		          }
		          else {
		            if (dbQuery.setQueryUpdate(
		                "update dmt set status=false where id like \'" + sOrigMesID +
		                "\'") == null) {
		              BridgeApp.LogError(Logs.Srvcs.SetText(
		                  "Error: 4.1 cannot update status field in dmt"));
		            }
		            Timestamp newtime = new Timestamp(System.currentTimeMillis());
		            if (dbQuery.setQueryUpdate("update dmt set end_time=to_timestamp('" +
		                                             newtime + "') where id like \'" + sOrigMesID + "\'") == null) {
		              BridgeApp.LogError(Logs.Srvcs.SetText(
		                "Error: 4.2 cannot update time_end field in dmt (Report parsing)"));
		            }
		          }
		        }
		        else {
		          BridgeApp.LogError(Logs.Srvcs.SetText(
		              "Error: 4.1 No Original Messgae ID in RFC report!"));
		        }
		      }// end RFC
		      //X-TAG case
		      else {
		        //dmtRecord r=new dmtRecord();
		        //search if we have a record in DB
		        // if sent+resent=received
		        //     set fields: dmt.status=false
		        //                 dmt.end_time=current time
		        // in vfiles:
		        //                 vfiles.status=false
		        String s_rec = dbQuery.getDmtRecord(sID);

		        if (s_rec != null) {
		          dmtRecord r= new dmtRecord(s_rec);
		          r.m_received++;
		          if ( (r.m_sent+r.m_resent) == r.m_received ){
		            if (dbQuery.setQueryUpdate(
		                "update dmt set status=false where id like \'" + sID + "\'") == null) {
		              BridgeApp.LogError(Logs.Srvcs.SetText(
		                  "Error: 4.2 cannot update status field in dmt (Report parsing)"));
		            }
		            Timestamp newtime = new Timestamp(System.currentTimeMillis());
		            if (dbQuery.setQueryUpdate("update dmt set end_time=to_timestamp('" +
		                                       newtime + "') where id like \'" + sID +
		                                       "\'") == null) {
		              BridgeApp.LogError(Logs.Srvcs.SetText(
		                  "Error: 4.2 cannot update time_end field in dmt (Report parsing)"));
		            }
		          }// end closing dmt
		          // close a vfile record
		          if ( sOrigMesID!=null ){
		             String v_rec = dbQuery.setQueryFiles("select * from vfiles where id like'"+sID+
		                 "' and mid like '"+sOrigMesID+"'");
		             if ( v_rec!= null ){
		               if (dbQuery.setQueryUpdate(
		                   "update vfiles set status=false where id like'"+sID+
		                   "' and mid like '"+sOrigMesID+"'") == null) {
		                 BridgeApp.LogError(Logs.Srvcs.SetText(
		                     "Error: 4.2 cannot update status field in vfiles (Report parsing)"));
		               }
		             }
		          }// end closing vfile record
		        }
		        else {
		          BridgeApp.LogDebug(Logs.Srvcs.SetText(
		              "Cannot find specifyed record with ID " + sID + " in DB"));
		          BridgeApp.LogError(Logs.Srvcs.SetText("1.2 corrupt mail syntax. "+
		               "There is not such ID " + sID));
		        }

		      }
		    }// end ok DN-report
		    else if (disp.equalsIgnoreCase(DBConsts.MDN_WARNING)) {
		      BridgeApp.LogWarning(Logs.Srvcs.SetText(
		          "Disposition Notification Report recieved ID="+report_id));
		      BridgeApp.LogWarning(getText(oPart));
		    }
		    else if (disp.equalsIgnoreCase(DBConsts.MDN_ERROR)) {
		      BridgeApp.LogError(Logs.Srvcs.SetText(
		          "Disposition Notification Report recieved ID="+report_id));
		      BridgeApp.LogError(getText(oPart));
		    }
		    else if (disp.equalsIgnoreCase(DBConsts.MDN_HARDERROR)) {
		      BridgeApp.LogError(Logs.Srvcs.SetText(
		          "Disposition Notification Report recieved ID="+report_id));
		      BridgeApp.LogError(getText(oPart));
		    }
	}


	private Vector getText(Part part){
		  Vector res = new Vector();

	      try {
				InputStream is = part.getInputStream();

				BufferedReader reader = new BufferedReader(
						new InputStreamReader(is));
				String thisLine = reader.readLine();

				while (thisLine != null) {
					res.add(thisLine);
					thisLine = reader.readLine();
				}
			} catch (Exception e) {
				BridgeApp.LogError(e);
			}

		  return res;
	  }

}

class BadSyntaxException extends Exception{
	public BadSyntaxException(String errStr) {
		super(errStr);
	}
}
