package dbridge;

import java.util.*;

import javax.mail.*;
import javax.mail.internet.MimeMessage;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.Security;

/*
 * Created: 01.11.03
 * Modified 25.01.04
 */

public class Receiver
{
    public Receiver()
    {
        this.host = null;
        this.user = null;
        this.pass = null;
        this.leaveMessage = false;
        this.imapFlag = false;
        this.mailParser = null;
        this.mPort = null;
        this.mStore = null;
        this.sslFlag = true;
    }

    public Receiver(dbMailRecord mailRec)
    {
        this.host = mailRec.getDomain();
        this.user = mailRec.getMail();
        this.pass = mailRec.getPassword();
        this.leaveMessage = false;
        this.imapFlag = mailRec.isImap();
        this.mailParser = null;
        this.mPort = Integer.toString(mailRec.getPort());
        this.sslFlag = mailRec.isSSL();
    }

    public void setIMAP(boolean flag)
    {
        this.imapFlag = flag;
    }

    public void setSSL(boolean flag)
    {
        this.sslFlag = flag;
    }

    public void setPort(String port)
    {
        this.mPort = port;
    }

    public boolean isIMAP()
    {
        return imapFlag;
    }

    public boolean getLeaveFlag()
    {
        return leaveMessage;
    }

    public void setLeaveFlag(boolean flag)
    {
        this.leaveMessage = flag;
    }

    public String getHost()
    {
        return host;
    }

    public void setHost(String host)
    {
        this.host = host;
    }

    public String getUser()
    {
        return user;
    }

    public void setUser(String user)
    {
        this.user = user;
    }

    public String getPass()
    {
        return pass;
    }

    public void setPass(String pass)
    {
        this.pass = pass;
    }

    public void setMailParser(MailParser o)
    {
        this.mailParser = o;
    }

    public boolean connect() throws Exception
    {
/*        if (mailParser == null)
        {
            throw new RuntimeException(
                    "Logic Error: not installed mailParser for MailReceiver");
        }*/
        if ((host == null) || (user == null))
        {
            throw new RuntimeException(
                    "Logic Error: Not enough information for getting mails. (host and username).");
        }

        // Create empty properties
        Properties props = new Properties();

        if (sslFlag)
        {
            Security.addProvider(new com.sun.net.ssl.internal.ssl.Provider());
            final String SSL_FACTORY = "dbridge.CMySSLSocketFactory";
            props.setProperty("mail.imap.socketFactory.class", SSL_FACTORY);
            props.setProperty("mail.pop3.socketFactory.class", SSL_FACTORY);

            props.setProperty("mail.imap.socketFactory.fallback", "false");
            props.setProperty("mail.pop3.socketFactory.fallback", "false");

            props.setProperty("mail.imap.port", "993");
            props.setProperty("mail.imap.socketFactory.port", "993");
            props.setProperty("mail.pop3.port", "995");
            props.setProperty("mail.pop3.socketFactory.port", "995");

        }

        if ((mPort != null) || (mPort.length() != 0))
        {
            if (imapFlag)
            {
                props.setProperty("mail.imap.port", mPort);
                props.setProperty("mail.imap.socketFactory.port", mPort);
            }
            else
            {
                props.setProperty("mail.pop3.port", mPort);
                props.setProperty("mail.pop3.socketFactory.port", mPort);
            }
        }

        // Get session
        mSession = Session.getInstance(props, null); //Default  ,null
        mSession.setDebug(false);

        if (imapFlag)
        {
            mStore = mSession.getStore("imap");
        }
        else
        {
            mStore = mSession.getStore("pop3");
        }
        BridgeApp.LogInfo(Logs.Srvcs.SetText(
                "Trying to establish connect: host=" + host + ", user=" + user));

        // Connect to store
        mStore.connect(host, user, pass);
        
        mIsConnected=true;
        
        return mIsConnected;
    }

    public void getMail()
    {
        try
        {
            connect();
            if (mStore.isConnected())
            {
                BridgeApp.LogInfo(Logs.Srvcs.SetText("Connection succeeded ..."));
            }

            boolean isProcessing=true;
            while (isProcessing) {
            	isProcessing=false;
				// Get folder
				Folder folder = mStore.getFolder("INBOX");

				// Open read-only
				folder.open(Folder.READ_WRITE);

				// if(folder.hasNewMessages()) {
				// Get directory
				Message message[] = folder.getMessages();
				BridgeApp.LogInfo(Logs.Srvcs
						.SetText("Receiver: Mails on server - "
								+ message.length));
				int n = message.length;
				if (n > 20)
					n = 20; // TODO use const
				if (n > 0)
					isProcessing = true;

				for (int i = 0; i < n; i++) {
					// Parse next message
					try {
						Message msg = message[i];

						//
						// Determine Content-Type of the message
						//
						ContentTypeAttributes cont_type_attrs = new ContentTypeAttributes(
								msg.getContentType());

						if (cont_type_attrs.getType().compareTo(
								Rfc2046Names.TYPE_MESSAGE_PARTIAL) == 0) {
							//
							// Collect this partial message
							//
							distributeNewPartialMessage(cont_type_attrs,
									(MimeMessage) msg);
						} else {
							//
							// Parse this "not splitted" message
							//
							mailParser.parse(msg);
							// Delete if not needed
							if (!leaveMessage) {
								BridgeApp.LogDebug(Logs.Srvcs
										.SetText("Receiver: delete the " + i
												+ "th message on the server"));
								msg.setFlag(Flags.Flag.DELETED, true);
							} else {
								BridgeApp.LogDebug(Logs.Srvcs
										.SetText("Receiver: leave the " + i
												+ "th message on the server"));
							}
						}
					} catch (Exception ex) {
						BridgeApp.LogError(Logs.Srvcs
								.SetText("Exception while processing message: "
										+ message[i].getSubject() + "\nDescr: "
										+ ex));
						// folder.close(!leaveMessage);
					}
				}
				/*
				 * this code has been transferred into a function 28.03.07
				 * Reason; optimal memory management
				 *  // // Processing of partial messages // Iterator
				 * itm=mPartialCollectionsMap.values().iterator(); while
				 * (itm.hasNext()) { PartialMessagesCollection
				 * partial_collection=(PartialMessagesCollection)itm.next();
				 *  // // Check if we can already reconstruct source message //
				 * if (isAllPartialMessagesPresent(partial_collection)) {
				 * MimeMessage
				 * enclosed_msg=recontructEnclosedMessageFromParts(mSession,partial_collection);
				 *  // // Parse message // mailParser.parse(enclosed_msg);
				 *  // // Delete all messages from server // if (!leaveMessage) {
				 * Iterator
				 * it=partial_collection.mMessagesMap.values().iterator();
				 * while(it.hasNext()) { Message msg2del=(Message)it.next();
				 * msg2del.setFlag(Flags.Flag.DELETED, true); } } } }
				 * mPartialCollectionsMap.clear();
				 */

				// }
				// Close connection
				if (folder.isOpen()) {
					folder.close(!leaveMessage);
				}
				processPartialMessage();
				reconnect();
			}// end while
            mPartialCollectionsMap.clear();
        }
        catch (Exception ex)
        {
            BridgeApp.LogError(Logs.Srvcs.SetText(
                    "Exception while checking mail: " + ex));
        }
        disconnect();
    }

    public boolean processPartialMessage(){
    	boolean res=true;
        //
        // Processing of partial messages
        //
        Iterator itm=mPartialCollectionsMap.values().iterator();
        try {
            while (itm.hasNext()) {
				PartialMessagesCollection partial_collection = (PartialMessagesCollection) itm
						.next();

				//
				// Check if we can already reconstruct source message
				//
				if (isAllPartialMessagesPresent(partial_collection)) {
					MimeMessage enclosed_msg = recontructEnclosedMessageFromParts(
							mSession, partial_collection);

					//
					// Parse message
					//
					mailParser.parse(enclosed_msg);

					//
					// Delete all messages from server
					//
					if (!leaveMessage) {
						Iterator it = partial_collection.mMessagesMap.values()
								.iterator();
						while (it.hasNext()) {
							Message msg2del = (Message) it.next();
							msg2del.setFlag(Flags.Flag.DELETED, true);
						}
					}
				}
			}// end while

		} catch (Exception ex) {
			BridgeApp.LogError(Logs.Srvcs
					.SetText("Exception while checking partial mail: " + ex));
		}
    	
    	return res;
    }
    public boolean disconnect()
    {
    	if ( !mIsConnected ) return false;
        try
        {
            mStore.close();
        }
        catch (Exception e)
        { return false;}
        mStore = null;
        mIsConnected = true;
        return mIsConnected;
    }

    public boolean check()
    {
        boolean checkRes = false;
        try
        {
            connect();
            checkRes = mStore.isConnected();
            disconnect();
            return checkRes;
        }
        catch (Exception ex)
        {
            return false;
        }
    }
    
    /**
     * Prevential connection reinitialization (required for some servers)
     * 28.03.07
     * @return boolean
     */
    public boolean reconnect() {
      try {
        disconnect();
        connect();
        return mIsConnected;
      } catch (Exception ex) {
        return false;
      }
    }
    
	/**
	 * Distribute message according to it ID from content type attributes
	 * @param cont_type_attrs - content type attributes
	 * @param msg - partial message for wich we are searching for the right place
	 */
	private void distributeNewPartialMessage(ContentTypeAttributes cont_type_attrs, MimeMessage msg)
	{
		String id=cont_type_attrs.getStringAttr(Rfc2046Names.ATTR_ID);
		int number=cont_type_attrs.getIntAttr(Rfc2046Names.ATTR_NUMBER);
		PartialMessagesCollection messages_for_id=(PartialMessagesCollection)mPartialCollectionsMap.get(id);
		if (messages_for_id==null)
		{
			messages_for_id=new PartialMessagesCollection();
			mPartialCollectionsMap.put(id,messages_for_id);
		}
		messages_for_id.mMessagesMap.put(new Integer(number),msg);
		if (cont_type_attrs.hasAttr(Rfc2046Names.ATTR_TOTAL)) 
			messages_for_id.mTotalAwaiting=cont_type_attrs.getIntAttr(Rfc2046Names.ATTR_TOTAL);
	}
	/**
	 * Check if we have a full collection of partial messages and can proceed with the reconstruction
	 * @param partial_collection Collection of partial-messages we have so far
	 * @return true if we have everything we need
	 */
	private boolean isAllPartialMessagesPresent(PartialMessagesCollection partial_collection)
	{
		if ( partial_collection.mMessagesMap.size()==partial_collection.mTotalAwaiting)
		{
			return true;
		}
		return false;
	}
	/**
	 * Reconstruct enclosed message from partial-messages according to RFC2046
	 * @param sess Session (used only to create new message and seems to be unnesessary)
	 * @param parts_collection All partial messages
	 * @return Reconstructed enclosed message
	 * @throws IOException
	 * @throws MessagingException
	 */
	private MimeMessage recontructEnclosedMessageFromParts(Session sess, PartialMessagesCollection parts_collection) throws IOException, MessagingException
	{
    	MimeMessage new_msg=new MimeMessage(sess);
    	// 1. Fragmentation agents must split messages at line
        // boundaries only. This restriction is imposed because
        // splits at points other than the ends of lines in turn
        // depends on message transports being able to preserve
        // the semantics of messages that don't end with a CRLF
        // sequence. Many transports are incapable of preserving
        // such semantics.
    	ByteArrayOutputStream enclosed_message_construction=new ByteArrayOutputStream();
    	for (int i=1; i<=parts_collection.mMessagesMap.size(); ++i)
    	{
    		Message msg_out=(Message)parts_collection.mMessagesMap.get(new Integer(i));
    		InputStream is=(InputStream)msg_out.getContent();
    		int val=0;
    		while((val=is.read())!=-1)
    			enclosed_message_construction.write(val);
    	}
    	MimeMessage msg_enclosed=new MimeMessage(sess,new ByteArrayInputStream(enclosed_message_construction.toByteArray()));
   	
    	
    	//
    	// 2. All of the header fields from the initial enclosing
        // message, except those that start with "Content-" and
        // the specific header fields "Subject", "Message-ID",
        // "Encrypted", and "MIME-Version", must be copied, in
        //  order, to the new message.
    	MimeMessage init_message=(MimeMessage)parts_collection.mMessagesMap.get(new Integer(1));
    	Enumeration enum_hdr=init_message.getAllHeaders();
    	while (enum_hdr.hasMoreElements())
    	{
    		Header hdr=(Header)enum_hdr.nextElement();
    		String sName=hdr.getName();
    		if (!sName.matches("Content-.*") &&
    		    sName.compareToIgnoreCase("subject")!=0 &&
    		   sName.compareToIgnoreCase("Message-ID")!=0 &&
    		   sName.compareToIgnoreCase("Encrypted")!=0 &&
        		sName.compareToIgnoreCase("MIME-Version")!=0)
    		{
    			new_msg.addHeader(hdr.getName(),hdr.getValue());
    		}
    	}
 

    	// 3. The header fields in the enclosed message which start
        // with "Content-", plus the "Subject", "Message-ID",
        // "Encrypted", and "MIME-Version" fields, must be
        // appended, in order, to the header fields of the new
        // message.  Any header fields in the enclosed message
        // which do not start with "Content-" (except for the
        // "Subject", "Message-ID", "Encrypted", and "MIME-
        // Version" fields) will be ignored and dropped.
    	enum_hdr=msg_enclosed.getAllHeaders();
    	while (enum_hdr.hasMoreElements())
    	{
    		Header hdr=(Header)enum_hdr.nextElement();
    		String sName=hdr.getName();
    		if (sName.matches("Content-.*") ||
    		    sName.compareToIgnoreCase("subject")==0 ||
    		   sName.compareToIgnoreCase("Message-ID")==0 ||
    		   sName.compareToIgnoreCase("Encrypted")==0 ||
        		sName.compareToIgnoreCase("MIME-Version")==0)
    		{
    			new_msg.addHeader(hdr.getName(),hdr.getValue());
    		}
    	}
    	new_msg.setContent((Multipart)msg_enclosed.getContent());
    	new_msg.saveChanges();

    	// 4. All of the header fields from the second and any
        // subsequent enclosing messages are discarded by the
        // reassembly process.

    	return new_msg;
	}

	/**
	 * Map for sorting Partial messages
	 */
	private Map mPartialCollectionsMap=new HashMap();

    /**
      pop3 host name
     */
    private String host;

    /**
         username for pop3
     */
    private String user;

    /**
         password
     */
    private String pass;

    /**
         Leave messages on server or not
         (default: true)
         WARNING!!!
     If leaveMessage set to false all received messages will be deleted from server
     */
    private boolean leaveMessage;

    /**
         type of mail server:
         true - if imap
         (default: false)
     */
    private boolean imapFlag;

    /**
         entity which process each Mail message
     */
    private MailParser mailParser;

    /**
     Port for connection
     */
    private String mPort;

    /**
     type of transport:
     if true - ssl connection will be used
     */
    private boolean sslFlag;

    /**
     variables for connect()/disconnect()
     */
    boolean  mIsConnected;
    Store    mStore;
    Session  mSession;
    
}

/**
 * class to parse and use ContentType header field in a convinient way
 * @author mcdi
 * 
 */
class ContentTypeAttributes
{
	private String m_type;
	private Map m_attributes=new HashMap();
	
	ContentTypeAttributes(String sContentType)
	{
    	StringTokenizer token=new StringTokenizer(sContentType);
//System.out.println(sContentType);
    	m_type=token.nextToken(";").toLowerCase();
    	while (token.hasMoreTokens()) 
    	{
    		String sName=token.nextToken("=;").toLowerCase();
    		String sVal="";
    		if (token.hasMoreTokens())
    		{
    			sVal=token.nextToken("=;");
    			m_attributes.put(sName,sVal);
    		}
        }            	
	}
	
	public String getType() {return m_type;}
	
	public boolean hasAttr(String key)
	{
		return m_attributes.containsKey(key);
	}
	public int getIntAttr(String key)
	{
		String sVal=(String)m_attributes.get(key);
		return Integer.parseInt(sVal);
	}
	public String getStringAttr(String key)
	{
		String sVal=(String)m_attributes.get(key);
		return sVal;
	}
}

/**
 * class to compile partial messages for one particular id
 * @author mcdi
 *
 */
class PartialMessagesCollection
{
	public Map mMessagesMap=new TreeMap();
	public int mTotalAwaiting=0;
	public String mId;
}

/**
 * Constants for the Names from RFC2046
 * @author mcdi
 *
 */
class Rfc2046Names
{
	public final static String TYPE_MESSAGE_PARTIAL="message/partial";
	public final static String ATTR_ID="id";
	public final static String ATTR_NUMBER="number";
	public final static String ATTR_TOTAL="total";
}
