package dbridge;

/**
 * <p>Title: DicomBridge</p>
 * <p>Description: Transfer  Images over e-mail</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: a</p>
 * @author unascribed
 * @version 1.0
 */

/*
 * Created: 01.11.03
 * Modified 25.01.04
 * Modified 22.06.04
 * Modified 17.08.04
 */

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;
import java.io.*;
//import java.sql.Date;
import java.sql.Timestamp;


// ---------------------------------------------------------
class CDecryptorTimerTask extends TimerTask
{
  public CDecryptorTimerTask(dbUser oUser)
  {
    mFilesProcessor = new CFilesDecryptor(oUser);
    mOverlookDirName = new String(oUser.getExtDirPath());
  }

  public void run()
  {
    //Vector vFiles=FileTools.GetFilesFromDir(mOverlookDirName);
    Vector vFiles = FileTools.GetNotHiddenFilesFromDir(mOverlookDirName);
    if (vFiles.size() <= 0) {
      return;
    }
    mFilesProcessor.ProcessFiles(vFiles);
  }

  private String mOverlookDirName;
  private CFilesDecryptor mFilesProcessor;
}

// ---------------------------------------------------------
class CSmtpSenderTimerTask extends TimerTask
{
  public CSmtpSenderTimerTask(dbUser oUser)
  {
    mFilesProcessor = new CSmtpFilesSender(oUser);
    mOverlookDirName = new String(oUser.getLocDirPath());
  }

  public void run()
  {
    //Vector vFiles=FileTools.GetFilesFromDir(mOverlookDirName);
    //Vector vFiles = FileTools.GetNotHiddenFilesFromDir(mOverlookDirName);
    Vector vFiles = FileTools.GetNotHiddenFilesWithinSubdirs(mOverlookDirName);
    if (vFiles.size() <= 0) {
      return;
    }
    mFilesProcessor.ProcessFiles(vFiles);
  }

  private String mOverlookDirName;
  private CSmtpFilesSender mFilesProcessor;
}

// ---------------------------------------------------------
class CDcmSenderTimerTask extends TimerTask
{
  public CDcmSenderTimerTask(dbBank oBank, dbUser oUser)
  {
    mFilesProcessor = new CDicomFilesSender(oBank, oUser);
    mOverlookDirName = new String(oUser.getDecryptedDirPath());
  }

  public void run()
  {
    //Vector vFiles=FileTools.GetFilesFromDir(mOverlookDirName);
    //Vector vFiles = FileTools.GetNotHiddenFilesFromDir(mOverlookDirName);
    Vector vFiles = FileTools.GetNotHiddenFilesWithinSubdirs(mOverlookDirName);

    if (vFiles.size() <= 0)
	{
      return;
    }
    mFilesProcessor.ProcessFiles(vFiles);
  }

  private String mOverlookDirName;
  private CDicomFilesSender mFilesProcessor;
}

// ---------------------------------------------------------
class CAppExecutorTimerTask extends TimerTask
{
  public CAppExecutorTimerTask(Vector TableData, dbUser oUser)
  {
    mFilesProcessor = new CAppExecutor(TableData, oUser);
    //mOverlookDirName = new String(oUser.getDecryptedDirPath());
    mOverlookDirName = new String(oUser.getWaitDirPath());
  }

  public void run()
  {
    //Vector vFiles=FileTools.GetFilesFromDir(mOverlookDirName);
//    Vector vFiles = FileTools.GetNotHiddenFilesFromDir(mOverlookDirName);
    Vector vFiles = FileTools.GetNotHiddenFilesWithinSubdirs(mOverlookDirName);

    if (vFiles.size() <= 0) {
      return;
    }
    mFilesProcessor.ProcessFiles(vFiles);

  }

  public void Terminate()
  {
    mFilesProcessor.Terminate();
  }

  private String mOverlookDirName;
  private CAppExecutor mFilesProcessor;
}

//-----------------------------------------------------------
class CMailCheckerTimerTask extends TimerTask
{
  public CMailCheckerTimerTask(dbUser oUser)
  {
    mMailFilesReceiver = new CMailFilesReceiver(oUser);
  }

  public void run()
  {
    mMailFilesReceiver.CheckMailForFiles();
  }

  private CMailFilesReceiver mMailFilesReceiver;
}

// ---------------------------------------------------------
abstract class CService
{
  public abstract TimerTask CreateTask(dbUser oUser);

  public abstract int GetTaskTimeout(dbUser oUser);

  public void ReleaseTask(dbUser oUser) {}

  protected CService()
  {
    mTimer = new Timer();
    mTasksMap = new HashMap();
  }

  public void AddUserTask(dbUser oUser)
  {
    // Removing previous task if found
    RemoveUserTask(oUser);

    // Adding new task
    String sLogin = oUser.getLogin();
    TimerTask oTask = CreateTask(oUser);
    mTasksMap.put(sLogin, oTask);
    // TODO read timeout from oUser
    mTimer.schedule(oTask, 0, GetTaskTimeout(oUser));
  }

  public void RemoveUserTask(dbUser oUser)
  {
    String sLogin = oUser.getLogin();
    TimerTask oOldTask = (TimerTask) mTasksMap.get(sLogin);
    if (oOldTask != null)
	{
      // cancel task
      oOldTask.cancel();
      mTasksMap.remove(sLogin);
      ReleaseTask(oUser);
    }
  }

  public boolean HasUserTask(dbUser oUser)
  {
    String sLogin = oUser.getLogin();
    TimerTask oOldTask = (TimerTask) mTasksMap.get(sLogin);
    if (oOldTask != null)
      return true;
    else
      return false;
  }

  public void RemoveAllTasks()
  {
    Iterator it = mTasksMap.values().iterator();
    while (it.hasNext()) {
      ( (TimerTask) it.next()).cancel();
    }
    mTasksMap.clear();
  }

  protected Timer mTimer;
  protected Map mTasksMap;
}

// ---------------------------------------------------------
class CDecryptService extends CService
{
  public CDecryptService(dbBank oBank)
  {
    mBank = oBank;
  }

  public TimerTask CreateTask(dbUser oUser)
  {
    return new CDecryptorTimerTask(oUser);
  }

  public int GetTaskTimeout(dbUser oUser)
  {
    return 30000;
  }

  private dbBank mBank;
}

//-----------------------------------------------------------
class CAppExecutorService extends CService
{

  public CAppExecutorService(Vector vTableData)
  {
    mTableData = vTableData;
    mAppExecutorTimerTask = null;
  }

  public TimerTask CreateTask(dbUser oUser)
  {
    mAppExecutorTimerTask = new CAppExecutorTimerTask(mTableData, oUser);
    return mAppExecutorTimerTask;
  }

  public int GetTaskTimeout(dbUser oUser)
  {
    return 3000;
  }

  public void setNewData(Vector vTableData)
  {
    mTableData = vTableData;
  }

  public void Terminate()
  {
    if (mAppExecutorTimerTask != null){
      mAppExecutorTimerTask.Terminate();
    }
  }

  private Vector mTableData;
  private CAppExecutorTimerTask mAppExecutorTimerTask;
}

// ---------------------------------------------------------
class CDcmSendService extends CService
{
  public CDcmSendService(dbBank oBank)
  {
    mBank = oBank;
  }

  public TimerTask CreateTask(dbUser oUser) {
    return new CDcmSenderTimerTask(mBank, oUser);
  }

  public int GetTaskTimeout(dbUser oUser) {
    return 30000;
  }

  private dbBank mBank;
}

// ---------------------------------------------------------
class CSmtpSendService extends CService
{
  public CSmtpSendService(dbBank oBank) {
    mBank = oBank;
  }

  public TimerTask CreateTask(dbUser oUser) {
//    return new CSmtpSenderTimerTask(mBank.getGPGWrapper(), oUser);
    return new CSmtpSenderTimerTask(oUser);
  }

  public int GetTaskTimeout(dbUser oUser) {
    int iTimeout = 30000;
    if (oUser.getAddressBook().getSize() > 0) {
      iTimeout = oUser.getAddressBook().getRecord(0).getTimeDelay();
      // minutes to Mili Seconds
      iTimeout *= 60000;
    }
    return iTimeout;
  }

  private dbBank mBank;
}

// ---------------------------------------------------------
class CMailCheckService extends CService
{
  public TimerTask CreateTask(dbUser oUser) {
    return new CMailCheckerTimerTask(oUser);
  }

  public int GetTaskTimeout(dbUser oUser) {
    return 30000;
  }
}

//----------------------------------------------------------------------------
//Added by Andrei
class CStreamReader implements Runnable
{
  private BufferedReader input;
  private String threadName;
  public CStreamReader(String threadName, BufferedReader input)
  {
    this.input = input;
    this.threadName = threadName;
  }

  public void run()
  {
    String currentLine;

    BridgeApp.LogInfo(Logs.Srvcs.SetText(threadName + " started.."));
    try {
      while ( (currentLine = input.readLine()) != null) {
        BridgeApp.LogInfo(Logs.Srvcs.SetText(threadName + ": " + currentLine));
      }
    }
    catch (Exception e) {
      BridgeApp.LogError(Logs.Srvcs.SetText(threadName + " Exception: " + e.getMessage()));
    }
    BridgeApp.LogInfo(Logs.Srvcs.SetText(threadName + " quit."));
  }
}

///////////////////////////////////////////////////////////////////////
// Garbage Collector
///////////////////////////////////////////////////////////////////////
class CCommonSchedulerService extends Timer
{
	public CCommonSchedulerService()
	{
		mTasksByName = new TreeMap();
		mTimer = new Timer();
	}
	public void AddNamedTask( String sName, TimerTask oTask, Date oStartTime, long nPeriod )
	{
		TimerTask oOld = (TimerTask)mTasksByName.put(sName, oTask);
		if (oOld!=null)
		{
			oOld.cancel();
		}
		mTimer.scheduleAtFixedRate(oTask, oStartTime, nPeriod);
	}
	public void RemoveNamedTask( String sName )
	{
		TimerTask oOld = (TimerTask)mTasksByName.remove(sName);
		if (oOld!=null)
		{
			oOld.cancel();
		}
	}
	public TimerTask GetNamedTask( String sName )
	{
		return (TimerTask)mTasksByName.get(sName);
	}
	public void RemoveAllTasks()
	{
		Collection oSet=mTasksByName.entrySet();
		for (Iterator it=oSet.iterator(); it.hasNext(); )
		{
			((TimerTask)((Map.Entry)it.next()).getValue()).cancel();
		}
		mTasksByName.clear();
	}

	private Timer mTimer;
	private TreeMap mTasksByName;
}

class CGarbageCollectorTask extends TimerTask
{
  public CGarbageCollectorTask(String sOverlookedDir, Collection colStandardDirs, double dFileTimeOut, Collection colExts)
  {
  	mFileTimeout=(long)(dFileTimeOut*3600000);
    mOverlookDir = new File(sOverlookedDir);
    mExtensions = new TreeSet(colExts);
    mStandardDirs = new TreeSet(colStandardDirs);
  }

  public void run()
  {
//System.out.println("Check");

        // DataBase
        BridgeApp.LogInfo(Logs.Srvcs.SetText("DataBase cleaning, try to find unreported messages"));
        lookReportsInDataBase();
        BridgeApp.LogInfo(Logs.Srvcs.SetText("DataBase cleaning, try to find old messages"));
        clearDataBase();
        BridgeApp.LogInfo(Logs.Srvcs.SetText("User directories cleaning"));
	//
	// Users
	//
  	File[] files=mOverlookDir.listFiles();
  	for (int i=0; i<files.length; i++)
  	{
  		//
  		// Users folders
  		//
  		File f=files[i];
  		if ( f.isDirectory() )
  		{
  			//
  			// In "@user" contents
  			//
		  	File[] files2=f.listFiles();
		  	for (int j=0; j<files2.length; j++)
		  	{
		  		File f2=files2[j];
		  		if ( f2.isDirectory() && mStandardDirs.contains(f2.getName()) )
		  		{
		  			ProcessStandardDirectoryContents(f2.listFiles());
		  		}
		  		else
		  		{
		  			ProcessFileInTempDirs(f2);
		  		}
		  	}
  		}
  		else
  		{
  			ProcessFileInTempDirs(f);
  		}
  	}
  	return;
  }
  private void ProcessFileInTempDirs (File f)
  {
  	//
  	// 1. Check: is old enough
  	//
  	Date fileDate = new Date(f.lastModified());
  	Date now = new Date();
  	if (now.getTime() - fileDate.getTime() < mFileTimeout)
  	{
  		return;
  	}
  	//
  	// Check if it's our old file
  	//
  	if (!mExtensions.contains(FileTools.getExtension(f)))
  	{
  		return;
  	}

	//
	// remove this file
	//
//System.out.println("Deleting file:" + f);
	f.delete();
  }

  private void ProcessStandardDirectoryContents (File[] files)
  {
  	for (int i=0; i<files.length; i++)
  	{
  		File f=files[i];
  		if ( f.isDirectory() && IsDirShouldBeDeleted(f) )
  		{
//System.out.println("Deleting dir:" + f);
			//remove this dir
			f.delete();
  		}
  	}
  }
  public boolean IsDirShouldBeDeleted(File f)
  {
  	//
  	// 1. Check: is empty
  	//
  	if (f.listFiles().length!=0) return false;
  	//
  	// 2. Check: is old enough
  	//
  	Date fileDate = new Date(f.lastModified());
  	Date now = new Date();

  	if (now.getTime() - fileDate.getTime() < mFileTimeout)
  	{
  		return false;
  	}
  	return true;
  }

  
  private void sendDNReport(String type, dmtRecord r, String msg){
	  
      //Sending DN report with error
      dbSmtpRecord smtp_rec =  DBridgeServer.Bank().getUserContainer().getUser(r.m_memo2).getSmtpTable().getDefaultRecord();
	   CSmtpDNSender report_sender = new CSmtpDNSender(r.m_report, r.m_name, 
			   smtp_rec.getIP(), smtp_rec.getLogin(), smtp_rec.getPassword(), 
			   String.valueOf(smtp_rec.getPort()), smtp_rec.isSSL());
	   
	   report_sender.SendReport(msg, r.m_name, type, r.m_memo1, "" , "", DBConsts.MDN_RFC, false );
	   BridgeApp.LogDebug(Logs.Srvcs.SetText("RFC Disposition Notification Report was generated"));	  
	  
  }
  
  public boolean lookReportsInDataBase(){
        //
        // 1. Check old records
        //
        
        Timestamp newtime = new Timestamp(System.currentTimeMillis());

        Date t              = new Date(newtime.getTime());
        Calendar calendar   = new GregorianCalendar();
        calendar.setTime(t);
        long day            = 24*60*60*1000;          // 1 day
        long yesterday      = calendar.getTimeInMillis()-day;
        String dmt_str      = "";
        String update_res   = "";
        Timestamp oldtime   = new Timestamp(yesterday);
        dmtRecord r         = new dmtRecord();
        dmt_str = dbQuery.setQueryDmt("select * from dmt where status=true and start_time<=to_timestamp('"+oldtime+"')");
        while ( dmt_str != null ){
           r.fromString(dmt_str);
           // extract transaction_id
           // if ended with DBConsts.PARSER, send a report error, GPG failed
           // send a number of files that where not sent
           if ( r.m_id.endsWith( DBConsts.PARSER) ){ // not reported transaction
              // send report: GPG failed
              String msg = dbReportCode.print(dbReportCode.ERROR,dbReportCode.GPG_CORRUPT_ENCRYPTION);
               
              sendDNReport(DBConsts.MDN_ERROR, r, msg);

              // close transaction
              update_res=dbQuery.setQueryUpdate("update dmt set end_time=to_timestamp('"+newtime+"') where dn_index="+r.m_dn_index);
              if ( update_res==null ) System.out.println("Error in DB end_time update by the GarbageCollector");

           } else { // report in logger about unreported sending
             BridgeApp.LogError(Logs.Srvcs.SetText("Unreported transaction found, id = "+r.m_id));
           }// end if

           // close transaction, by updating status=false
           update_res=dbQuery.setQueryUpdate("update dmt set status=false where dn_index="+r.m_dn_index);
           if ( update_res == null ){
               System.out.println("Error in DB status update by the GarbageCollector");
           }
           // next record
           dmt_str=dbQuery.setQueryDmt("select * from dmt where status=true and start_time<=to_timestamp('"+oldtime+"')");

       }// end while

        return true;
  }

  public boolean clearDataBase(){
        //
        // 1. Check old records // older than 10 Days
        //
        dbQuery query     = new dbQuery();
        Timestamp newtime = new Timestamp(System.currentTimeMillis());

        Date t            = new Date(newtime.getTime());
        Calendar calendar = new GregorianCalendar();
        calendar.setTime(t);
        long days         = 24*60*60*1000*10; // 10 days
        long period=calendar.getTimeInMillis()-days;
        String dmt_str    = "";
        String delete_res = "";
        dmtRecord r       = new dmtRecord();
        Timestamp oldtime = new Timestamp(period);
        dmt_str=query.setQueryDmt("select * from dmt where status=true and start_time<=to_timestamp('"+oldtime+"')");
        while ( dmt_str != null ){
           r.fromString(dmt_str);
           // delete the records
           delete_res=query.setQueryDelete("vfiles where id="+r.m_id);
           delete_res=query.setQueryDelete("dmt where id="+r.m_id);
           if (delete_res==null ) System.out.println("Services: cannot delete database records");
           // next record
           dmt_str=query.setQueryDmt("select * from dmt where status=true and start_time<=to_timestamp('"+oldtime+"')");
       }// end while

        return true;
  }



  private File mOverlookDir;
  private long mFileTimeout;
  private TreeSet mExtensions;
  private TreeSet mStandardDirs;

}
public class Services
{
	public static void main(String[] args)
	{
		//
		// Create the Garbage Collector Task
		//
		Vector vExtensions=new Vector(3);
		vExtensions.add(DBConsts.DECRYPT_TMP_EXT);
		vExtensions.add(DBConsts.ASC_TMP_EXT);
		vExtensions.add(DBConsts.BUILD_TMP_EXT);
		Vector vDirs=new Vector(5);
		vDirs.add(DBConsts.ARCHIVE_DIR);
		vDirs.add(DBConsts.DECRYPTED_DIR);
		vDirs.add(DBConsts.EXT_DIR);
		vDirs.add(DBConsts.LOC_DIR);
		vDirs.add(DBConsts.WAIT_DIR);
		double dTimeoutHours = DBConsts.GARBAGE_BAD_FILE_DIR_TIMEOUT;

		CGarbageCollectorTask oTask = new CGarbageCollectorTask("F:\\Documents and Settings\\mcdi\\My Documents\\JavaProjects\\dbridge\\run\\tmp",
								vDirs, dTimeoutHours, vExtensions );
		//oTask.run();

		//
		// Calculate start time
		//
		String sTimeForBegin= DBConsts.GARBAGE_START_TIME;
		DateFormat oDateFormat=new SimpleDateFormat("hh:mm");
		Calendar calNow=new GregorianCalendar(), calFromSettings=new GregorianCalendar();
		Date dateFromSettings=new Date();
		try { dateFromSettings = oDateFormat.parse(sTimeForBegin); }
		catch (Exception e) { System.out.println(e); }

		calNow.setTime(new Date());
		calFromSettings.setTime(dateFromSettings);
		calFromSettings.set(Calendar.YEAR,calNow.get(Calendar.YEAR));
		calFromSettings.set(Calendar.MONTH,calNow.get(Calendar.MONTH));
		calFromSettings.set(Calendar.DAY_OF_MONTH,calNow.get(Calendar.DAY_OF_MONTH));
		if (calFromSettings.getTimeInMillis()<calNow.getTimeInMillis())
		{
			calFromSettings.add(Calendar.DAY_OF_MONTH, 1);
		}
		Date oStartTime=calFromSettings.getTime();
System.out.println(new SimpleDateFormat().format(oStartTime));

		//
		// Calculate period in milis
		//
		long nPeriodMillis = DBConsts.GARBAGE_REPEAT_PERIOD * 60 * 60 * 1000;

		//
		// Start service
		//
		CCommonSchedulerService oSrvc=new CCommonSchedulerService();
		oSrvc.AddNamedTask(CServices.GARBAGE_COLLECOR_TASK, oTask, oStartTime, nPeriodMillis);

		return;
	}
}
