package dbridge;

//////////////////////////////////////////////////////////////////
/**
* This class creates a frame with a JEditorPane for loading HTML
* help files
*/
//package goes here
import java.io.*;
import javax.swing.event.*;
import javax.swing.*;
import java.net.*;
import java.awt.event.*;
import java.awt.*;
import java.util.Locale;
import java.util.ResourceBundle;

// Modification of HelpWindow

public class XHelp extends JFrame implements ActionListener{
    private final int WIDTH = 800;
    private final int HEIGHT = 600;
    private JEditorPane editorpane;
    private URL helpURL;

    String mLabelTitle;
    String mLabelButtonContents;
    String mLabelButtonClose;

    //////////////////////////////////////////////////////////////////
    /**
    * XHelp constructor
    * @param String and URL
    */
public XHelp(String title, URL hlpURL) {
    super(title);
    setIconImage(Toolkit.getDefaultToolkit().getImage(DBConsts.WINDOW_ICON_FILENAME));
    setLocale();
    helpURL = hlpURL;
    editorpane = new JEditorPane();
    editorpane.setEditable(false);
    try {
        editorpane.setPage(helpURL);
    } catch (Exception ex) {
        ex.printStackTrace();
    }
    //anonymous inner listener
    editorpane.addHyperlinkListener(new HyperlinkListener() {
        public void hyperlinkUpdate(HyperlinkEvent ev) {
            try {
                if (ev.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
                     editorpane.setPage(ev.getURL());
                }
            } catch (IOException ex) {
                //put message in window
                ex.printStackTrace();
            }
        }
    });

    getContentPane().add(new JScrollPane(editorpane));
    addButtons();
    // no need for listener just dispose
    setDefaultCloseOperation(DISPOSE_ON_CLOSE);
    // dynamically set location
    calculateLocation();
    setVisible(true);
    // end constructor
}

  /**
    *  Internationalization method.
    */
   private void setLocale(){
     Locale currentLocale = BridgeApp.getCurrentLocale();
     ResourceBundle labels = ResourceBundle.getBundle(
         "dbridge.prop.XHelpLabelsBundle", currentLocale);
     mLabelTitle =  labels.getString("Title");

     mLabelButtonContents = labels.getString("ButtonContents"); // Apply
     mLabelButtonClose = labels.getString("ButtonClose"); // Cancel
  }
/**
 * An Actionlistener so must implement this method
 *
*/
public void actionPerformed(ActionEvent e) {
    String strAction = e.getActionCommand();
    URL tempURL;
    try {
        if (strAction.compareTo(mLabelButtonContents) == 0 ) {
          tempURL = editorpane.getPage();
          editorpane.setPage(helpURL);
        }
        if (strAction.compareTo(mLabelButtonClose) == 0 ) {
          // more portable if delegated
          processWindowEvent(new WindowEvent(this,
          WindowEvent.WINDOW_CLOSING));
        }
    } catch (IOException ex) {
        ex.printStackTrace();
    }
}
/**
 * add buttons at the south
 */
private void addButtons() {
    JButton btncontents = new JButton(mLabelButtonContents);
    //btncontents.setIcon(new ImageIcon("./lib/res/thumbup.gif"));
    btncontents.addActionListener(this);
    JButton btnclose = new JButton(mLabelButtonClose);
    btnclose.addActionListener(this);
    btnclose.setIcon(new ImageIcon("./lib/res/thumbdown.gif"));
    //put into JPanel
    JPanel panebuttons = new JPanel();
    panebuttons.add(btncontents);
    panebuttons.add(btnclose);
    //add panel south
    getContentPane().add(panebuttons, BorderLayout.SOUTH);
}
/**
 * locate in middle of screen
 */
private void calculateLocation() {
    Dimension screendim = Toolkit.getDefaultToolkit().getScreenSize();
    setSize(new Dimension(WIDTH, HEIGHT));
    int locationx = (screendim.width - WIDTH) / 2;
    int locationy = (screendim.height - HEIGHT) / 2;
    setLocation(locationx, locationy);
}
public static void main(String [] args){
    URL index = ClassLoader.getSystemResource("index.html");
    new XHelp("Test", index);

}
}//end XHelp class
////////////////////////////////////////////////////////////////
