package dbridge;

import java.util.*;
import java.io.*;

/**
 * <p>Title: DicomBridge</p>
 * <p>Description: Transfer  Images over e-mail</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: a</p>
 * @author unknown
 * @version 1.0
 *
 * This class is a local database for the whole application
 * It provides an interface to storage containers such as
 * UserContainer, dbBox etc.
 */

 /*
 * Created: 21.01.03
 * Modified 25.01.04
 * Modified  2.03.04
 * Modified  6.03.04
 * Modified  21.03.04
 * Modified  25.03.04
 * Modified  26.03.04
 * Modified  11.08.04
 * Modified  16.09.04
 * Modified  29.09.04
 * Modified  30.09.04
 * Modified  12.10.04
 * Modified  30.12.04
 * Modified  15.04.05
 * Modified  25-26.11.05
 * Modified  22-29.01.06
 * Modified  2-27.02.06
 * Modified  1-28.05.06
 * Modified  9.08.07
 */

public class dbBank
{
  UserContainer mUserContainer;       // store information about users
  dbBox         mBox;                 // store information about a computer
  dcmStore      mDcmStore;            // dicom server threads
  UserInfo      mCurrentUser;         // login information about the currently logged user
  Vector        mDcmServerList;       // List of DICOM servers (server 0 is default)
  String        mFileSeparator;       // system separator
  dbVersion     mVersion;             // bank version is intoduced for compatibility managing between various versions

  public dbBank()
  {
    mUserContainer = new UserContainer();
    mBox           = new dbBox();
    mCurrentUser   = new UserInfo();
    mDcmServerList = new Vector();
    mFileSeparator = new String (System.getProperty("file.separator"));
    BridgeApp.SetLogger(new Logger(new Integer(this.getBox().getLogLevel()) ,this.getBox().getLogDir()));
  }
  public dbBank(String filename)
  {
    mVersion=new dbVersion("3.032.007"); // current version number Ver.Modif.DBnr 18.12.08
    mFileSeparator = new String (System.getProperty("file.separator"));
    mUserContainer = new UserContainer(); // an empty container
    mBox 		   = new dbBox();                   // an empty box
    mCurrentUser   = new UserInfo();        // an empty user
    mDcmServerList = new Vector();

    // activate mBox
    iniReader iniReader1 = new iniReader();
    int res=iniReader1.readFile(filename);
    if ( res==0 ){
     System.out.print("Error: ini file is bad or missed");
    } else {
      String id,val;
      /* take vaues from the ini-file. If a field is not defined into the
         file, then user second argument as a default value
      */
      String prefix;
      String postfix1;
      String postfix2;
      String postfix3;
      String cmd="";
      /*
      * LOGS
      */
      // read information about a software box
      mBox.setLogDir(this.correctPath(iniReader1.getField("COMMON.LogDir","."+mFileSeparator+"Logs")));
      // read information about logger activation
      String log_flag=iniReader1.getField("COMMON.LogEnableFlag","1");
      mBox.setLogLevel(Integer.parseInt(log_flag));

      // create the dir if necessary
      this.checkAndCreateDir( mBox.getLogDir() );
      
      /*
      * GabageCollector parameters
      * Will be read by the Server, not by the client
      */
      mBox.setStartTimeGC(iniReader1.getField("GCS.StartTime","4:0"));
      Long repeat=new Long(iniReader1.getField("GCS.RepeatPeriod","24"));
      mBox.setRepeatePeriodGC((long)repeat.longValue());
      Double use=new Double(iniReader1.getField("GCS.Not-in-use","1"));
      mBox.setNotInUseGC((double)use.doubleValue());
      Integer runInit = new Integer(iniReader1.getField("GCS.RunInit","0"));
      mBox.setRunInit(runInit.intValue());
      /*
      * Server
      */
      String run_str[];
      run_str=new String[9];

      mBox.setServer(correctName(iniReader1.getField("DICOM.Server","")));
      run_str[0]=new String(mBox.getServer());

      mBox.setServerOptions(correctName(iniReader1.getField("DICOM.Server_Options","")));
      run_str[1]=new String(mBox.getServerOptions());

      // setup peer
      mBox.setIP(iniReader1.getField("DICOM.IPAddress",""));
      prefix=iniReader1.getField("DICOM.IPAddress_prefix","");
      if ( !prefix.equals("null") ) {            // check if prefix is required
        cmd=cmd + " " + prefix + " " + mBox.getIP();

      }
      // setup AETitle
      mBox.setAET(iniReader1.getField("DICOM.AETITLE",""));
      prefix=iniReader1.getField("DICOM.AETITLE_prefix","");
      if ( !prefix.equals("null") ) {            // check if prefix is required
        run_str[2]=new String(prefix);
        run_str[3]=new String(mBox.getAET());
        mBox.setAetPrefix(prefix);
      }
      prefix=iniReader1.getField("DICOM.AETITLEC_prefix","");
      if ( !prefix.equals("null") ) {            // check if prefix is required
        mBox.setAetPrefix(prefix);
      }
      // setup temporal dir
      mBox.setTmpDir(this.correctPath(iniReader1.getField("DICOM.TmpDir","")));

      prefix=iniReader1.getField("DICOM.TmpDir_prefix","");
      if ( !prefix.equals("null") ) {            // check if prefix is required
        run_str[4]=new String(prefix);
        run_str[5]=new String(mBox.getTmpDir());
      }
      // create the temporal dir if necessary
      this.checkAndCreateDir(mBox.getTmpDir());

      // setup external command
      mBox.setExtCmd(correctName(iniReader1.getField("DICOM.Command","")));
      prefix=iniReader1.getField("DICOM.Command_prefix","");
      postfix1=iniReader1.getField("DICOM.Command_postfix1","");
      postfix2=iniReader1.getField("DICOM.Command_postfix2","");
      postfix3=iniReader1.getField("DICOM.Command_postfix3","");
      if ( !prefix.equals("null") ) {            // check if prefix is required
        //cmd=cmd + " " + prefix + " \"" + mBox.getExtCmd()+" "+postfix1+" "+postfix2+" "+postfix3+" \" ";
        // modification is made for Unix/MacOs
        cmd=cmd + " " + prefix + " " + mBox.getExtCmd()+" "+postfix1+" "+postfix2+" "+postfix3 +" ";
        run_str[6]=new String(prefix);
        run_str[7]=new String(mBox.getExtCmd()+" "+postfix1+" "+postfix2+" "+postfix3);
      }

      // setup port
      mBox.setPort(iniReader1.getField("DICOM.Port",""));
      prefix=iniReader1.getField("DICOM.Port_prefix","");
      if ( !prefix.equals("null") ) {             // check if prefix is required
        cmd=cmd + " " + prefix + " " + mBox.getPort();
        run_str[8]=new String(mBox.getPort());
      }
      mBox.setName(iniReader1.getField("DICOM.Name","server1"));

      /*
      * Sender
      */
      // setup DCM sender
      mBox.setDCMSender(correctName(iniReader1.getField("DICOM.Sender","./bin/storescu")));
      mBox.setDCMSenderOptions(correctName(iniReader1.getField("DICOM.Sender_Options","")));
      if ( iniReader1.getField("DICOM.InternalSender","off").compareTo("on")==0 ){
        mBox.setIntSender(true);
        //BridgeApp.LogInfo(Logs.Srvcs.SetText("#####Internal sender on"));
      }else{
        mBox.setIntSender(false);
        //BridgeApp.LogInfo(Logs.Srvcs.SetText("#####Internal sender off"));
      }
      // setup a disposition notification method
      Integer method=new Integer(iniReader1.getField("Transaction.MDNMethod","0"));
      if ( method.intValue()<DBConsts.MDN_NONE || method.intValue()>DBConsts.MDN_MIX )
        mBox.setMethodDN( DBConsts.MDN_NONE );
      else
        mBox.setMethodDN( method.intValue() );
      //BridgeApp.LogDebug(Logs.Bank.SetText("Disposition Notification method was set to " + mBox.getMethodDN() ));

      // setup a signature method
      method=new Integer(iniReader1.getField("Transaction.SIGNMethod","0"));
      if ( method.intValue()<DBConsts.PGPSIGN_NONE || method.intValue()>DBConsts.PGPSIGN_PGP )
        mBox.setMethodSign( DBConsts.PGPSIGN_NONE );
      else
        mBox.setMethodSign( method.intValue() );
      //BridgeApp.LogDebug(Logs.Bank.SetText("Signature method was set to " + mBox.getMethodSign() ));

      // setup a splitter
      Integer splitter_size=new Integer(iniReader1.getField("Transaction.SPLITTED","0"));
      if ( splitter_size.intValue()==0 ){
        mBox.setSplitterOn(false);
        mBox.setSplitterSize(splitter_size.intValue());
      }
      else{
        mBox.setSplitterOn(true);
        mBox.setSplitterSize(splitter_size.intValue());
      }
      
      // Updates
      mBox.setUpdateURL(iniReader1.getField("Update.URL","http://www.ultraosteon.com/Services/DBridge/dbridge.html"));

      /*
      * USERS
      */
      // activate users
      if ( fromFile() ){  // user databank exists
       //TODO mLog.println("MESSAGE: User file was loaded");
       //BridgeApp.LogInfo(Logs.Bank.SetText("User file was loaded"));
      } else {
       //mLog.println("MESSAGE: User file was not loaded and will be created now");
       BridgeApp.LogWarning(Logs.Bank.SetText("User file was not loaded and will be created now"));
       dbUser adm=new dbUser();
       adm.setName("Administrator");
       adm.setLogin("admin");
       adm.setID(0);
       adm.setPassword("");
       adm.setSmtp("");
       mUserContainer.append(adm);
       flush();                         // save data to file
      }

      // GPG Service
      mBox.setGpgPath(correctPath(iniReader1.getField("GPG.Path","./bin/pgp/")));
      GpgWrapper.setGpgPath(mBox.getGpgPath());

      // start servers
      mBox.setCmd(run_str);

      // DICOM STORE SERVICE

      // DICOM SERVER
      String dStr1="";
      String dStr2="";
      String dStr3="";
      String dStr4="";
      dStr1 = iniReader1.getField("DICOM.DefaultDicomServerIP","134.155.64.2");
      dStr2 = iniReader1.getField("DICOM.DefaultDicomServerPort","8000");
      dStr3 = iniReader1.getField("DICOM.DefaultDicomServerAET","DCMT");
      dStr4 = iniReader1.getField("DICOM.DefaultDicomServerComments","Default");
      dsDcmServerClass defaultSrv=new dsDcmServerClass(dStr1,dStr2,dStr3,dStr4);

         // activate servers
      if ( this.readDcmServerList() ){
         // there exists at least one server
         // copy default server into the first position
         ((dsDcmServerClass)mDcmServerList.get(0)).setComments(defaultSrv.getComments());
         ((dsDcmServerClass)mDcmServerList.get(0)).setDcmServer(defaultSrv.getDcmServer());
         ((dsDcmServerClass)mDcmServerList.get(0)).setDcmAeTitle(defaultSrv.getDcmAeTitle());
         ((dsDcmServerClass)mDcmServerList.get(0)).setDcmPort(defaultSrv.getDcmPort());
         // update users, if the default server was changed in the ini file
         //this.setUserDefaultServer();
         //this.flushDcmServerList();
      } else {  // new server
        //mLog.println("MESSAGE: \n  the server database was created. The default server was recorded.\n");
        BridgeApp.LogInfo(Logs.Bank.SetText("the server database was created. The default server was recorded.\n"));
        mDcmServerList.add(defaultSrv);
      }
       // update users, if the default server was changed in the ini file
       this.setUserDefaultServer();
       this.flushDcmServerList();
       flush();                         // save user data to the file
    }
  };

  //
  // methods for Dicom Servers List
  //
  public Vector getDcmServerList () {
    return mDcmServerList;
  }//

  public void setDcmServerList (Vector DcmServerList){
    mDcmServerList=DcmServerList;
  }//

  // save the current list of servers into the servers database
  public boolean flushDcmServerList(){
    boolean r=false;
//System.out.println("flush the DCM servers Nr" + mDcmServerList.size());
    try {
       FileOutputStream file = new FileOutputStream("servers.dbu");
       ObjectOutputStream oFile=new ObjectOutputStream(file);
       try {
         oFile.writeObject(mVersion); // save the version
       }catch(Exception e){
           BridgeApp.LogError(Logs.Bank.SetText("Cannot copy the version " + mVersion.getMainVersionN() +":"+ mVersion.getModificationN() +":"+ mVersion.getDataBaseN() + "into the server database"));
       }

       for ( int i=0; i<mDcmServerList.size(); i++ ){
          dsDcmServerClass obj=(dsDcmServerClass)mDcmServerList.get(i);
          oFile.writeObject(obj);
          oFile.flush();
       }
       oFile.close();
       r=true;
    }
    catch(Exception e){
      BridgeApp.LogError(Logs.Bank.SetText(e.toString()));
      r=false;
    }
    return r;
  }// end flushDcmServerList

  public boolean readDcmServerList(){ // read the vector from a file
    boolean r=false;
    mDcmServerList.clear();
    try {
      String mDcmServersDbName="servers.dbu";  // TODO make a member
      FileInputStream file = new FileInputStream(mDcmServersDbName);
      ObjectInputStream iFile = new ObjectInputStream(file);

      //
      // Check the version
      //
      dbVersion ver;
      try {
        ver = new dbVersion( (dbVersion) iFile.readObject());
      }
      catch (Exception e) {
        BridgeApp.LogError(Logs.Bank.SetText(
            "Cannot read the version number from the dicom server database"));
        iFile.close();
        return false;
      }
      if (ver.getDataBaseN() != mVersion.getDataBaseN()) {
        BridgeApp.LogError(Logs.Bank.SetText(
            "The version of the software ("+ mVersion.getDataBaseN() +") is not coinsident with the version of the server database (" +ver.getDataBaseN()+")"));
        iFile.close();
        // bachup the old DB
        File source = new File(mDcmServersDbName);
        String new_name = new String(mDcmServersDbName + ".bcp");
        File dest = new File(new_name);
        if (source.renameTo(dest)){
          BridgeApp.LogInfo(Logs.Bank.SetText(
              "Servers database has been backuped into a file with the *.bcp extension "));
        } else {
          BridgeApp.LogInfo(Logs.Bank.SetText(
              "Servers database has been not backuped into a file with the *.bcp extension"));
        }
        return false;
      }

      boolean done = false;
      while (!done) {
        try {
          dsDcmServerClass obj = (dsDcmServerClass) iFile.readObject();
          mDcmServerList.add(obj);
        }
        catch (IOException ioe) {
          iFile.close();
          done = true;
          r = true;
        } // end catch
      } // end while
    } catch(Exception e){
      BridgeApp.LogError(Logs.Bank.SetText(e.toString()));
      r=false;}
    return r;
  }// end readDcmServerList

  //
  // reading UserContainer from a file
  public boolean fromFile(){
     boolean r = mUserContainer.fromFile("users.dbu",mVersion); // read the vector from a file
     if ( r == true ){
        mUserContainer.activateHomes(mBox.getTmpDir());
     } else
       BridgeApp.LogInfo(Logs.Bank.SetText("Users.dbu was created"));
     return r;
  }// end fromFile

  // write UserContainer to a file
  public boolean flush(){
     boolean r= mUserContainer.toFile("users.dbu",mVersion); // read the vector from a file
     if ( r == true ){
        mUserContainer.activateHomes(mBox.getTmpDir());
     }
     return r;
  }// end fromFile

 // write UserContainer to a file
  public boolean setUserDefaultServer(){
     mUserContainer.setDefaultServer((dsDcmServerClass)mDcmServerList.get(0));
     boolean r= mUserContainer.toFile("users.dbu",mVersion); // read the vector from a file
     //startServices();
     return r;
  }// end fromFile

  public UserContainer getUserContainer(){
    return mUserContainer;
  };
   public void setUserContainer(UserContainer uc){
    mUserContainer=uc;
  };

  // methods that include indoramtion about the box (computer)
  public dbBox getBox(){
    return mBox;
  };
  public void setBox(dbBox box){
    mBox=box;
  };

  // login/exit information (last registered user)
  public UserInfo getCurrentUser(){
    return mCurrentUser;
  };
  public void setCurrentUser(UserInfo user){
    mCurrentUser=user;
  };

  public String getVersion(){ return mVersion.getVersion(); };
  public dbVersion getDBVersion(){ return mVersion; };

  /*
  *     Protected Methods
  */
  protected void checkAndCreateDir(String dir_name){
      File dir=new File( dir_name );
      if (!dir.exists()){
          dir.mkdir();
      }
  }

  /*
  *  Add a FileSeparator at the end if necessary, and correct all other separators
  */
  protected String correctPath(String path){
      int n = path.length();
      char c = path.charAt(n-1);
      if ( c!='/' && c!='\\' ) {
         path=path + mFileSeparator;
      }
      return correctName(path);
  }
  // substitute \\ by / for Unix* and MacOS
  // substitute / by \\ for Windows
  // in the specified filename
  protected String correctName(String filename){
      String anti="/";
      if ( mFileSeparator.compareTo(anti)==0 ) anti="\\";
      return filename.replace(anti.charAt(0),mFileSeparator.charAt(0));
  }

  /*
  *     Debug Methods
  */
  public static void main(String[] args) {
    dbBank dbBank1 = new dbBank("DBridge.ini");
  }
}
