package dbridge;

/**
 * <p>Title: DicomBridge</p>
 * <p>Description: Transfer  Images over e-mail</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: a</p>
 * @author unknown
 * @version 1.0
 *
 * This class converts MIME types into/from file extensions
 * using a Properties object.
 * To recognize extension, our local map is used
 * To recognize mime-type, system information is used is the local map
 * is not available
 */

//import java.lang.*;
import java.io.*;
import java.util.*;
import javax.activation.*;

public class dbMimeManager {
  Properties mProps;        // properties object
  String mComments;         // Ex: "#" comments
  String mMimeTypeMapFile;  // local storage of mime types
  MimetypesFileTypeMap mMimeTypeMap;

  // constructor
  public dbMimeManager(String filename) {
    mProps=new Properties();
    mComments = new String("#");
    // try to read a local mime types map (filename)
    // if the file does not exists, try to read system maps
    if (filename!=null){ /* local map */
       mMimeTypeMapFile=filename;
       if ( readMapFile(mMimeTypeMapFile) !=0 ){
         // classify
         try {
           mMimeTypeMap = new MimetypesFileTypeMap(filename);
         }
         catch (Exception pce) {
           //System.err.println("Cannot read a local mime.types file: " +
           //                  pce.getMessage());
           BridgeApp.LogError(Logs.Srvcs.SetText("Cannot read a local mime.types file"));
         }
       } else {
         mMimeTypeMapFile=null;
       }
     } else mMimeTypeMapFile=null;
     if (mMimeTypeMapFile == null){/* Use System map */
       try {
         //System.err.println("Attempt to read the system one ...");
         BridgeApp.LogInfo(Logs.Srvcs.SetText("Attempt to read the system one ..."));
         mMimeTypeMap=new MimetypesFileTypeMap(); // system mime.types is used
       } catch (Exception pce) {
          //System.err.println("Cannot read the system mime.types file: "+
          //                        pce.getMessage());
          BridgeApp.LogError(Logs.Srvcs.SetText("Cannot read the system mime.types file "));
       }
       // converting to extension is not awailable
    }
  }

  // Get all properties
  public Properties getProperies(){
    return mProps;
  };

  public MimetypesFileTypeMap getMimeTypesMap(){ return mMimeTypeMap;};

  // Set/Get methods
  public void setComments(String comments){
    mComments =  new String (comments);
  }
  public String getComments(){
    return mComments;
  }

  // Get field value. If does not exists, take default value "val"
  public String getExtension(String key, String val){
    return mProps.getProperty(key,val);
  };
  // Get field value, or null if no such key in the table
  public String getExtension(String key){
    return mProps.getProperty(key);
  };

  // check either a correct extension exists or not
  public boolean isExtensionRequired(String filename){
    String type=getMimeType(filename);
    String ext=getExtension(type);
    if ( ext!=null ){
      if ( filename.endsWith(ext) )
        return false;
    }
    return true;
  }

  // System MimeTypeMap
  public String getMimeType(String file_name){
      if ( file_name != null ) {
         return mMimeTypeMap.getContentType(file_name.toLowerCase());
      }
      return null;
  };

    // Read file and extract records
    public int readMapFile(String filename){
//      DataInputStream inifile=null;
      BufferedReader inifile=null;

      String buf=null;

      if (filename.length()==0 ){
       // System.out.println("Error: wrong file name");
        BridgeApp.LogWarning(Logs.Srvcs.SetText("Warning: wrong mime types file name"));
        return 0;
      }

      // open file
      try {
        inifile = new BufferedReader(new InputStreamReader(new FileInputStream(filename)));
      } catch(FileNotFoundException e){
          //System.out.println("Error: file is not found");
          BridgeApp.LogError(Logs.Srvcs.SetText("Error: mime types file is not found"));
          return 0;
      }

      // read first line
      boolean done=false;
      try{
        buf=inifile.readLine();
      } catch(IOException e){
        done=true;
      }
      // line analysis
      while (!done && buf!=null){
        // section
        if ( !buf.startsWith(mComments)){
            StringTokenizer token=new StringTokenizer(buf," ");
            if ( token.hasMoreTokens() ){
              String mime_type = new String(token.nextToken() );

              if ( token.hasMoreTokens()) {
               String ext =  new String(token.nextToken());
               mProps.setProperty(mime_type,ext);
              }
            }// end if
        }// end if

        // read next line
        try{
          buf=inifile.readLine();
        } catch(IOException e){
           done=true;
        }
      }// end while
      return 1;
  }// end readFile

/*
* DEBUG
*/
  public static void main(String[] args) {

  dbMimeManager dmm=new dbMimeManager("mymime.types");
  dmm.setComments("#");
  //dmm.readMapFile("mymime.types");
  System.out.println("EXTENSION: "+dmm.getExtension("image/jpeg"));
  System.out.println("EXTENSION: "+dmm.getExtension("application/msword"));
  System.out.println("MIME: "+dmm.getMimeType("help.doc"));
  System.out.println("MIME: "+dmm.getMimeType("help.pdf.pdf"));
 }
 /*
 * END DEBUG
 */
}
