package dbridge;

import java.awt.HeadlessException;
import java.awt.event.MouseListener;

import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;

import java.util.Vector;
import java.util.Iterator;
import java.util.ResourceBundle;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JScrollPane;
import java.awt.Component;
import java.awt.BorderLayout;
import javax.swing.Box;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionListener;
import javax.swing.DefaultListSelectionModel;
import javax.swing.table.JTableHeader;


/**
 * <p>Title: dsFileTable</p>
 * <p>Description: JTable for files selection</p>
 * <p>Copyright: Copyright (c) 2005</p>
 * <p>Company: AiLAB</p>
 * @author Dzmitry Stsepankou
 * @version 1.0
 */

public class dsFileTable extends JPanel {

  ResourceBundle labels;
  JTable mFileTable = null;
  dsFileTableModel mModel = null;
  TableSorter mSorter = null;
  CFileFolder mCurrFolder = null;
  ListSelectionListener mListener = null;
  Vector mTableData = new Vector();
  boolean ext_filter_applyed = false;
  boolean study_filter_applyed = true;

  public dsFileTable(ListSelectionListener listener,
                     CFileFolder data,
                     ResourceBundle Labels)  {

     mCurrFolder = data;
     labels=Labels;
     mListener = listener;

     try{
       initGUI();
     }catch(Exception e){ }

  }

  public void setHeader(){
       mSorter.setTableHeader(mFileTable.getTableHeader());
  }

  public void addTableMouseListener(MouseListener adapter){
    mFileTable.addMouseListener(adapter);
  }

 private void initGUI() throws HeadlessException{
   mModel = new dsFileTableModel(mCurrFolder.getFileFields());
   mSorter = new TableSorter(mModel);
   mFileTable = new JTable(mSorter);


   initColumnSizes(mFileTable);

   DefaultListSelectionModel selModel = new DefaultListSelectionModel();
   selModel.addListSelectionListener(mListener);
   mFileTable.setSelectionModel(selModel);

   JScrollPane jspMain = new JScrollPane();

   jspMain.getViewport().add(mFileTable);
   this.setLayout(new BorderLayout());
   this.add(jspMain, BorderLayout.CENTER);

   this.add(Box.createVerticalStrut(10), BorderLayout.NORTH);
   this.add(Box.createVerticalStrut(10), BorderLayout.SOUTH);
 //mSorter.setTableHeader(mFileTable.getTableHeader());

 }


  public void applyFolderChange(CFileFolder new_folder){
    mFileTable.clearSelection();
    mCurrFolder = new_folder;
    mTableData = mCurrFolder.getFileFields();
    //Vector mTableData = new Vector(FileTools.FilterFields(vFields,
    //     new AllFilesFilter()));
    mModel.setData( mTableData);
    mSorter.tableDataChanged();
    mFileTable.updateUI();
  }

  public void applyContainerSelection(Vector vFields){
    mFileTable.clearSelection();
    mModel.setData(vFields);
    mSorter.tableDataChanged();
    mFileTable.updateUI();
  }

  public void applyRestoreFolder(CFileFolder new_folder){
    mFileTable.clearSelection();
    mCurrFolder = new_folder;
    mTableData = mCurrFolder.getStoredFileFields();

    Vector data = clearDisabled(mTableData);
    mModel.setData(data);
    mSorter.tableDataChanged();
    mFileTable.updateUI();


  }

  private Vector clearDisabled(Vector vFileFields){
    Vector res = new Vector();
    for (Iterator it = vFileFields.iterator(); it.hasNext();) {
      CFileFieds curr = (CFileFieds) it.next();
      if (! curr.isDisabled()) {
        res.add(curr);
      }
    }
    return res;
  }

  public void updateAfterContainer(Vector vFields){
    mFileTable.clearSelection();

    mTableData = vFields;

    mModel.setData(clearDisabled(mTableData));
    mSorter.tableDataChanged();
    mFileTable.updateUI();

//    System.out.println("-- update --");
//   System.out.println("data size " + mTableData.size());
  }

  public Vector getTableData(){
    return mTableData;
  }

  public Vector getEnabledTableData(){
  return mModel.getData();
}




  public void appy_filters(){
    mTableData = new Vector(FileTools.FilterFields(mCurrFolder.getFileFields(),
         new ExtensionsFilter(mCurrFolder.getSelectedExtensionsStrings())));

    Vector vStudyIDs = mCurrFolder.getSelectedStudyIDsStrings();
    if (vStudyIDs.size()>0){
      mTableData = new Vector(FileTools.FilterFields(mTableData,
                                                     new StudyIdsFilter(vStudyIDs)));
    }

    Vector vEnabled = clearDisabled(mTableData);
    mModel.setData(vEnabled);
    mSorter.tableDataChanged();
    mFileTable.updateUI();

  }

  public void applyXTagChange() {
      mSorter.tableDataChanged();
      mFileTable.updateUI();
  }

  public void selectAll() {
    //mFileTable.selectAll(); NOT WORKS !!!!
    Vector data = mModel.getData();
    if (data != null) {
      if (mTableData.size() > 0) {
        mFileTable.setRowSelectionInterval(0,data.size() - 1);
      }
      else {
        mFileTable.clearSelection();
      }
    }
  }

  public void clearSelection(){
    mFileTable.clearSelection();
  }

  public Vector getSelectedFiles(){
    Vector result = new Vector();
    Vector data = mModel.getData();
    int [] sel_inds = mFileTable.getSelectedRows();
    for (int i=0; i<sel_inds.length; i++){
      result.add(((CFileFieds) data.get(sel_inds[i])).GetFileObject());
    }
    return result;
  }

  public Vector getSelectedFileFields(){
   Vector result = new Vector();
   Vector data = mModel.getData();
   int [] sel_inds = mFileTable.getSelectedRows();

//   System.out.println("-- selection --");
//   System.out.println("data size " + mTableData.size());
   for (int i=0; i<sel_inds.length; i++){
//     System.out.println("selection " + sel_inds[i]);
//     System.out.println("sorter " + mSorter.modelIndex(sel_inds[i]));
     result.add((CFileFieds) data.get(mSorter.modelIndex(sel_inds[i])));
   }
   return result;
 }


  public int getSelectedNum(){
    return mFileTable.getSelectedRows().length;
  }

  /**
   * A method to tune Table column width
   * @param table JTable
   */

  private void initColumnSizes(JTable table) {
    TableModel model = table.getModel();
    TableColumn column = null;
    Component comp = null;
    int headerWidth = 0;
    int cellWidth = 0;
    Object[] longValues = mModel.longValues;
    TableCellRenderer headerRenderer =
        table.getTableHeader().getDefaultRenderer();

    for (int i = 0; i < mModel.columns; i++) {
      column = table.getColumnModel().getColumn(i);

      comp = headerRenderer.getTableCellRendererComponent(
          null, column.getHeaderValue(),
          false, false, 0, 0);
      headerWidth = comp.getPreferredSize().width;

      comp = table.getDefaultRenderer(model.getColumnClass(i)).
          getTableCellRendererComponent(
          table, longValues[i],
          false, false, 0, i);

      cellWidth = comp.getPreferredSize().width;

      //XXX: Before Swing 1.1 Beta 2, use setMinWidth instead.
      column.setPreferredWidth(Math.max(headerWidth, cellWidth));
      //column.setMinWidth(Math.max(headerWidth, cellWidth));

    }
  }

  public static void main(String[] args) throws HeadlessException {

    Vector data = new Vector();
    JTable table = new JTable(data, null);
    table.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);



  }


  /**
   * TableModel subclass
   */
  class dsFileTableModel
    extends AbstractTableModel {
  private Vector table_data = null;
  public final int columns = 5;

  public dsFileTableModel(Vector data) {
    super();
    table_data = data;
    this.fireTableStructureChanged();
  }

  public JTableHeader getTableHeader(){
    return mFileTable.getTableHeader();
  }

 private String[] columnNames = {
      labels.getString("name"),
      labels.getString("size"),
      labels.getString("data"),
      labels.getString("type"),
      labels.getString("xtag")};

  public final Object[] longValues = {
      "XXXXXXXXXXXXX", "XXXXXX",
      "XXXXXXXXXXXX", "XXXX",
      "XXXXXXXXXX"};

  public int getColumnCount() {
    return columnNames.length;
  }

  public void setData(Vector data){
    table_data = data;
    this.fireTableDataChanged();
  }

  public Vector getData(){return table_data;}

  public int getRowCount() {
    return table_data.size();
  }

  public String getColumnName(int col) {
    return columnNames[col];
  }

  public Object getValueAt(int row, int col) {
    return ((CFileFieds) table_data.get(row)).Fields().get(col);
  }

/*  public boolean isCellEditable(int row, int col) {
    return false;
  }*/

//  public void setValueAt(Object value, int row, int col) {
//    ( (Vector) table_data.get(row)).setElementAt(value, col);
//    fireTableCellUpdated(row, col);
//  }
}
}
