package dbridge;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;

import javax.swing.AbstractCellEditor;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPasswordField;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

/**
 * The Table for keyID - Password
 * (initially located at Manual user main settings panel)
 * @author bob
 *
 */
public class dsKeyIDTable extends JScrollPane {

	private JTable table;
	private ChangesTracker mTracker;
	
	public void setDataVector(Vector keys){
		mGpgKeys = keys;
		newLine();
		table.updateUI();
	}
	
	private void newLine(){
			
		//New line if empty or User had entered a Key ID
		if (mGpgKeys.size()==0){
			mGpgKeys.add(new KeyPasswd(lbls.getString("new_id"), "".toCharArray()));
		}else if (!((KeyPasswd) mGpgKeys.get(table.getRowCount()-1)).get_keyID().equals(lbls.getString("new_id")))
		   {
			mGpgKeys.add(new KeyPasswd(lbls.getString("new_id"), "".toCharArray()));
		   }
	}
	
	public dsKeyIDTable(Vector GpgKeys, ChangesTracker tracker){
		mGpgKeys = GpgKeys;
		mTracker = tracker;
		
		Init();
		
	
		table = new JTable(new dsKeyIdTableModel());
		
		this.initColumnSizes();
		this.setUpKeyIdColumn(table.getColumnModel().getColumn(0));
		this.setUpPasswdColumn(table.getColumnModel().getColumn(1));
		table.updateUI();
		
		this.getViewport().add(table, null);
	}
	
	
	private String getPwdCaption(int index){
		String res;
		if (((KeyPasswd ) mGpgKeys.get(index)).get_passwd().length<1){
			res = lbls.getString("empty");
		}else{
			res = lbls.getString("valid");
		}
		return res;
	}
	
	private String getKeyIdCaption(int index){
		return ((KeyPasswd ) mGpgKeys.get(index)).get_keyID();
	}
	
	class KeyIdCellEditor
	extends AbstractCellEditor
    implements TableCellEditor, ActionListener{
		
		private JButton button = new JButton();
		private int mRow;
		private dsGPGKeyDialog gpg = new dsGPGKeyDialog(null, true);
		
		public KeyIdCellEditor(){
			button.setActionCommand("EDIT");
			button.addActionListener(this);
		}
		
		public Component getTableCellEditorComponent(JTable arg0, Object arg1, boolean isSelected, int row, int column) {
			mRow = row;
			button.setText(getKeyIdCaption(row));
			return button;
		}

		public Object getCellEditorValue() {
		       return null;
		}

		public void actionPerformed(ActionEvent e) {
			if ((e.getActionCommand()).equals("EDIT")){
				gpg.setVisible(true);
				  if (gpg.isOk()) {
					      ((KeyPasswd) mGpgKeys.get(mRow)).set_keyID(gpg.getSelectedSecretKeyID());
					      button.setText(getKeyIdCaption(mRow));
					      mTracker.UsersChanged = true;
					      newLine();
					   }
			}
			
		}
	}
	
	class PassCellEditor
    extends AbstractCellEditor
    implements TableCellEditor, ActionListener, PropertyChangeListener {
		
		JButton button = new JButton();
		JOptionPane optionPane;
		JLabel pwd_lbl = new JLabel();
		JLabel conf_lbl = new JLabel();
		JPasswordField pwd_field = new JPasswordField();
		JPasswordField conf_field = new JPasswordField();
		Object[] array = {pwd_lbl, pwd_field, conf_lbl, conf_field};
		int mRow;
		
		public PassCellEditor(){
	
			pwd_lbl.setText(lbls.getString("passwd"));
			conf_lbl.setText(lbls.getString("confirm"));
			button.setActionCommand("EDIT");
			button.addActionListener(this);
	
			optionPane = new JOptionPane(array, JOptionPane.QUESTION_MESSAGE, JOptionPane.DEFAULT_OPTION);
			optionPane.addPropertyChangeListener(this);
		
	
		}
		
		public Component getTableCellEditorComponent(JTable arg0, Object arg1, boolean isSelected, int row, int column) {
			mRow = row;
			button.setText(getPwdCaption(row));
			return button;
		}

		public Object getCellEditorValue() {
		       return null;
		}

		public void actionPerformed(ActionEvent e) {
			if ((e.getActionCommand()).equals("EDIT")){
                pwd_field.setText("");
                conf_field.setText("");
				JDialog d = optionPane.createDialog(table, lbls.getString("passwd"));
				d.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
				d.setModal(true);
				d.setVisible(true);
				
			}
			
		}
		
		public void propertyChange(PropertyChangeEvent e) {
		       String prop = e.getPropertyName();
		        if (isVisible() && (e.getSource() == optionPane) &&
		           (prop.equals(JOptionPane.VALUE_PROPERTY) ||
		            prop.equals(JOptionPane.INPUT_VALUE_PROPERTY))) {
		              Object value = optionPane.getValue();
		              if (value == JOptionPane.UNINITIALIZED_VALUE) {
		               // ignore reset
		                return;
		              }
		              optionPane.setValue(JOptionPane.UNINITIALIZED_VALUE);
		              String pass = new String (pwd_field.getPassword());
		              String confirmPass = new String (conf_field.getPassword());
		                  if (pass.equals(confirmPass)) {
				              
		                	  //Set the password
		                	  ((KeyPasswd) mGpgKeys.get(mRow)).set_passwd(pwd_field.getPassword());
		                	  button.setText(getPwdCaption(mRow));
		                	  mTracker.UsersChanged = true;
				              //  setVisible(false);
				                
		                  } else {

		                     JOptionPane.showMessageDialog(table,lbls.getString("incorrect_msg"),
							 lbls.getString("error"), JOptionPane.ERROR_MESSAGE);
		                     pwd_field.setText("");
		                     conf_field.setText("");
		                  }
		        }
		
			
		}


		
	}
	
	private void setUpPasswdColumn(TableColumn pwdColumn){
		//JButton pwd_but = new JButton(lbls.getString("pwd_but_text"));
		pwdColumn.setCellEditor(new PassCellEditor());

        //Set up tool tips for the sport cells.
        DefaultTableCellRenderer renderer =
                new DefaultTableCellRenderer();
        renderer.setToolTipText("Click for combo box");
        pwdColumn.setCellRenderer(renderer);
		
	}
	
	private void setUpKeyIdColumn(TableColumn keyIdColumn){
		//JButton pwd_but = new JButton(lbls.getString("pwd_but_text"));
		keyIdColumn.setCellEditor(new KeyIdCellEditor());

        //Set up tool tips for the sport cells.
        DefaultTableCellRenderer renderer =
                new DefaultTableCellRenderer();
        renderer.setToolTipText("Click for combo box");
        keyIdColumn.setCellRenderer(renderer);
		
	}
	

	/**
	 * Table Model subclass
	 * 
	 * @author bob
	 * 
	 */
	class dsKeyIdTableModel extends AbstractTableModel {

		/**
		 * The Names of the columns
		 */
		private String[] columnNames = { 
				lbls.getString("key_id"),
				lbls.getString("passwd") };
		
		//private JButton pwd_but = new JButton();

		/**
		 * The mask to calculate columns size
		 */
		public final Object[] longValues = { "XXXXXXXX", "XXXXXXXXXX" };

		public int getColumnCount() {
			return columnNames.length;
		}

		public int getRowCount() {
			return mGpgKeys.size();
		}

		public String getColumnName(int col) {
			return columnNames[col];
		}

		public boolean isCellEditable(int row, int col) {
			return true;
		}

		/**
		 * The main method to define "what to show"
		 */
		public Object getValueAt(int row, int col) {
			String res = null;
			switch (col) {
			case 0:
				res = ((KeyPasswd) mGpgKeys.get(row)).get_keyID();
				break;
			case 1:
				//TODO temp change to button   
				if (((KeyPasswd ) mGpgKeys.get(row)).get_passwd().length<1){
					res = lbls.getString("empty");
				}else{
					res = lbls.getString("valid");
				}
				break;
			}

			return res;
		}

	}
	
	  private void initColumnSizes(){
		  dsKeyIdTableModel model = (dsKeyIdTableModel) table.getModel();
		    TableColumn column = null;
		    Component comp = null;
		    int headerWidth = 0;
		    int cellWidth = 0;
		    Object[] longValues = model.longValues;
		    TableCellRenderer headerRenderer =
		        table.getTableHeader().getDefaultRenderer();

		    for (int i = 0; i < 2; i++) {
		      column = table.getColumnModel().getColumn(i);

		      comp = headerRenderer.getTableCellRendererComponent(
		          null, column.getHeaderValue(),
		          false, false, 0, 0);
		      headerWidth = comp.getPreferredSize().width;

		      comp = table.getDefaultRenderer(model.getColumnClass(i)).
		          getTableCellRendererComponent(
		          table, longValues[i],
		          false, false, 0, i);

		      cellWidth = comp.getPreferredSize().width;

		      //XXX: Before Swing 1.1 Beta 2, use setMinWidth instead.
		      column.setPreferredWidth(Math.max(headerWidth, cellWidth));
		      column.setMinWidth(Math.max(headerWidth, cellWidth));
		    }

		  }

	
	private void Init(){
		   Locale currentLocale = BridgeApp.getCurrentLocale();
		   lbls = ResourceBundle.getBundle("dbridge.prop.dsKeyIDTableBundle",
		                                    currentLocale);
	}
	
	private Vector mGpgKeys;
	private ResourceBundle lbls;
	
}
