package dbridge;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Calendar;

/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: </p>
 * @author not attributable
 * @version 1.0
 */
class dsSendStatusPanel extends JPanel implements IProcessOwner{

  private JProgressBar mProgressBar_files;
  private JLabel lFileName;
  private JLabel lSent;
  private JButton bAbort;
  private JPanel main;
  private ISenderControl m_ctrl = null;

  final private String ERROR = "EVENT_ERROR";
  final private String REPAINT = "EVENT_REPAINT";

  private ResourceBundle lbls;

  public dsSendStatusPanel() {

    try {
      Init();
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }

  }

  public void setProcController(ISenderControl controller){
    m_ctrl = controller;
  }

  private void Init() {

    Locale currentLocale = BridgeApp.getCurrentLocale();
    lbls = ResourceBundle.getBundle("dbridge.prop.dsSendStatusDlgBundle",
                                    currentLocale);

    mProgressBar_files = new JProgressBar(0, 100);

    lFileName = new JLabel();
    lFileName.setText(lbls.getString("filename"));
    bAbort = new JButton();
    bAbort.setText(lbls.getString("bAbort"));
    bAbort.addActionListener(new ActionListener(){
      public void actionPerformed(ActionEvent e){
        if (m_ctrl != null) m_ctrl.stop_send();
      }
    });
    lSent = new JLabel();
    lSent.setText(lbls.getString("sent"));

    main = new JPanel(new GridLayout(0, 1, 1, 1), true);
    main.add(lFileName);

    Box progressBox = Box.createHorizontalBox();
    progressBox.add(mProgressBar_files);
    progressBox.add(bAbort);

    main.add(progressBox);
    main.add(lSent);

    main.setMaximumSize(new Dimension(120, 40));
    this.add(main);
  }

  public void ProcessProgress(int i, int n) {
    int progress = 0;

    if (n != 0) {
      progress = (int) 100.0 * (i + 1) / n;
    }

    try {
      showProgress(progress, i);
    }
    catch (Exception e) {
      JOptionPane.showMessageDialog(this, e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
    }
  }

  private void showProgress(int progress, int i) throws Exception {

    final int iFiles = i;
    final int p = progress;
    Runnable updateProgressBar = new Runnable() {
      public void run() {
        lSent.setText(lbls.getString("sent") + " " + (iFiles + 1));
        mProgressBar_files.setValue(p);
        mProgressBar_files.updateUI();
      }
    };
    SwingUtilities.invokeLater(updateProgressBar);

  }

  private void createErrorPanel(int failed) throws Exception {
    final int f = failed;

    Runnable showErrorPanel = new Runnable() {
      public void run() {

        Calendar rightNow = Calendar.getInstance();
        int sec = rightNow.get(Calendar.SECOND);
        int minuts = rightNow.get(Calendar.MINUTE);
        int hour = rightNow.get(Calendar.HOUR_OF_DAY);
        String time = new String(Integer.toString(hour) + ":" +
                                 Integer.toString(minuts) + ":" +
                                 Integer.toString(sec) + " ");

        main.removeAll();

        main.add(new JLabel(time));
        main.add(lFileName);
        main.add(lSent);
        if (f > 0) {
          lFileName.setText(lbls.getString("SendError"));
          lSent.setText(lbls.getString("FilesFailed") + " " + f);
        }
        else {
          lFileName.setText(lbls.getString("SendOk"));
        }
      }
    };
    SwingUtilities.invokeLater(showErrorPanel);

  }

  public void ProcessFinished(int size, String msg) {
    this.mProgressBar_files.setValue(0);

    try {
      createErrorPanel(size);
    }
    catch (Exception e) {
      JOptionPane.showMessageDialog(this, e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
    }
  }

}


/**
 * SMTP send status
*/

class dsSMTPStatusPanel extends JPanel implements IProcessOwner{

  private JProgressBar mProgressBar_files;
  private JProgressBar mProgressBar_xtags;
  private JLabel lFileName;
  private JLabel lSent;
  private JButton bAbort;
  private JPanel main;
  private int m_curr_xtag = 0;
  private int m_xtags_total = 0;
  private ISenderControl m_ctrl = null;
  private boolean m_stop = false;


  final private String ERROR = "EVENT_ERROR";
  final private String REPAINT = "EVENT_REPAINT";

  private ResourceBundle lbls;

  public dsSMTPStatusPanel() {

    try {
      Init();
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }

  }

  public void setProcController(ISenderControl controller) {
    m_ctrl = controller;
  }


  private void Init() {

    Locale currentLocale = BridgeApp.getCurrentLocale();
    lbls = ResourceBundle.getBundle("dbridge.prop.dsSendStatusDlgBundle",
                                    currentLocale);

    mProgressBar_files = new JProgressBar(0, 100);
    mProgressBar_xtags = new JProgressBar(0, 100);

    lFileName = new JLabel();
    lFileName.setText(lbls.getString("filename"));
    bAbort = new JButton();
    bAbort.setText(lbls.getString("bAbort"));
    bAbort.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (m_ctrl != null) m_ctrl.stop_send();
        m_stop = true;
      }
    });

    lSent = new JLabel();
    lSent.setText(lbls.getString("sent"));

    main = new JPanel(new GridLayout(0, 1, 1, 1), true);
    main.add(lFileName);

    Box mainBox = Box.createVerticalBox();
    mainBox.add(mProgressBar_xtags);
    mainBox.add(mProgressBar_files);
    Box progressBox = Box.createHorizontalBox();
    progressBox.add(mainBox);
    progressBox.add(bAbort);

    main.add(progressBox);

    main.setMaximumSize(new Dimension(120, 40));
    this.add(main);
  }

  public void ProcessProgress (int curr, int total){
    int progress_files = 0;
    int progress_xtags = 0;

    if (m_xtags_total != 0) {
      progress_xtags = (int) 100.0 * (m_curr_xtag + 1) / m_xtags_total;
    }


    if (total != 0) {
      progress_files = (int) 100.0 * (curr + 1) / total;
    }

    try {
      showProgress(progress_files, progress_xtags);
    }
    catch (Exception e) {
      JOptionPane.showMessageDialog(this, e.getMessage(), "Error",
                                    JOptionPane.ERROR_MESSAGE);
    }

  }

  public void XtagProgress(int curr_xtag, int total_xtags){
    m_curr_xtag = curr_xtag;
    m_xtags_total = total_xtags;
  }

  private void showProgress(int progress, int progr_xtags) throws Exception {

    final int p = progress;
    final int px = progr_xtags;
    Runnable updateProgressBar = new Runnable() {
      public void run() {
        mProgressBar_files.setValue(p);
        mProgressBar_xtags.setValue(px);
        mProgressBar_files.updateUI();
        mProgressBar_xtags.updateUI();
      }
    };
    SwingUtilities.invokeLater(updateProgressBar);

  }

  private void createErrorPanel(int failed) throws Exception {
    final int f = failed;

    Runnable showErrorPanel = new Runnable() {
      public void run() {

        Calendar rightNow = Calendar.getInstance();
        int sec = rightNow.get(Calendar.SECOND);
        int minuts = rightNow.get(Calendar.MINUTE);
        int hour = rightNow.get(Calendar.HOUR_OF_DAY);
        String time = new String(Integer.toString(hour) + ":" +
                                 Integer.toString(minuts) + ":" +
                                 Integer.toString(sec) + " ");

        main.removeAll();

        main.add(new JLabel(time));
        main.add(lFileName);

        if (f > 0) {
          main.add(lSent);
          lFileName.setText(lbls.getString("SendError"));
          lSent.setText(lbls.getString("FilesFailed") + " " + f);
        }
        else {
          lFileName.setText(lbls.getString("SendOk"));
        }

        if (m_stop){
          lFileName.setText(lbls.getString("Stop"));
        }
      }
    };
    SwingUtilities.invokeLater(showErrorPanel);

  }

  public void ProcessFinished(int size, String msg) {
    this.mProgressBar_files.setValue(0);

    try {
      createErrorPanel(size);
    }
    catch (Exception e) {
      JOptionPane.showMessageDialog(this, e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
    }
  }

}

public class dsSendStatusDlg
    extends JDialog {

  private JScrollPane mainPane;
  private JPanel mPanel;
  private ResourceBundle lbls;
  private JButton bClear;
  private JButton bClose;
  private String mTitle;
  private boolean m_smtp_mode = false;

  public dsSendStatusDlg(Frame frame, String title, boolean modal, boolean smtp_mode) {
    super(frame, title, modal);
    mTitle = title;
    m_smtp_mode = smtp_mode;
    try {
      jbInit();
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  public dsSendStatusDlg(String title, boolean smtp_mode) {
    this(null, title, false, smtp_mode);

    mTitle = title;
  }

  private void jbInit() throws Exception {

    Locale currentLocale = BridgeApp.getCurrentLocale();
    lbls = ResourceBundle.getBundle("dbridge.prop.dsSendStatusDlgBundle",
                                    currentLocale);

    this.setTitle(mTitle);
    this.setSize(new Dimension(300, 300));
    this.getContentPane().setLayout(new BorderLayout());
    mainPane = new JScrollPane();
    mPanel = new JPanel(new GridLayout(0, 1, 10, 10), true);
    mPanel.setMaximumSize(new Dimension(180, 10000));
    mainPane.getViewport().add(mPanel);

    JPanel button_pane = new JPanel();
    bClear = new JButton();
    bClear.setText(lbls.getString("bClear"));
    bClear.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        bClear_actionPerformed(e);
      }
    });

    bClose = new JButton(lbls.getString("bClose"));
    bClose.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        Close_action();
      }
    });


    button_pane.add(bClose);
    button_pane.add(bClear);

    this.getContentPane().add(mainPane, BorderLayout.CENTER);
    this.getContentPane().add(button_pane, BorderLayout.SOUTH);
    this.getContentPane().add(Box.createVerticalStrut(10), BorderLayout.NORTH);
    this.getContentPane().add(Box.createHorizontalStrut(10), BorderLayout.EAST);
    this.getContentPane().add(Box.createHorizontalStrut(10), BorderLayout.WEST);

  }

  private void Close_action(){
    this.setVisible(false);
  }

  public void bClear_actionPerformed(ActionEvent e) {
    int compts = mPanel.getComponentCount();
    for (int i = 0; i < compts; i++) {
      mPanel.getComponent(i).invalidate();
    }
    mPanel.removeAll();
    mPanel.updateUI();
  }

  public IProcessOwner addStatusProcessor() {

    IProcessOwner pane = null;

    if (m_smtp_mode) pane = new dsSMTPStatusPanel();
    else pane = new dsSendStatusPanel();

    mPanel.add((JPanel) pane, 0);
    this.validate();

    return pane;
  }

}
