package dbridge;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.util.*;
import java.io.*;
import java.awt.event.*;


import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeSelectionModel;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import java.net.URL;


/**
 * <p>Title: dsUserInfoTree</p>
 * <p>Description: Manual User interface main class</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: </p>
 * @author Dzmitry Stsepankou
 * @version 1.0
 */

  //01.10.2004 Created
  //06.10.2004 Changed
  //1.12.04-28.12.04 Updated
  //1.04.05-15.04.05 Updated

/**
* Class for internal JTree data nodes
* consists of:
* @param mContainer (Leaf GUI)
* @param Name (GUI description) which will appears on the Tree
*/

class dsTreeLeaf{

    private String Name="";
    private boolean mInitiated=false;
    private Container mContainer=null;

    public dsTreeLeaf(String Title, Container cont){
      Name = Title;
      mContainer = cont;
      if (mContainer!=null){
        if (mContainer.getComponentCount() > 0) {
          mInitiated = true;
        }
      }
    }

    public void setContainer(Container cont){
      mContainer = cont;
      if (mContainer.getComponentCount()>0){
        mInitiated=true;
      }
    }
    public String toString(){
      return Name;
    }

    public Container getContainer(){
      return mContainer;
    }

    /**
     *@param mInitialized is used to dinamically
     * create the GUI only in case if it necessary.
     */
    public boolean isInitiated(){
      return mInitiated;
    }
  }

interface ICAppExecuteServiceUpdt{
  public void updateAppExecService();
}


interface IUpdateUser{
  public void userChanged();
}


/**
 * Class dsUserInfoTree_1 holds the frames
 * with JTree GUI
 */

public class dsUserInfoTree_1 extends JFrame implements TreeSelectionListener,
    IGuiWithFolderChecker, ICAppExecuteServiceUpdt, IUpdateUser{

  JPanel jPanel1 = new JPanel();
  JPanel jPanel2 = new JPanel();
  JScrollPane jScrollPane1 = new JScrollPane();
  JScrollPane jScrollPane2 = new JScrollPane();
  BorderLayout borderLayout1 = new BorderLayout();

  JTree jTree1 = null;
  BorderLayout borderLayout2 = new BorderLayout();

  ResourceBundle lbls;
  ResourceBundle lbls_gen;

  JPanel jPanel3 = new JPanel();
  JButton jbHelp = new JButton();
  JButton jbOk = new JButton();
  JButton jbRestartServer = new JButton();

  private CFolderChecker mCheckExt;
  private CFolderChecker mCheckLoc;
  private CFolderChecker mCheckDecr;
  private CAppExecutorService mAppExecutor = null;
  private String msUserHome;
  private String msRootDir;

  private dbUser mUser;

  //GUI components
  private dsUserGeneralInfo mInfoPanel = null;
  private dsUserDlg1Frame mEmailSendFrame = null;
  private dsUserDlg2Frame mDCMSendFrame = null;
  private dsAppChooserDlg mAppChooserDlg = null;
  private dsPOPPanel mPOPPan = null;
  private dsADDRPanel mADDRPan = null;
  private dsPOPPanelActionListener mPOPlistener = null;
  private boolean mUserChanged = false;


  File Dir_loc, Dir_ext, Dir_decr;
  String num_files_loc, num_files_ext, num_files_decr;

  public void userChanged(){
   DBridgeClient.Bank().flush();    // save Bank
   mUserChanged = true;
 }


  //=========================== internal class =============================
  private class dsUserGeneralInfo extends JPanel{

    Box box1;

    JPanel jpUser = new JPanel();
    JPanel jpEmail = new JPanel();
    JPanel jpDCM = new JPanel();
    JPanel jpAutoOptions = new JPanel();

    TitledBorder tbEmail;
    TitledBorder tbDCM;
    TitledBorder tbAutoOptions;

    JLabel jlEmail_files = new JLabel();
    JLabel jlDcm_files = new JLabel();
    JLabel jlUserName = new JLabel();
    JLabel jlUser = new JLabel();

    Box box2;
    JCheckBox jcbAutoSend = new JCheckBox();
    JCheckBox jcbAutoNotify = new JCheckBox();
    JCheckBox jcbAutoExecute = new JCheckBox();
    JPanel jpDCMServers = new JPanel();
    // comment if debug GUI
    JComboBox jcbDCMServers = null; //new JComboBox(DBridgeClient.Bank().getDcmServerList());
   // uncomment if GUI debug
   //JComboBox jcbDCMServers = new JComboBox();

    JButton jbExploreWait = new JButton();

    private String sTitle = null;

    public dsUserGeneralInfo(String title){
         sTitle = title;
         String sUserName = mUser.getName();
         jcbDCMServers = new JComboBox(DBridgeClient.Bank().getDcmServerList()); // create and fill a combobox

         // comment if debug GUI
         Locale currentLocale = BridgeApp.getCurrentLocale();
         //Locale currentLocale = new Locale("en");
         ResourceBundle labels = ResourceBundle.getBundle("dbridge.prop.dsUserGenInfoPanelBundle",currentLocale);
         lbls_gen = labels;

         box1 = Box.createVerticalBox();
         tbEmail = new TitledBorder(BorderFactory.createEtchedBorder(),
                                          labels.getString("EMAILfiles"));

         tbDCM = new TitledBorder(BorderFactory.createEtchedBorder(),
                                          labels.getString("DCMfilesTitle"));

         tbAutoOptions = new TitledBorder(BorderFactory.createEtchedBorder(),
                                          labels.getString("OptionsTitle"));

         jlUser.setText(labels.getString("UserName"));
         jlUser.setText(jlUser.getText() + " " + sUserName);
         jpUser.setBorder(BorderFactory.createLoweredBevelBorder());
         jpUser.add(jlUser);
         box1.add(jpUser);

         jpDCM.setBorder(tbDCM);
         jlDcm_files.setText(labels.getString("HaveFiles"));
         jpDCM.add(jlDcm_files);
         box1.add(jpDCM);

         jpEmail.setBorder(tbEmail);
         jlEmail_files.setText(labels.getString("HaveFiles"));
         jpEmail.add(jlEmail_files);
         box1.add(jpEmail);

         box2 = Box.createVerticalBox();
         jpAutoOptions.setBorder(tbAutoOptions);

         jcbAutoSend.addActionListener(new java.awt.event.ActionListener() {
           public void actionPerformed(ActionEvent e) {
             jcbAutoSend_actionPerformed(e);
           }
         });
         jcbAutoSend.setText(labels.getString("cbAutoDCMSend"));
         jcbAutoExecute.setText(labels.getString("cdAutoOpen"));
         jcbAutoExecute.addActionListener(new java.awt.event.ActionListener() {
          public void actionPerformed(ActionEvent e) {
            jcbAutoExecute_actionPerformed(e);
          }
        });
         
         jcbAutoNotify.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(ActionEvent e) {
               jcbAutoNotify_actionPerformed(e);
             }
           });
           jcbAutoNotify.setText(labels.getString("cbAutoNotify"));
           jcbAutoNotify.setSelected(mUser.getAutoNotifyFlag());
   

         jbExploreWait.setText(labels.getString("bExplore"));

         //jpDCMServers.add(new JLabel("      "));
         jpDCMServers.add(jcbDCMServers);

         box2.add(jcbAutoSend);
         box2.add(jpDCMServers);
         box2.add(jcbAutoExecute);
         box2.add(jcbAutoNotify);
         box2.add(jbExploreWait);
         jbExploreWait.addActionListener(new java.awt.event.ActionListener() {
           public void actionPerformed(ActionEvent e) {
             jbExploreWait_actionPerformed(e);
           }
         });
         
         

         jpAutoOptions.add(box2);
         box1.add(jpAutoOptions);

         this.add(box1);

         //System.out.println("Found default DCM  Server :" + mUser.getDefDCMsrv().toString());

         //
         // Auto DCM send stuff
         //
         if (mUser.getAutoDCMSendFlag()) {
           //System.out.println("Auto send is on ");
           jcbAutoSend.setSelected(true);
         }
         else {
           //System.out.println("Auto send is off ");
           jcbAutoSend.setSelected(false);
         }

         // comment if debug GUI
         int index=0;
         String defServName=mUser.getDefDCMsrv().toString();
         //System.out.println("In Combobox Server :" + jcbDCMServers.getItemCount());

         for (int i=0; i<jcbDCMServers.getItemCount(); i++ ){
           String name=jcbDCMServers.getItemAt(i).toString();
           // find appropirate index
           //System.out.println("Candidate :" + i + "  "+name + " compared with " + defServName);
           if ( name.compareTo(defServName)==0 ){
             index=i;
             //System.out.println("Selected Index :" + index);
             jcbDCMServers.setSelectedIndex(index);
           } else {  // doing nothing
             //System.out.println(" .... not equal");
           }
         }
         //this.updateUI();
         //jcbDCMServers.setSelectedItem(mUser.getDefDCMsrv());  // this method does not properly !!!
         //System.out.println("Test Server :" + mUser.getDefDCMsrv().toString());
         jcbDCMServers.addActionListener(new java.awt.event.ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                      jcbDCMServers_actionPerformed(e);
                    }
         });
         //System.out.println("GUI done");
    }

    //
    // Auto send files to DICOM server selected
    //
    void jcbAutoSend_actionPerformed(ActionEvent e){
      if (jcbAutoSend.isSelected()){
        //System.out.println("jcbAutoSend_  AutoSend is selected ...");
        dsDcmServerClass sel = (dsDcmServerClass) jcbDCMServers.getSelectedItem();
        mUser.setDefDCMsrv(sel);
        //
        // Start auto send to DICOM
        //
        mUser.setAutoDCMSendFlag(true);  // activate auto send
        DBridgeClient.Bank().flush();    // save Bank

       //try{
       //  Thread.sleep(10);
       //}catch(InterruptedException ex){}
        // send a requesr to the server to start a service for the user
        int index = DBridgeClient.Bank().getUserContainer().getUserIndex(mUser);
        DBridgeClient.sendRequest("DBSTARTAUTODCMSEND$" +
                                  Integer.toString(index));

//int ind1 = DBridgeClient.Bank().getUserContainer().getUserIndex(mUser);
//String user =  DBridgeClient.Bank().getUserContainer().getUser(ind1).getDefDCMsrv().toString();
//System.out.println(user);


      }else{//System.out.println("jcbAutoSend_ is not selected ...");
        //
        // Stop Auto send to DICOM
        //
        mUser.setAutoDCMSendFlag(false);
        DBridgeClient.Bank().flush();
        int index = DBridgeClient.Bank().getUserContainer().getUserIndex(mUser);
        DBridgeClient.sendRequest("DBSTOPAUTODCMSEND$" + Integer.toString(index));

      }
    }

    //
    // Auto open files check box selected
    //

    void jcbAutoExecute_actionPerformed(ActionEvent e){
      //System.out.println("checkbox is selected ...");
      if (jcbAutoExecute.isSelected()){//System.out.println("xxx is selected ...");
        if (mAppExecutor != null) {
          if (!mAppExecutor.HasUserTask(mUser)) {
            mAppExecutor.AddUserTask(mUser);
          }
        }
      }else{//System.out.println("xxx is not selected ...");
        if (mAppExecutor != null) {
          if (mAppExecutor.HasUserTask(mUser)) {
            mAppExecutor.Terminate();
            mAppExecutor.RemoveUserTask(mUser);
          }
        }
      }
    }
    
    void jcbAutoNotify_actionPerformed(ActionEvent e){
    	if (jcbAutoNotify.isSelected()){
    	      
    		  mUser.setAutoNotifyFlag(true);  // activate auto send
    		  DBridgeClient.Bank().flush();    // save Bank

       	        // send a requesr to the server to start a service for the user
    	        int index = DBridgeClient.Bank().getUserContainer().getUserIndex(mUser);
    	        DBridgeClient.sendRequest("DBUPDATEUSER$" +
    	                                  Integer.toString(index));
    	}else{
            mUser.setAutoNotifyFlag(false);
            DBridgeClient.Bank().flush();
            int index = DBridgeClient.Bank().getUserContainer().getUserIndex(mUser);
            DBridgeClient.sendRequest("DBUPDATEUSER$" + Integer.toString(index));
    	}
    }


    //
    // Explore Wait folder
    //

    void jbExploreWait_actionPerformed(ActionEvent e){
      JFileChooser jFileChooser1 = new JFileChooser();
      jFileChooser1.setCurrentDirectory(new File(mUser.getWaitDirPath()));
      jFileChooser1.setMultiSelectionEnabled(false);

      if (JFileChooser.APPROVE_OPTION == jFileChooser1.showDialog(this, "Select")) {
        //Execute the selected file if ext supported
        File f_sel = jFileChooser1.getSelectedFile();
        dsXMLAppTable m_TableIO = new dsXMLAppTable();
        Vector mTableData = m_TableIO.ReadTableData(msUserHome);
        AppFileProcessor mProcessor = new AppFileProcessor(mTableData, mUser);
        mProcessor.processFile(f_sel.getPath());
      }
    }

    //
    //   Select Auto send DICOM server
    //

    void jcbDCMServers_actionPerformed(ActionEvent e){
      //System.out.println("Action performed is selected ...");
      //System.out.println("get selected item ..." + jcbDCMServers.getSelectedItem().toString());
      mUser.setDefDCMsrv((dsDcmServerClass) jcbDCMServers.getSelectedItem());

      this.jcbAutoSend.setSelected(false);
      this.jcbAutoSend_actionPerformed(null);
    }

    public String toString(){
      return sTitle;
    }
  }
  //========================================================================

  public void updateAppExecService(){
    dsXMLAppTable m_TableIO = new dsXMLAppTable();
    Vector mTableData = m_TableIO.ReadTableData(msUserHome);
    mAppExecutor.setNewData(mTableData);
    if (mAppExecutor.HasUserTask(mUser)) {
      mAppExecutor.RemoveUserTask(mUser);
      mAppExecutor.AddUserTask(mUser);
    }
  }

// Constructor
  public dsUserInfoTree_1(dbUser User_data, String root_dir) throws HeadlessException {

    mUser = User_data;
    msRootDir =root_dir;
    setIconImage(Toolkit.getDefaultToolkit().getImage(DBConsts.WINDOW_ICON_FILENAME));
    try {
      // RELEASE
      Locale currentLocale = BridgeApp.getCurrentLocale();

      // DEBUG
      //Locale currentLocale = new Locale("en");

      lbls = ResourceBundle.getBundle("dbridge.prop.dsUserInfoFrameBundle",
                                      currentLocale);

      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  // create tree nodes
  private void createNodes(DefaultMutableTreeNode top) {

   DefaultMutableTreeNode sendMail_node = null;
   DefaultMutableTreeNode sendDCM_node = null;
   DefaultMutableTreeNode autoOpen_node = null;
   DefaultMutableTreeNode POP_node = null;
   DefaultMutableTreeNode addr_node = null;

   //sendMail_node = new DefaultMutableTreeNode(mEmailSendFrame);
   sendMail_node = new DefaultMutableTreeNode(new dsTreeLeaf(lbls.getString("sendEmail"), null));
   top.add(sendMail_node);

   //sendDCM_node = new DefaultMutableTreeNode(mDCMSendFrame);
   sendDCM_node = new DefaultMutableTreeNode(new dsTreeLeaf(lbls.getString("sendDICOM"), null));
   top.add(sendDCM_node);

   //autoOpen_node = new DefaultMutableTreeNode(mAppChooserDlg);
   autoOpen_node = new DefaultMutableTreeNode(new dsTreeLeaf(lbls.getString("AutoExec"), null));
   top.add(autoOpen_node);

   POP_node = new DefaultMutableTreeNode(new dsTreeLeaf(lbls.getString("POP"), null));
   top.add(POP_node);

   addr_node = new DefaultMutableTreeNode(new dsTreeLeaf(lbls.getString("addrBook"), null));
   top.add(addr_node);
}

 public void valueChanged(TreeSelectionEvent e) {
   DefaultMutableTreeNode node = (DefaultMutableTreeNode)
                          jTree1.getLastSelectedPathComponent();

    if (node == null) return;

    dsTreeLeaf nodeInfo = (dsTreeLeaf) node.getUserObject();
    if (!nodeInfo.isInitiated()){
      Container m_pan1;
      if ((nodeInfo.toString().compareTo(lbls.getString("Main")))==0){
        dsUserGeneralInfo InfoPanel = new dsUserGeneralInfo(lbls.getString("Main"));
        m_pan1 = InfoPanel;
      }
      else if ( (nodeInfo.toString().compareTo(lbls.getString("sendEmail"))) == 0) {
         mEmailSendFrame = new dsUserDlg1Frame(lbls.
                                               getString("sendEmail"), mUser);
         mEmailSendFrame.setPanelMode();
         m_pan1 = mEmailSendFrame.getContentPane();
       }
      else if ( (nodeInfo.toString().compareTo(lbls.getString("sendDICOM"))) == 0) {
         mDCMSendFrame = new dsUserDlg2Frame(lbls.
                                             getString("sendDICOM"), mUser);
         mDCMSendFrame.setDCMServers(DBridgeClient.Bank().getDcmServerList());
         mDCMSendFrame.setPanelMode();
         m_pan1 = mDCMSendFrame.getContentPane();
      }
      else if ( (nodeInfo.toString().compareTo(lbls.getString("AutoExec"))) == 0) {
          mAppChooserDlg = new dsAppChooserDlg(lbls.getString("AutoExec"), this, false);
          mAppChooserDlg.setPanelMode();
          mAppChooserDlg.setFilename(msUserHome);
          mAppChooserDlg.setAppExecServiceHolder(this);
          m_pan1 = mAppChooserDlg.getContentPane();
      }
      else if ( (nodeInfo.toString().compareTo(lbls.getString("POP"))) == 0) {

        mPOPPan = new dsPOPPanel(this);
        mPOPlistener = new dsPOPPanelActionListener(mPOPPan);
        mPOPPan.setActionListener(mPOPlistener);
        mPOPPan.setObject(mUser.getMailTable());

        m_pan1 = mPOPPan;
      }
      else if ( (nodeInfo.toString().compareTo(lbls.getString("addrBook"))) == 0) {
        mADDRPan = new dsADDRPanel(this);
        mADDRPan.setActionListener(new dsADDRPanelActionListener(mADDRPan));
        mADDRPan.setObject(mUser);
        m_pan1 = mADDRPan;
      }
      else
      {
          m_pan1 = new Container();
      }

      jScrollPane2.getViewport().removeAll();
      jScrollPane2.getViewport().add(m_pan1);
      jScrollPane2.getViewport().updateUI();

     nodeInfo.setContainer(m_pan1);

    }
    else{
      Container m_pan1 = nodeInfo.getContainer();
      jScrollPane2.getViewport().removeAll();
      jScrollPane2.getViewport().add(m_pan1);
      jScrollPane2.getViewport().updateUI();
    }
 }

// DEBUG
  public static void main(String[] args) throws HeadlessException {
    dsUserInfoTree_1 test1 = new dsUserInfoTree_1(new dbUser(), "");
    test1.setVisible(true);
  }
// END DEBUG

  private void jbInit() throws Exception {

    DBridgeClient.setIsUserGUIActivated(true);

    this.setSize(new Dimension(800, 600));
    this.setTitle(lbls.getString("FormTitle"));
    this.addWindowListener(new dsUserInfoTree_1_this_windowAdapter(this));

    jPanel1.setBorder(BorderFactory.createEtchedBorder());
    jPanel1.setMinimumSize(new Dimension(200, 200));
    jPanel1.setPreferredSize(new Dimension(200, 200));
    jPanel1.setLayout(borderLayout1);
    jPanel2.setLayout(borderLayout2);
    jPanel2.setBorder(BorderFactory.createEtchedBorder());
    jPanel3.setBorder(BorderFactory.createEtchedBorder());
    jbHelp.setText(lbls.getString("bHelp"));
    jbHelp.setIcon(new ImageIcon("./lib/res/opendoc.gif"));
    jbHelp.addActionListener(new dsUserInfoTree1_jbHelp_actionAdapter(this));
    jbOk.setText(lbls.getString("bClose"));
    jbOk.setIcon(new ImageIcon("./lib/res/thumbdown.gif"));
    jbOk.addActionListener(new dsUserInfoTree1_jbOk_actionAdapter(this));

    jbRestartServer.setText(lbls.getString("bUpdate"));
    jbRestartServer.setIcon(new ImageIcon("./lib/res/thumbright.gif"));
    jbRestartServer.addActionListener(new dsUserInfoTree1_jbRestart_actionAdapter(this));
    this.getContentPane().add(jPanel1, BorderLayout.WEST);
    jPanel1.add(jScrollPane1, BorderLayout.CENTER);
    this.getContentPane().add(jPanel2,  BorderLayout.CENTER);
    jPanel2.add(jScrollPane2,  BorderLayout.CENTER);
    this.getContentPane().add(jPanel3,  BorderLayout.SOUTH);
    jPanel3.add(jbOk, null);
    jPanel3.add(jbRestartServer, null);
    jPanel3.add(jbHelp, null);

//Create AppExecutor service for this user
    dsXMLAppTable m_TableIO = new dsXMLAppTable();

    msUserHome = new String(DBridgeClient.Bank().getBox().getTmpDir() + File.separator +
                            mUser.getLogin() + File.separator +
                            "applications.xml");

    Vector mTableData = m_TableIO.ReadTableData(msUserHome);

    mAppExecutor = new CAppExecutorService(mTableData);


    //
    // Create tree components GUI
    //

    mInfoPanel = new dsUserGeneralInfo(lbls.getString("Main"));
    DefaultMutableTreeNode top = new DefaultMutableTreeNode(new dsTreeLeaf(lbls.getString("Main"), mInfoPanel));

    //
    // Assign the components GUI to the tree structure
    //
    this.createNodes(top);

    jTree1 = new JTree(top);
    jTree1.getSelectionModel().setSelectionMode
            (TreeSelectionModel.SINGLE_TREE_SELECTION);

        //Listen for when the selection changes.
        jTree1.addTreeSelectionListener(this);
        jTree1.setSelectionRow(0);

    jScrollPane1.getViewport().add(jTree1, null);

    if (msRootDir == null || msRootDir == "") {
      JOptionPane.showMessageDialog(this,
                                    lbls.getString("RootDirProblem"));
      return;
    }

    String Loc_folder = mUser.getLocDirPath();
    // Modification 28.03.07 Ext is only for temporary storing
    // The messages and non-dicom files are stored in Wait
    //String Ext_folder = mUser.getExtDirPath();
    String Ext_folder = mUser.getWaitDirPath();
    String Decr_folder = mUser.getDecryptedDirPath();

    Dir_loc = new File(Loc_folder);
    Dir_ext = new File(Ext_folder);
    Dir_decr = new File(Decr_folder);

    if (!Dir_loc.exists() || !Dir_ext.exists() || !Dir_decr.exists()) {
      JOptionPane.showMessageDialog(this,
                                    lbls.getString("UserDirProblem"));
      return;
    }

    num_files_loc = Integer.toString(0);
    num_files_ext = Integer.toString(0);
    num_files_decr = Integer.toString(0);

    // start checking directories
    int timeout=5;  // TODO use constant
    mCheckExt = new CGuiFolderChecker(Dir_ext.getAbsolutePath(), this);
    mCheckExt.StartChecking(timeout);
    mCheckDecr = new CGuiFolderChecker(Dir_decr.getAbsolutePath(), this);
    mCheckDecr.StartChecking(timeout);
    mCheckLoc = new CGuiFolderChecker(Dir_loc.getAbsolutePath(), this);
    mCheckLoc.StartChecking(timeout);

}
  //------------------------------------------------------------------------

//  protected void processWindowEvent(WindowEvent e) {
//  super.processWindowEvent(e);
//  if (e.getID() == WindowEvent.WINDOW_CLOSING) {
    //Change for final version
//    this.setVisible(false);
//  }
//}
//------------------------------------------------------------------------

public void UpdateFiles(Vector vFiles, String sFolder) {
  if (sFolder.compareTo(Dir_loc.getAbsolutePath()) == 0) {
    num_files_loc = Integer.toString(vFiles.size());
    this.mInfoPanel.jlDcm_files.setText(lbls_gen.getString("HaveFiles") + " " +
                                        num_files_loc);
  }
// 28.03.07
//  else if (sFolder.compareTo(Dir_decr.getAbsolutePath()) == 0) {
//    num_files_decr = Integer.toString(vFiles.size());
//    this.mInfoPanel.jlEmail_files.setText(lbls_gen.getString("HaveFiles") +
//                                          " " +
//                                          num_files_decr);
    else if (sFolder.compareTo(Dir_ext.getAbsolutePath()) == 0) {
        num_files_decr = Integer.toString(vFiles.size());
        this.mInfoPanel.jlEmail_files.setText(lbls_gen.getString("HaveFiles") +
                                              " " +
                                              num_files_decr);  
    
  }

}

  void jbOk_actionPerformed(ActionEvent e) {
    this.this_windowClosed(null);
  }

  void jbHelp_actionPerformed(ActionEvent e) {
    URL index = ClassLoader.getSystemResource(lbls.getString("HlpURL"));
      if ( index!=null )
        new XHelp(lbls.getString("HlpTitle"), index);
        //new XHelp("DBridge User Help"+DBridgeClient.Bank().getVersion(), index);
  }


  void jbRestartServices_actionPerformed(ActionEvent e){

    if (mUserChanged) {
      DBridgeClient.Bank().flush();
      DBridgeClient.sendRequest("DBRESTART");

      JOptionPane.showMessageDialog(this, lbls.getString("db_changed"),
                                    lbls.getString("info"),
                                    JOptionPane.PLAIN_MESSAGE);

    }

  }

  void this_windowClosed(WindowEvent e) {

    DBridgeClient.setIsUserGUIActivated(false);
    if (mInfoPanel!=null) mInfoPanel.invalidate();
    if (mEmailSendFrame!=null) mEmailSendFrame.invalidate();
    if (mDCMSendFrame!=null) mDCMSendFrame.invalidate();
    if (mAppChooserDlg!=null) mAppChooserDlg.invalidate();

    this.setVisible(false);
    this.invalidate();

  }

}

class dsUserInfoTree1_jbOk_actionAdapter implements java.awt.event.ActionListener {
  dsUserInfoTree_1 adaptee;

  dsUserInfoTree1_jbOk_actionAdapter(dsUserInfoTree_1 adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.jbOk_actionPerformed(e);
  }
}

class dsUserInfoTree1_jbHelp_actionAdapter implements java.awt.event.ActionListener {
  dsUserInfoTree_1 adaptee;

  dsUserInfoTree1_jbHelp_actionAdapter(dsUserInfoTree_1 adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.jbHelp_actionPerformed(e);
  }
}

class dsUserInfoTree1_jbRestart_actionAdapter implements java.awt.event.ActionListener {
  dsUserInfoTree_1 adaptee;

  dsUserInfoTree1_jbRestart_actionAdapter(dsUserInfoTree_1 adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.jbRestartServices_actionPerformed(e);
  }
}


class dsUserInfoTree_1_this_windowAdapter extends java.awt.event.WindowAdapter {
  dsUserInfoTree_1 adaptee;

  dsUserInfoTree_1_this_windowAdapter(dsUserInfoTree_1 adaptee) {
    this.adaptee = adaptee;
  }
  public void windowClosing(WindowEvent e) {
    adaptee.this_windowClosed(e);
  }
}
