package dbridge;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.dom.DOMSource;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import javax.xml.transform.stream.StreamResult;

import org.xml.sax.SAXException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import java.io.File;
import java.io.IOException;
import java.util.Vector;

/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: </p>
 * @author bob
 * @version 1.0
 */

//===========================================================================
//                   dsXMLparser
//            Base Class to parse XML files
//===========================================================================

class dsXMLparser {

  protected Document m_doc;

  //=========================================================================
  public dsXMLparser() {
    try {
      jbInit();
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  //-------------------------------------------------------------------------
  protected Document CreateDOM() {
    Document result;
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    try {
      DocumentBuilder builder = factory.newDocumentBuilder();
      result = builder.newDocument();
    }
    catch (ParserConfigurationException pce) {
      // Parser with specified options can't be built
      //pce.printStackTrace();
      throw new RuntimeException("Can not create XML Document: " +
                                 pce.getMessage());
    }

    return result;
  }

  //-------------------------------------------------------------------------
  public Document ReadFile(String filename) {
    Document result;
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();

    File mFile = new File(filename);
    if (!mFile.exists()) {
      try {
        mFile.createNewFile();
      }
      catch (IOException ioe) {
        throw new RuntimeException("File I/O error:" + ioe.getMessage());
      }
      result = null;
    }
    else if (mFile.length()<10){result = null;}
    else{

      try {
        DocumentBuilder builder = factory.newDocumentBuilder();
        result = builder.parse(new File(filename));
      }
      catch (ParserConfigurationException pce) {
        throw new RuntimeException("Can not configure XML parser: " +
                                   pce.getMessage());
      }
      catch (SAXException sxe) {
        throw new RuntimeException("Parsing error:" + sxe.getMessage());
      }
      catch (IOException ioe) {
        throw new RuntimeException("File I/O error:" + ioe.getMessage());
      }
    }
    return result;
  }

  //-------------------------------------------------------------------------
  public void WriteFile(Document xml_doc, String filename) {
    try {
      Transformer transformer = TransformerFactory.newInstance().newTransformer();
      transformer.transform(new DOMSource(xml_doc),
                            new StreamResult(new File(filename)));
    }
    catch (TransformerConfigurationException e) {
      //System.out.println("There's a Transformer Config Excpt");
      throw new RuntimeException("Can not save XML Document: " + e.getMessage());
    }
    catch (TransformerException e) {
      //System.out.println("There is a Transformer Exception");
      throw new RuntimeException("Can not save XML Document: " + e.getMessage());
    }
  }

  //-------------------------------------------------------------------------
   public Node findSubNode(String name, Node node) {
    if (node.getNodeType() != Node.ELEMENT_NODE) {
      System.err.println("Error: Search node not of element type");
      System.exit(22);
    }

    if (!node.hasChildNodes()) {
      return null;
    }

    NodeList list = node.getChildNodes();
    for (int i = 0; i < list.getLength(); i++) {
      Node subnode = list.item(i);
      if (subnode.getNodeType() == Node.ELEMENT_NODE) {
        if (subnode.getNodeName() == name) {
          return subnode;
        }
      }
    }
    return null;
  }

//----------------------------------------------------------------------------
  public String getText(Node node) {
    StringBuffer result = new StringBuffer();
    if (!node.hasChildNodes()) {
      return "";
    }

    NodeList list = node.getChildNodes();
    for (int i = 0; i < list.getLength(); i++) {
      Node subnode = list.item(i);
      if (subnode.getNodeType() == Node.TEXT_NODE) {
        result.append(subnode.getNodeValue());
      }
      else if (subnode.getNodeType() ==
               Node.CDATA_SECTION_NODE) {
        result.append(subnode.getNodeValue());
      }
      else if (subnode.getNodeType() ==
               Node.ENTITY_REFERENCE_NODE) {
        // Recurse into the subtree for text
        // (and ignore comments)
        result.append(getText(subnode));
      }
    }
    return result.toString();
  }

//---------------------------------------------------------------------------

  public String getRoot(String pathFromFile) {
    String[] parsedPath = pathFromFile.split("[\\x2f\\x5c]");
    String restoredPath = parsedPath[0];
    for (int tokenCount = 1; tokenCount < parsedPath.length - 2; ++tokenCount) {
      restoredPath = restoredPath + File.separator + parsedPath[tokenCount];
    }
    restoredPath = restoredPath + File.separator;
    return restoredPath;
  }

  private void jbInit() throws Exception {
  }

  //-------------------------------------------------------------------------
}

//===========================================================================
//                   dsXMLAppTable
//        The class which converts JTable into XML and viseversa
//===========================================================================

class dsXMLAppTable
    extends dsXMLparser {

  //-------------------------------------------------------------------------
  public dsXMLAppTable() {
    super();

  }

  //-------------------------------------------------------------------------
  public void WriteTableData(Vector table_data, String filename) {
    Document document = CreateDOM();

    //Create thr root i.e. first level
    Element root =
        (Element) document.createElement("Applications");
    document.appendChild(root);

    for (int extCount = 0; extCount < table_data.size(); ++extCount) {
      Vector currNodeData = (Vector) table_data.get(extCount);
      if ( ( (String) currNodeData.get(0)).length() > 0 ||
          ( (String) currNodeData.get(1)).length() > 1) {
        Element ext =
            (Element) document.createElement("EXT");
        //Create second level
        root.appendChild(ext);
        ext.appendChild(document.createTextNode( (String) currNodeData.get(0)));

        Element path =
            (Element) document.createElement("Name");
        ext.appendChild(path);
        path.appendChild(document.createTextNode( (String) currNodeData.get(1)));

        Element opt =
            (Element) document.createElement("Options");
        ext.appendChild(opt);
        opt.appendChild(document.createTextNode( (String) currNodeData.get(2)));

        Element comments =
            (Element) document.createElement("Comments");
        ext.appendChild(comments);
        comments.appendChild(document.createTextNode( (String) currNodeData.get(
            3)));
      }
    }

    this.WriteFile(document, filename);
  }

  //--------------------------------------------------------------------------
  public Vector ReadTableData(String filename) {
    Vector result = new Vector();
    Element root;

    Document document = ReadFile(filename);
    if (document != null){
    root = document.getDocumentElement();

    for (int extCount = 0;
         extCount < root.getChildNodes().getLength(); ++extCount) {
      Node m_Ext = root.getChildNodes().item(extCount);
      Vector temp = new Vector();
      temp.add(getText(m_Ext));
      temp.add(getText(findSubNode("Name", m_Ext)));
      temp.add(getText(findSubNode("Options", m_Ext)));
      temp.add(getText(findSubNode("Comments", m_Ext)));
      result.add(temp);
    }
    }
    return result;
  }

}

//=============================================================================
