package dbridge;

/**
 * <p>Title: DicomBridge</p>
 * <p>Description: Transfer  Images over e-mail</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: a</p>
 * @author unascribed
 * @version 1.0
 */

/*
 * Modified 22.06.04
 * Modified 30.07.04
 * Modified 11.08.04
 * Modified 17.08.04
 * Modified 01.03.05
 * Modified 15.04.05
 * Modified 26.11.05
 * Modified 12.12.05
 * Modified 22.01.06
 * Modified 1-13.02.06
 */

import java.io.*;
import java.util.*;

import javax.mail.*;
import javax.mail.internet.*;
//import javax.activation.*;
import javax.swing.*;

import com.pixelmed.dicom.*;
import com.pixelmed.network.*;
import java.lang.RuntimeException;

//
// Interface for all files processors
//
class IFilesManager {
  public void ProcessFiles(Vector vFiles) {};
  public void Stop() {};
}

interface IProcessOwner {
  public void ProcessProgress(int i, int n);
  public void ProcessFinished(int vFailed, String msg);
}

//****************************************************************************
// FilesManager which sends files on Dicom server
//
class CDicomFilesSender
    extends IFilesManager {
  public CDicomFilesSender(String sExePath, String sOptions, String sServer, String sPort,
                           String sAeTitleC) {
    mExePath = sExePath;
    mOptions = sOptions;
    mServer = sServer;
    mPort = sPort;
    mAeTitleC = sAeTitleC;
    mAeTitle = "DBridgeSCP"; //ooo 7.07.08
    mDeleteFilesFlag = true;
    mIntSenderFlag = false;
    nIntSenderNotAble=false;
  }

  public CDicomFilesSender(dbBank oBank, dbUser oUser) {
    //
    // Configuring base Command string
    //
    // mExePath = ".\\bin\\storescu";
    //mExePath = DBridgeClient.Bank().getBox().getDCMSender();
    mExePath = oBank.getBox().getDCMSender();
    mOptions = oBank.getBox().getDCMSenderOptions();
    mServer = oUser.getDefDCMsrv().getDcmServer();
    mPort = oUser.getDefDCMsrv().getDcmPort();
    mAeTitleC = oUser.getDefDCMsrv().getDcmAeTitle();
    mAeTitle = oBank.getBox().getAET(); //ooo 7.07.08
    mDeleteFilesFlag = true;
    mIntSenderFlag = oBank.getBox().getIntSender();
  }

  public void ProcessFiles(Vector vFiles) {
    BridgeApp.LogInfo(Logs.Srvcs.SetText("Files for Dicom send: " +
                                         vFiles.size()));
    // Debug
    BridgeApp.LogInfo(Logs.Srvcs.SetText("DCMSender ProcessFiles"));

    Vector vFailed = new Vector();
    int nSendTogether = DBConsts.DCM_FILES_SEND_TOGETHER; // 40 in former variant. change is does not work
    if ( mIntSenderFlag ){
      nSendTogether=1;             // internal sender will send each file separately
      nIntSenderNotAble=false;
    }
    int nFiles = vFiles.size();
    for (int i = 0; i < nFiles; i += nSendTogether) {
      if (mProcessOwner != null)
        mProcessOwner.ProcessProgress(i, nFiles);
        //
        // Fill current files vector
        //
      Vector vCurr = new Vector(nSendTogether);
      int nCurrTop = (nFiles > i + nSendTogether) ? i + nSendTogether :
          nFiles;
      for (int j = i; j < nCurrTop; j++)
        vCurr.add(vFiles.elementAt(j));

        //
        // Send current vector
        //
//System.out.println("*************************** Call SendFiles");
      if (!SendFiles(vCurr)) {
//System.out.println("*************************** Failed SendFiles");
        BridgeApp.LogError(Logs.Srvcs.SetText("Cannot send a block of "
                                              +vCurr.size()+ " dicom files"));
        for (int j = 0; j < vCurr.size(); j++)
          vFailed.add(vCurr.elementAt(j));
      }else {
//System.out.println("*************************** Successed SendFiles");
      }

      vCurr.clear();
    }

    if (mProcessOwner != null) {
      mProcessOwner.ProcessProgress(nFiles - 1, nFiles);
      String msg = new String();
      mProcessOwner.ProcessFinished(vFailed.size(), msg);
    }
    BridgeApp.LogInfo(Logs.Srvcs.SetText("Total file sends failed: " +
                                         vFailed.size()));
  }

  /**
   * Sends files fron an input array to a given DICOM sender. If mIntServerFlag
   * is true, then our internal sender will be used. Otherwise an external sender
   * (e.g. storescu) will be used. Returns true, only if only all files have been sent.
   * @param vFiles Vector
   * @return boolean
   */
  private boolean SendFiles(Vector vFiles) { // MAIN SENDER !!!
    boolean bProcessedOk = false;

    if ( mIntSenderFlag ){

      // Internal Sender
//System.out.println("Internal Sender");
      String filename=null;
      String mAET="astorescp";
      String mClassUID=null;
      String mInstanceUID=null;
      int sCmdCount = 0;


      for (int i = 0; i < vFiles.size(); ++i) {
        filename=(String)vFiles.elementAt(i);
        if ( nIntSenderNotAble ){
//System.out.println("!!!! Internal sender aborted for "+filename);
          return SendFile(filename);
        }

        BridgeApp.LogDebug(Logs.Srvcs.SetText("*** IntSender File="+filename));
        boolean can_send=true;
        boolean is_sent=true;
//System.out.println("i="+i+" filename="+filename);
        try
        {
          AttributeList list = new AttributeList();
          InputStream is = new FileInputStream(filename);
          list.read(new DicomInputStream(is));
//System.out.println("get tsuid");
          //String tsuid = (new DicomInputStream(is)).getTransferSyntaxInUse().getUID();
          is.close();
          // Read required data from the dicom file

          mClassUID = Attribute.getSingleStringValueOrDefault(list,
                    TagFromName.SOPClassUID, "");
          //System.out.println("mClassUID "+mClassUID);

          mInstanceUID = Attribute.getSingleStringValueOrDefault(list,
                    TagFromName.SOPInstanceUID, "");

          String tUID = Attribute.getSingleStringValueOrDefault(list,
                    TagFromName.TransferSyntaxUID, "");
//          System.out.println("### transferUID "+tUID);
//          System.out.println("Transfer System "+TransferSyntax.PixelMedEncapsulatedRawLittleEndian);
          if (tUID.equals(TransferSyntax.PixelMedEncapsulatedRawLittleEndian)) {
            BridgeApp.LogDebug(Logs.Srvcs.SetText(">>> ClassUID="+mClassUID+" InstanceUID="+mInstanceUID));
          } else {
//System.out.println("### unsupported");
            nIntSenderNotAble=true;
            if (SendFile(filename)){ // try to send by the external sender
              is_sent = true;
              can_send = false;
            }
          }


        }
        catch (Exception e)
        {
            // shit happens
            can_send=false;
            BridgeApp.LogError(Logs.Srvcs.SetText("Cannot extract DICOM attributes from the file "+filename));
            // ToDO copy into Unsupported
        }
       if ( can_send){
         try {
//System.out.println(">>> Sending Internaly");
           BridgeApp.LogInfo(Logs.Srvcs.SetText("IntSender sending "+filename+" Server="+mServer+" Port="+mPort));
           new StorageSOPClassSCU(mServer, mPort, mAeTitleC, mAET, filename,
                                  mClassUID, mInstanceUID, 0, 1);
//System.out.println(">>> Sending was finished");
           BridgeApp.LogInfo(Logs.Srvcs.SetText("IntSender sending was completed successfuly"));
         }
         catch (IOException ioe) {
           ioe.printStackTrace(System.err);
//System.out.println(">>> IO Error in access to the file " + filename);
           BridgeApp.LogDebug(Logs.Srvcs.SetText(">>> IO Error IntSender File="+filename+", JPEG decompression problem possible"));
           is_sent=false;
           nIntSenderNotAble=true;
           if (SendFile(filename)) // try to send by the external sender
             is_sent=true;
         }
         catch (DicomException de) {
           de.printStackTrace(System.err);
//System.out.println(">>> Dicom Error");
           BridgeApp.LogDebug(Logs.Srvcs.SetText(">>> DICOM Error IntSender File="+filename));
           // ToDO copy file into Unsupported directory
           is_sent=false;
         }
         catch (DicomNetworkException dne) {
           dne.printStackTrace(System.err);
//System.out.println(">>> Error in Dicom Network");
           BridgeApp.LogDebug(Logs.Srvcs.SetText(">>> DICOM Network Error IntSender File="+filename+", possibly cannot connect to peer"));
           is_sent=false;
         }
         catch (Exception e) {
           e.printStackTrace(System.err);
//System.out.println(">>> Unknown Error");
           BridgeApp.LogDebug(Logs.Srvcs.SetText(">>> Unknown Error IntSender File="+filename));
           is_sent=false;
         }
       }// ens if can_send
//System.out.println(">>> Analyse is_sent variable");
       if ( is_sent ) {
           sCmdCount++;
           if (mDeleteFilesFlag) {
            (new File(filename)).delete();
            BridgeApp.LogDebug(Logs.Srvcs.SetText(">>> Delete IntSender File="+filename));
           }
       }
      }// end for
      // TODO store a number of failed sends in a class variable
//System.out.println(">>> sCmdCount="+sCmdCount+" vFiles.size()="+vFiles.size());
      if ( sCmdCount==vFiles.size() ){
        //System.out.println(">>> Return true");
        return true;
      }
      else{
        //System.out.println(">>> Return false");
        return false;
      }
    }
    /*
    * SEND TO DICOM SERVER USING THE EXTERNAL SENDER
    */
    // String sCmd [];
    // {mExePath, "-aec", mAeTitle, mServer, mPort};
    String sFiles = new String();
    int arrSize = vFiles.size() + DBConsts.DCM_PARAMS_SEND;
    String sCmd[] = new String[arrSize];
    int sCmdCount = 0;
    // DBConsts.DCM_PARAMS_SEND=8 parameters
    sCmd[sCmdCount++] = mExePath;
    sCmd[sCmdCount++] = mOptions;
    sCmd[sCmdCount++] = "-aec"; // !!!!!! NO CONSTANTS!!!!!!
    sCmd[sCmdCount++] = mAeTitleC;
    sCmd[sCmdCount++] = "-aet"; // !!!!!! NO CONSTANTS!!!!!!
    sCmd[sCmdCount++] = mAeTitle;
    sCmd[sCmdCount++] = mServer;
    sCmd[sCmdCount++] = mPort;
    for (int i = 0; i < vFiles.size(); i++) {
      sCmd[sCmdCount++] = (String) vFiles.elementAt(i);
    }

    String sLogCmd = new String("DCMSender Command line: ");
    for (int i = 0; i < sCmd.length; ++i) {
      sLogCmd += sCmd[i] + " ";
    }
    BridgeApp.LogInfo(Logs.Srvcs.SetText(sLogCmd));

    try {
      Process p = Runtime.getRuntime().exec(sCmd);
      // Strange?! but required
      InputStreamReader in = new InputStreamReader(p.getInputStream());
      in.close();

      int res = 0;
      if ( (res = p.waitFor()) == 0) {
        bProcessedOk = true;
      }
      else {
        BridgeApp.LogError(Logs.Srvcs.SetText("DCMSender Process failed: " +
                                                res));
      }
      if (bProcessedOk) {
        // delete all files if necessary
        if (mDeleteFilesFlag) {
          for (int i = 0; i < vFiles.size(); i++) {
            String sFileName = (String) vFiles.elementAt(i);
            (new File(sFileName)).delete();
          }
        }
      }
    }
    catch (Exception ex) {
      BridgeApp.LogError(Logs.Srvcs.SetText("Exception: " + ex));
    }
    return bProcessedOk;
  }

  /**
 * Sends one file that was failed to be sent by the internal sender.
 * Returns true, only if only all the file have been sent.
 * @param filename String
 * @return boolean
 */
private boolean SendFile(String filename) { // Emmergency sender !!!
  boolean bProcessedOk = false;
//System.out.println("!!!! Send only one file");
  /*
  * SEND ONLY ONE FILE TO DICOM SERVER USING EXTERNAL SENDER
  */

  int arrSize = 1 + DBConsts.DCM_PARAMS_SEND;
  String sCmd[] = new String[arrSize];
  int sCmdCount = 0;
  sCmd[sCmdCount++] = mExePath;
  sCmd[sCmdCount++] = mOptions;
  sCmd[sCmdCount++] = "-aec"; // !!!!!! NO CONSTANTS!!!!!!
  sCmd[sCmdCount++] = mAeTitleC;
  sCmd[sCmdCount++] = "-aet"; // !!!!!! NO CONSTANTS!!!!!!
  sCmd[sCmdCount++] = mAeTitle;
  sCmd[sCmdCount++] = mServer;
  sCmd[sCmdCount++] = mPort;
  sCmd[sCmdCount++] = (String) filename;


  String sLogCmd = new String("DCMSender Command line: ");
  for (int i = 0; i < sCmd.length; ++i) {
    sLogCmd += sCmd[i] + " ";
  }
  BridgeApp.LogInfo(Logs.Srvcs.SetText(sLogCmd));
//System.out.println(sLogCmd);
  try {
    Process p = Runtime.getRuntime().exec(sCmd);
    // Strange?! but required
    InputStreamReader in = new InputStreamReader(p.getInputStream());
    in.close();

    int res = 0;
    if ( (res = p.waitFor()) == 0) {
      bProcessedOk = true;
    }
    else {
      BridgeApp.LogError(Logs.Srvcs.SetText("DCMSender Process failed: " +
                                              res));
//System.out.println("!!!! Emergency DCMSender failed");
    }
  }
  catch (Exception ex) {
    BridgeApp.LogError(Logs.Srvcs.SetText("Emergency DCMSender Exception: " + ex));
  }
  return bProcessedOk;
}


  public void SetProcessOwner(IProcessOwner o) {
    mProcessOwner = o;
  }

  public void SetServer(String sServer) {
    mServer = sServer;
  }

  public void SetPort(String sPort) {
    mPort = sPort;
  }

  public void SetAeTitleC(String sAeTitle) {
    mAeTitleC = sAeTitle;
  }
  
  public void SetAeTitleBox(String sAeTitle) {
	    mAeTitle = sAeTitle;
	  }

  public void SetDeleteAfterSend(boolean bDeleteFiles) {
    mDeleteFilesFlag = bDeleteFiles;
  }

  public void SetIntSenderFlag(boolean bFlag) {
    mIntSenderFlag = bFlag;
  }

  private String mExePath;
  private String mOptions;
  private String mServer;
  private String mPort;
  private String mAeTitleC;
  private String mAeTitle;
  private boolean mDeleteFilesFlag;
  private boolean mIntSenderFlag;
  private boolean nIntSenderNotAble;   // frue if first sending is failed

  private IProcessOwner mProcessOwner = null;
}

//***************************************************************************
// FilesManager which decrypts and sends files by smtp
//
class CSmtpFilesSender
    extends IFilesManager {
  private dbUser mUser = null;
  private boolean mFinishStatusAllowed = true;
  private int m_failed_num = 0;
  private boolean m_stop = false;

//  private vector mSenders;

  public CSmtpFilesSender(dbUser oUser) {
    mUser = oUser;
    mDeleteFilesFlag = true;
    mEraseNamesFlag = false;
    // added 22.07.05
    //mXTag= new XTag(""); // check this

    BridgeApp.LogDebug(Logs.Srvcs.SetText("--- CSmtpFilesSender created ---- "));
    BridgeApp.LogDebug(Logs.Srvcs.SetText("USER="+mUser.toString()));

    //
    // Configuring SMTP sender for FilesManager
    //
    mSenders = new Vector();
    String From = "";
    Vector vRec = new Vector();

    if (oUser.getAddressBook().getSize() > 0) {
      dbMailRecord Rec = oUser.getAddressBook().getRecord(0);
      if ( oUser.isManual() ){ // form comment name (no for automatic)
        mComments = oUser.getAddressBook().getRecord(0).getComments();
      }
      vRec.add(Rec);
    }
    else {
      BridgeApp.LogWarning(Logs.Srvcs.SetText(
          "Address book is empty for this user !!!"));
    }
    this.setRecords(vRec);
    if (mComments == null) {
      mComments = "";
    }
  }

  public void allowFinishStatus(boolean flag){
    mFinishStatusAllowed = flag;
  }

  public void Stop(){
    m_stop = true;
  }


  // modification 13.02.06. To support MDN every message will be builded
  // in according with the MethodDN of the server. Therefore the message builder
  // is moved to the server loop
  public void ProcessFiles(Vector vFiles) {
    boolean bServersOk = true;
    Vector vSenders = (Vector) mSenders.clone();
    String sEmsg = new String();
    int total=0;                    // indicates a total number of files to send
    int part=0;                     // indicates a current number of a file to send

    if (m_stop) return;  // stop sending
//---------------------------------------------------------
    BridgeApp.LogDebug(Logs.Srvcs.SetText("Sending Comments with X-Tag: " + mXTag.toString() ));

    // Introduction the MDN mechanisms does not allow to build one message and
    // send it to all recipients as it was realized in the version 2.x. Therefore
    // one message will be generated per one recipient.
    // Algorithm:
    // Connect a recipient
    // Send a comment
    // Send all files
    // Change recipient
    // if it is a last recipient, copy all to the archive (if the delete flag is true)

    BridgeApp.LogDebug(Logs.Srvcs.SetText("Sending Files"));
    int nFiles = vFiles.size();
    Vector vFailed = new Vector();
    dbGUID transaction_id=new dbGUID();
    transaction_id.generateGUID();

    String separator=new File("").separator;
    String current_file=null;

    // DEBUG
    BridgeApp.LogDebug(Logs.Srvcs.SetText("Files number = "+nFiles));
    BridgeApp.LogDebug(Logs.Srvcs.SetText("Temporaty directory " +
                                          BridgeApp.getRoot()));

    String TmpCollectionName = mUser.getArchiveDirPath() +
        File.separator + transaction_id.getGUID();

    if ( mDeleteFilesFlag ) {
      File destdir = new File(TmpCollectionName);
      boolean created = destdir.mkdirs();
      if (!created){
         BridgeApp.LogError(Logs.Srvcs.SetText("Creating directory "+destdir + " : failed"));
         return;
      }else{
         BridgeApp.LogDebug(Logs.Srvcs.SetText("Creating directory "+destdir + " : OK"));
      }
    }
    //
    //    CONNECT
    //

    BridgeApp.LogDebug(Logs.Srvcs.SetText("--- ProcessFiles ---- "));
    BridgeApp.LogDebug(Logs.Srvcs.SetText("mSenders.size="+ mSenders.size()));
    int last_recipient=mSenders.size()-1;
    for (int i = 0; (i < mSenders.size()) && !m_stop; i++) {
      try {
        ( (Sender) vSenders.get(i)).connect();
      }
      catch (AuthenticationFailedException ae) {
        bServersOk = false;
        sEmsg += ( (Sender) vSenders.get(i)).getHost() +
            ":\nAuthentication Failed \n";
        vSenders.remove(i);
      }
      catch (Exception e) {
        bServersOk = false;
        String msg = e.getMessage();
        if (msg != null) {
          int indexOfEndLine = msg.indexOf('\n');
          if (indexOfEndLine != -1) {
            sEmsg += msg.substring(0, indexOfEndLine) + "\n";
          } else {
                sEmsg += msg + "\n";
          }
        }
        else {
          sEmsg += ( (Sender) vSenders.get(i)).getHost() +
              ":\nUnknown error\n";
        }
        vSenders.remove(i);
      }

      if ( bServersOk ){
          //..............Sending the Comment................
        CBaseMessageBuilder builder = null;
        //int iMethodSign = ( (Sender) vSenders.get(i)).getMethodSign();
        switch (mMethodSign[i]) {
          case DBConsts.PGPSIGN_RFC:
            builder = new CSignedEncryptedMessageBuilder(mComments, mXTag.toString(), mPubKeys[i]);
            builder.setSecretKey(mSecKeys[i]);
            builder.setPass(mPasses[i]);
            break;
          case DBConsts.PGPSIGN_PGP:
            builder = new CEncapsulatedMessageBuilder(mComments, mXTag.toString(), mPubKeys[i]);
            builder.setSecretKey(mSecKeys[i]);
            builder.setPass(mPasses[i]);
            break;
          default:
             builder = new CEncryptedMessageBuilder(mComments, mXTag.toString(), mPubKeys[i]);       
            break;
        }
        // set the DN method assigned for the recipient
        // Todo check the parameter in DBBank
        builder.setMethodDN( mMethodDN[i]/*( (Sender) vSenders.get(i)).getMethodDN()*/ );
        builder.setID( transaction_id.getGUID() );
        builder.setReplayToDN(mReplayTo[i]/*((Sender) vSenders.get(i)).getReplayTo()*/);
        builder.setKeyIDDN(mSecKeys[i]); 
        total=nFiles;
        builder.setTotal(total);
        part=0;
        if ( (mComments != null) && (mComments.length() != 0)) {
          try { // sending comments to a list of receipiens
            total++;
            builder.setTotal(total);
            part++;
            builder.setPart(part);
            MimeMessage msg = builder.BuildMessage("");
            
            String old_mid = msg.getMessageID();
            
            BridgeApp.LogDebug(Logs.Srvcs.SetText("The Comment Message was built succesfully"));
            ( (Sender) vSenders.get(i)).sendMail(msg);
            
            CBaseMessageBuilder.UpdateDBRecord(old_mid, msg.getMessageID());
            BridgeApp.LogDebug(Logs.Srvcs.SetText("The Comment Message sent for the sender (sendMail) = "+ i));
            //bProcessedOk = true;
          }
          catch (Exception e) {
            BridgeApp.LogError(Logs.Srvcs.SetText("Exception while Build or send to the server "+ i));
            BridgeApp.LogError(e);
            
            /*
                    TODO: add some reasonable code here...
             */
          }
          builder.ClearResources(); // ????
        }
        //...............................................

        //---------------------------------------------------------
        //
        //    SEND FILES
        //

        String TmpDirName = BridgeApp.getRoot() + FileTools.makeUniqName();
        BridgeApp.LogDebug(Logs.Srvcs.SetText("Tmp path: "+ TmpDirName));

        if (mEraseNamesFlag) {
            boolean tmpdirsuccess=(new File(TmpDirName)).mkdirs();
            if ( !tmpdirsuccess ){
              BridgeApp.LogError(Logs.Srvcs.SetText("Cannot create the directory: "+ TmpDirName));
            }else{
              BridgeApp.LogDebug(Logs.Srvcs.SetText("The tmp directory "+ TmpDirName+" has been created"));
            }
        }

        // reconnect conditions
        int last_before_reconnect=DBConsts.SMTP_RECONNECT_NFILES;
        Date start_time=new Date();

        //******************** FOR II ******************
        for (int ii = 0; (ii < nFiles) && !m_stop; ii++) {
          boolean bProcessedOk = false;
          String sFileName = (String) vFiles.elementAt(ii);
          String sFileOriginal = sFileName;
          part++;
          builder.setPart(part);

          // It is not allowed to send any files started with ".", ".."
          // Function FileTools.GetNotNiddenFilesFromDir(dir) takes care of it
          int j = 0;
          boolean dicom_noname_remove_flag=false;
          try {
              //added 07.04.2005
              if (FileTools.isDICMfile(new FileInputStream(sFileName))) {
                BridgeApp.Stats().SaveStatsForFile(sFileName, "send");
                BridgeApp.LogDebug(Logs.Srvcs.SetText("Detected as a DICOM"));
              }

              if ((mEraseNamesFlag) && (FileTools.isDICMfile(new FileInputStream(sFileName)))) {
                  String newFileName = TmpDirName + File.separator + (new File(sFileName)).getName();
                  BridgeApp.LogDebug(Logs.Srvcs.SetText("Delete user info in "+sFileName));
                  AttributeList list = new AttributeList();
                  list.read(sFileName, null, true, true);
                  list.replaceWithZeroLengthIfPresent(TagFromName.PatientName);
                  list.write(newFileName, null, true, true);
                  sFileName = newFileName;
                  dicom_noname_remove_flag=true;
              }
              BridgeApp.LogDebug(Logs.Srvcs.SetText("File to send "+sFileName));
//---------------------------------------------------------------------------------------------------------------------------------------
              
              
              MimeMessage msg = builder.BuildMessage(sFileName);
              String old_mid = msg.getMessageID();
              ( (Sender) vSenders.get(i)).sendMail(msg);
              
              CBaseMessageBuilder.UpdateDBRecord(old_mid, msg.getMessageID());
              
                
              bProcessedOk = true;
              // TODO TEST stats for file (SEND)
              // Moved
          } catch (Exception e) {
            e.printStackTrace();
            String msg = e.getMessage();
            if (msg != null) {
              int indexOfEndLine = msg.indexOf('\n');
              if (indexOfEndLine != -1)
                sEmsg += msg.substring(0, indexOfEndLine) + "\n";
              else
                sEmsg += msg + "\n";
            } else {
              sEmsg += ( (Sender) vSenders.get(j)).getHost() +
                  " Unknown error \n";
            }
            BridgeApp.LogError(Logs.Srvcs.SetText(sEmsg));
          }
          BridgeApp.LogDebug(Logs.Srvcs.SetText("Starting Removing temporary files"));
          builder.ClearResources(); //???
          if (dicom_noname_remove_flag) {
            boolean delete_dicom_success=(new File(sFileName)).delete();
            if ( !delete_dicom_success ){
               BridgeApp.LogError(Logs.Srvcs.SetText("Dicom file "+ sFileName + "cannot be removed"));
            }else{
               BridgeApp.LogDebug(Logs.Srvcs.SetText("Dicom file "+ sFileName + "has been removed"));
            }
            sFileName=sFileOriginal;       // restore the original file name
            System.out.println("Original file "+sFileName);
            dicom_noname_remove_flag = false;
          }
          //
          //  Delete files if needed
          //
          if (bProcessedOk && bServersOk) {
            if ( i==last_recipient){ // delete files after last sending
              if (mDeleteFilesFlag) {
                  // move this file
                  int index=sFileName.lastIndexOf(separator);  //  a file without separator is not possible
                  String dest=TmpCollectionName+sFileName.substring(index);       // with a separator
                  if ( sFileName.compareTo(dest)!=0 ){                            // if the file is not from Archive
                    File source = new File(sFileName);                            // File (or directory) to be moved
                    boolean success = source.renameTo(new File(dest));               // Move file to new directory
                    if (!success) {
                        // File was not successfully moved
                        BridgeApp.LogError(Logs.Srvcs.SetText("Moving file "+source +" to "+dest + " : failed"));
                        return;
                    }else{
                          // ToDo enter into database
                          BridgeApp.LogDebug(Logs.Srvcs.SetText("Moving file "+source +" to "+dest + " : OK"));
                    }
                  }
              } else { // add into database
                  //
              }// end else
            }// end i == last_recipient
          } else {
            vFailed.add(sFileName);
            BridgeApp.LogError(Logs.Srvcs.SetText(sEmsg));
          }
          BridgeApp.LogDebug(Logs.Srvcs.SetText("End Removing temporary files"));
//**
          if (mProcessOwner != null) {
            mProcessOwner.ProcessProgress(ii, nFiles+1); //add nFiles+1
          }

          //
          //    RECONNECT
          //
          Date now=new Date();
          long connect_time=start_time.getTime()-now.getTime();
          if ( ii==last_before_reconnect || connect_time >= DBConsts.SMTP_RECONNECT_TIME ){
              // reconnect to all servers
              BridgeApp.LogDebug(Logs.Srvcs.SetText("SMTP: Reconnect smtp servers after sending "+i+" messages"));
              if ( !( (Sender) mSenders.get(i)).reconnect() ){
                  BridgeApp.LogError(Logs.Srvcs.SetText("Reconnect failed to " +i +"smtp server"));
              }
              last_before_reconnect=ii+DBConsts.SMTP_RECONNECT_NFILES;
              now=new Date();
          }

        } //******************* end for ii *******************

        // Delete temorary dir for dicom files without user names
        if ( mEraseNamesFlag ) {
            (new File(TmpDirName)).delete();
        }

          BridgeApp.LogDebug(Logs.Srvcs.SetText("Failed="+vFailed.size()));
          //
          //    DISCONNECT
          //
          ( (Sender) mSenders.get(i)).disconnect();

      }// end if bServersOk
    }// end for vServers

    //
    //  Send final message
    //

          if (m_stop){mFinishStatusAllowed = true;}

          if (mProcessOwner != null) {
            if (mFinishStatusAllowed)  {
              int size = m_failed_num + vFailed.size();
              mProcessOwner.ProcessFinished(size, sEmsg);
              m_failed_num = 0;
            }else{
              m_failed_num += vFailed.size();
            }
          }

  }// end method

/*
  public void ProcessFiles(Vector vFiles) {
    boolean bServersOk = true;
    Vector vSenders = (Vector) mSenders.clone();
    String sEmsg = new String();

    //
    //    CONNECT
    //

    BridgeApp.LogDebug(Logs.Srvcs.SetText("--- ProcessFiles ---- "));
    BridgeApp.LogDebug(Logs.Srvcs.SetText("mSenders.size="+ mSenders.size()));

    for (int i = 0; (i < mSenders.size()) && !m_stop; i++) {
      try {
        ( (Sender) vSenders.get(i)).connect();
      }
      catch (AuthenticationFailedException ae) {
        bServersOk = false;
        sEmsg += ( (Sender) vSenders.get(i)).getHost() +
            ":\nAuthentication Failed \n";
        vSenders.remove(i);
      }
      catch (Exception e) {
        bServersOk = false;
        String msg = e.getMessage();
        if (msg != null) {
          int indexOfEndLine = msg.indexOf('\n');
          if (indexOfEndLine != -1) {
            sEmsg += msg.substring(0, indexOfEndLine) + "\n";
          } else {
          	sEmsg += msg + "\n";
          }
        }
        else {
          sEmsg += ( (Sender) vSenders.get(i)).getHost() +
              ":\nUnknown error\n";
        }
        vSenders.remove(i);
      }
    }

    if (m_stop) return;
//---------------------------------------------------------
    BridgeApp.LogDebug(Logs.Srvcs.SetText("Sending Comments with X-Tag: " + mXTag.toString() ));

    CMessageBuilder builder = new CMessageBuilder(mComments, mXTag.toString(),
                                                  mPubKeys);
    if ( (mComments != null) && (mComments.length() != 0)) {
//............................................

      int j = 0;
      try { // sending comments to a list of receipiens

        Message msg = builder.BuildMessage("");
        BridgeApp.LogDebug(Logs.Srvcs.SetText("The Message was built succesfully"));
        for (j = 0; j < vSenders.size(); j++) {
          ( (Sender) vSenders.get(j)).sendMail(msg);
          BridgeApp.LogDebug(Logs.Srvcs.SetText("The Message sent for the sender (sendMail) = "+ j));
        }
        //bProcessedOk = true;
      }
      catch (Exception e) {
        BridgeApp.LogDebug(Logs.Srvcs.SetText("Exception while Build or send!!!"+ j));

      }
      builder.ClearResources();
//............................................
    }
//---------------------------------------------------------
    //
    //    SEND FILES
    //

    BridgeApp.LogDebug(Logs.Srvcs.SetText("Sending Files"));
    int nFiles = vFiles.size();
    Vector vFailed = new Vector();

    BridgeApp.LogDebug(Logs.Srvcs.SetText("Files number = "+nFiles));
    //
    BridgeApp.LogDebug(Logs.Srvcs.SetText("Temporaty directory " +
                                          BridgeApp.getRoot()));
    //String TmpDirName = DBridgeClient.Bank().getBox().getTmpDir() + FileTools.makeUniqName();
    String TmpDirName = BridgeApp.getRoot() + FileTools.makeUniqName();
    BridgeApp.LogDebug(Logs.Srvcs.SetText("Tmp path: "+ TmpDirName));
    if (mEraseNamesFlag) {
      (new File(TmpDirName)).mkdirs();
    }

    // reconnect conditions
    int last_before_reconnect=DBConsts.SMTP_RECONNECT_NFILES;
    Date start_time=new Date();
    // ToDo: add also a time limit for reconnection
    for (int i = 0; (i < nFiles) && !m_stop; i++) {
      boolean bProcessedOk = false;

      String sFileName = (String) vFiles.elementAt(i);
      String sFileToDelite = sFileName;
      //BridgeApp.LogDebug(Logs.Srvcs.SetText("File path: "+ sFileName));
      // It is not allowed to send any files started from ., ..
      // Function FileTools.GetNotNiddenFilesFromDir(dir) takes care of it
      int j = 0;
      boolean dicom_noname_remove_flag=false;
      try {
//added 07.04.2005
        if (FileTools.isDICMfile(new FileInputStream(sFileName))) {
          BridgeApp.Stats().SaveStatsForFile(sFileName, "send");
          BridgeApp.LogDebug(Logs.Srvcs.SetText("Detected as a DICOM"));
        }

        if ((mEraseNamesFlag) && (FileTools.isDICMfile(new FileInputStream(sFileName)))) {
          //(new File(TmpDirName)).mkdirs();
          String newFileName = TmpDirName + File.separator + (new File(sFileName)).getName();
          BridgeApp.LogDebug(Logs.Srvcs.SetText("Delete user info in "+sFileName));
          AttributeList list = new AttributeList();
          list.read(sFileName, null, true, true);
          list.replaceWithZeroLengthIfPresent(TagFromName.PatientName);
          list.write(newFileName, null, true, true);
          sFileName = newFileName;
          dicom_noname_remove_flag=true;
        }
        BridgeApp.LogDebug(Logs.Srvcs.SetText("File to send "+sFileName));
//---------------------------------------------------------------------------------------------------------------------------------------
        Message msg = builder.BuildMessage(sFileName);
        for (j = 0; j < vSenders.size(); j++) {
          ( (Sender) vSenders.get(j)).sendMail(msg);
        }
        bProcessedOk = true;
        // TODO TEST stats for file (SEND)
        // Moved
        //if (FileTools.isDICMfile(new FileInputStream(sFileName))) {
        //  BridgeApp.Stats().SaveStatsForFile(sFileName, "send");
        //}
      }
      catch (Exception e) {
        e.printStackTrace();
        String msg = e.getMessage();
        if (msg != null) {
          int indexOfEndLine = msg.indexOf('\n');
          if (indexOfEndLine != -1)
          	sEmsg += msg.substring(0, indexOfEndLine) + "\n";
          else
          	sEmsg += msg + "\n";
        }
        else {
          sEmsg += ( (Sender) vSenders.get(j)).getHost() +
              " Unknown error \n";
        }
        BridgeApp.LogError(Logs.Srvcs.SetText(sEmsg));
      }
      BridgeApp.LogDebug(Logs.Srvcs.SetText("Starting Removing temporary files"));
      builder.ClearResources();
      if (dicom_noname_remove_flag) {
        (new File(sFileName)).delete();
        //(new File(TmpDirName)).delete();
        dicom_noname_remove_flag = false;
      }
      //
      //  Delete files if needed
      //
      if (bProcessedOk && bServersOk) {
        if (mDeleteFilesFlag) {
          (new File(sFileToDelite)).delete();
        }
      }
      else {
        vFailed.add(sFileName);
        BridgeApp.LogError(Logs.Srvcs.SetText(sEmsg));
      }
      BridgeApp.LogDebug(Logs.Srvcs.SetText("End Removing temporary files"));

      if (mProcessOwner != null) {
         mProcessOwner.ProcessProgress(i, nFiles);
      }

      //
      //    RECONNECT
      //
      Date now=new Date();
      long connect_time=start_time.getTime()-now.getTime();
      if ( i==last_before_reconnect || connect_time >= DBConsts.SMTP_RECONNECT_TIME ){
        // reconnect to all servers
        BridgeApp.LogDebug(Logs.Srvcs.SetText("SMTP: Reconnect smtp servers after sending "+i+" messages"));
        for (int ii = 0; ii < mSenders.size(); ii++) {
          if ( !( (Sender) mSenders.get(ii)).reconnect() ){
             BridgeApp.LogError(Logs.Srvcs.SetText("Reconnect failed to " +ii +"smtp server"));
          }
        }
        last_before_reconnect=i+DBConsts.SMTP_RECONNECT_NFILES;
        now=new Date();
      }


    } // end for i

    // Delete temorary dir for dicom files without user names
    if (mEraseNamesFlag) {
        (new File(TmpDirName)).delete();
    }


    BridgeApp.LogDebug(Logs.Srvcs.SetText("Failed="+vFailed.size()));
    //
    //    DISCONNECT
    //
    for (int i = 0; i < mSenders.size(); i++) {
      ( (Sender) mSenders.get(i)).disconnect();
    }
    //
    //  Send final message
    //

    if (m_stop){mFinishStatusAllowed = true;}

    if (mProcessOwner != null) {
      if (mFinishStatusAllowed)  {
        int size = m_failed_num + vFailed.size();
        mProcessOwner.ProcessFinished(size, sEmsg);
        m_failed_num = 0;
      }
      else{
        m_failed_num += vFailed.size();
      }
    }

  }
*/

  public void SetProcessOwner(IProcessOwner o) {
    mProcessOwner = o;
  }

  public void SetDeleteAfterSend(boolean bDeleteFiles) {
    mDeleteFilesFlag = bDeleteFiles;
  }

  // public void SetTo(String sTo)
  // {
  //   mSender.setTo(sTo);
  // }
  // public void SetFrom(String sFrom)
  // {
  //   mSender.setFrom(sFrom);
  //   }
  // public void SetPubKeyN(String sPubKeyN)
  // {
  //   mPubKeyN[0]=sPubKeyN;
  //  }
  public void SetComments(String sComments) {
    mComments = sComments;
  }

  // public void SetRecords (String [] aKeys) //BOB 28.12.2003
  // {
  //   mPubKeyN = aKeys;
  // }

  public void setRecords(Vector vMailRecords) {

    mPubKeys = new String[vMailRecords.size()];
    mMethodDN = new int[vMailRecords.size()];
    mMethodSign = new int[vMailRecords.size()];
    mSecKeys = new String[vMailRecords.size()];
    mPasses = new String[vMailRecords.size()];
    mReplayTo = new String[vMailRecords.size()];
    
    HashMap oIndMap = new HashMap();
    mSenders.clear();

    BridgeApp.LogDebug(Logs.Srvcs.SetText("--- FileManagers.setRecords ---- "));

    BridgeApp.LogInfo(Logs.Srvcs.SetText("RECORDS = " + vMailRecords.size()));

    for (int i = 0; i < vMailRecords.size(); i++) {
      int iSmtpIndex = ( (dbMailRecord) vMailRecords.elementAt(i)).
          getSrvIndex();
//----      
      dbSmtpRecord Rec = mUser.getSmtpTable().getRecord(iSmtpIndex);      
      boolean isStandardLogin = true;
      String sFrom=null;
      // 24.09.07 if a field From is not defined in the record
      // assemble the address as user@host
      if ( Rec.getFrom().compareTo("")==0 ){
    	  if ( Rec.getLogin().indexOf("@")>=0 ){
    		  isStandardLogin = false;
    		  sFrom = Rec.getLogin();
    	  } else {
    		  sFrom = Rec.getLogin() + "@" + Rec.getIP();
    	  }
      } else {
    	  sFrom = Rec.getFrom();
      }
      BridgeApp.LogDebug(Logs.Srvcs.SetText("Send From= " + sFrom ));
//----      
      Sender curr = (Sender) oIndMap.get(new Integer(iSmtpIndex));      
      if (curr == null) {
        curr = SenderFactory.newInstance();
        /*
        *  Some smtp servers require user accounts to be differ from the template login@server
        *  Therefore, if login consists of "@", the field "From" will be equal to "login",
        *  otherwise login@server
        */
        curr.setFrom(sFrom);
        // !IMPORTANT Conception. ReplayTo depents on the used SMTP servers
/*Andrei: Why this should go to Sender?
        String sReplayTo=Rec.getReplayTo();
        if ( sReplayTo.compareTo("")==0 ) sReplayTo=sFrom; // use the same addr as SMTP server
        BridgeApp.LogDebug(Logs.Srvcs.SetText("ReplayTo= " + sReplayTo ));
        curr.setReplayTo(sReplayTo);
*/
        BridgeApp.LogDebug(Logs.Srvcs.SetText("Send To= " + ( (dbMailRecord) vMailRecords.elementAt(i)).getMailDomainAddr()));
        curr.setTo( ( (dbMailRecord) vMailRecords.elementAt(i)).
                   getMailDomainAddr());
        BridgeApp.LogDebug(Logs.Srvcs.SetText("SMTP_HOST= "+Rec.getIP()));
        curr.setHost(Rec.getIP());
        BridgeApp.LogDebug(Logs.Srvcs.SetText("SMTP_Login= "+Rec.getLogin()));
        curr.setUserName(Rec.getLogin());
        BridgeApp.LogDebug(Logs.Srvcs.SetText("SMTP_Password length= "+Rec.getPassword().length()));
        curr.setPassword(Rec.getPassword());
        curr.setPort(Integer.toString(Rec.getPort()));
        BridgeApp.LogDebug(Logs.Srvcs.SetText("Port= "+Rec.getPort()));
        curr.setSSL(Rec.isSSL());
        BridgeApp.LogDebug(Logs.Srvcs.SetText("SSL= "+Rec.isSSL()));
        // this information will be used for message building
        /* I think we can store this information locally in this (CSmtpFilesSender) class
         * like public keys... 
        curr.setMethodDN( ((dbMailRecord) vMailRecords.elementAt(i)).getMethodDN());
        BridgeApp.LogDebug(Logs.Srvcs.SetText("DN Method= "+curr.getMethodDN() ));
        curr.setMethodSign( ((dbMailRecord) vMailRecords.elementAt(i)).getMethodPGPSign());
        BridgeApp.LogDebug(Logs.Srvcs.SetText("Signature Method= "+curr.getMethodSign() ));
        */  
        oIndMap.put(new Integer(iSmtpIndex), curr);
        mSenders.add(curr);
      } else {
        String sOld = curr.getTo();
        sOld += "," +
            ( (dbMailRecord) vMailRecords.elementAt(i)).
            getMailDomainAddr();
        curr.setTo(sOld);
      }
      mPubKeys[i] = ( (dbMailRecord) vMailRecords.elementAt(i)).getPubKeyN();
//---      
      mMethodDN[i] = ((dbMailRecord) vMailRecords.elementAt(i)).getMethodDN();
      BridgeApp.LogDebug(Logs.Srvcs.SetText("DN Method= "+mMethodDN));
      mMethodSign[i] = ((dbMailRecord) vMailRecords.elementAt(i)).getMethodPGPSign();
      BridgeApp.LogDebug(Logs.Srvcs.SetText("Signature Method= "+mMethodSign));
      mReplayTo[i]=Rec.getReplayTo();
      if ( mReplayTo[i].compareTo("")==0 ) mReplayTo[i]=sFrom; // use the same addr as SMTP server
      BridgeApp.LogDebug(Logs.Srvcs.SetText("ReplayTo= " + mReplayTo ));
//---      
      mSecKeys[i] = ((dbMailRecord) vMailRecords.elementAt(i)).getPrivKeyID();
      BridgeApp.LogDebug(Logs.Srvcs.SetText("PrivateKeyID for sign= " + mSecKeys[i]));
      mPasses[i] = mUser.getDecryptPwd(mSecKeys[i]);
//System.out.println("pass = " + mPasses[i]);
    }
    BridgeApp.LogDebug(Logs.Srvcs.SetText("mSenders.size= "+mSenders.size()));
    BridgeApp.LogDebug(Logs.Srvcs.SetText("mPubKeys.length= "+mPubKeys.length));
  }

  public void setXTag(XTag xtag) {
    mXTag = xtag;
  }

  public void setEraseNames(boolean bErase) {
    mEraseNamesFlag = bErase;
  }
  // smtp
  // private Sender mSender;
  private Vector mSenders;
  private String mComments;
  private String[] mPubKeys;
  private int[] mMethodDN;
  private int[] mMethodSign;
  private String[] mSecKeys;
  private String[] mPasses;
  private String[] mReplayTo;

  //XTag:
  private XTag mXTag=new XTag(); // 23.07.05 it will assign a default tag
                                 // this tag can be changes using setXTag method
  private boolean mDeleteFilesFlag;
  private boolean mEraseNamesFlag;

//  private GpgWrapper mWrapper;

  private IProcessOwner mProcessOwner = null;
}// end class CSmtpFileManager

//****************************************************************************
// FilesManager which sends files on Dicom server
//
class CFilesDecryptor
    extends IFilesManager {
  public CFilesDecryptor(dbUser oUser) {
    //
    // Configuring base Command string
    //
    //mDecryptPassword = oUser.getDecryptPwd();
    //mPubKeyN = oUser.getKeyN();
    mWaitDirPath = oUser.getWaitDirPath();
    mMailParser = new MailParser(new CParseMsgContr(oUser));
    
    mUser = oUser;
  }

  public void ProcessFiles(Vector vFiles) {
    BridgeApp.LogInfo(Logs.Srvcs.SetText("Files for decrypt: " +
                                         vFiles.size()));

    Vector vFailed = new Vector();

    // Create empty properties
    Properties props = new Properties();

    // Get session
    Session session = Session.getInstance(props, null); //Default

    // Processing files
    for (int i = 0; i < vFiles.size(); i++) {
      boolean bMailParsedOk = false;

      String sFileName = (String) vFiles.get(i);
      File oFile = new File(sFileName);
      String sTargetFilePath = oFile.getPath() + "." + DBConsts.DECRYPT_TMP_EXT;

      boolean bDecryptedOk = true;
      String sDecryptPassword = null;
      
      try {
        Vector keys = GpgWrapper.getEncryptedTo(sFileName);
     
        int keyIndex = 0;
        while ((keyIndex<keys.size()) &&
        		((sDecryptPassword = mUser.getDecryptPwd(((GpgKeyInfo)keys.elementAt(keyIndex)).getKeyID()))==null)) {
          
          ++keyIndex;
        }
        if (sDecryptPassword == null){
        	String encryptedTo="";
        	for (Iterator it=keys.iterator(); it.hasNext();){
        		String temp = ((GpgKeyInfo)it.next()).getKeyID();
        		encryptedTo += temp+"; ";
        	}
        	throw new RuntimeException("Password not found for KeyIDs: "+ encryptedTo);
        	
        }
        GpgWrapper.decrypt(sFileName, sTargetFilePath, sDecryptPassword);
      }
      catch (badSignatureException bse) {
    	  BridgeApp.LogError(Logs.Srvcs.SetText(bse.getMessage()));
    	  bMailParsedOk = true;
    	  bDecryptedOk = false;
    	  String sID = (new File(sTargetFilePath)).getName().split("_part")[0];
    	  String sPartNum = (new File(sTargetFilePath)).getName().split("_part")[1];
    	  String sBadSign = dbReportCode.print(dbReportCode.ERROR, dbReportCode.GPG_CORRUPT_SIGN);
    	  
    	  if ((sPartNum != "") && (sPartNum != null)){
    		  int part_num = Integer.parseInt(sPartNum);
    		   if (dbQuery.setQueryUpdate("update vfiles set report=\'"
						+ sBadSign + "\' where id like \'" + sID
						+ "\' and number=" + part_num) == null) {
					BridgeApp.LogError(Logs.Srvcs.SetText("Error: 2.2.1 cannot update vfiles.report"));
				}
    	  }else{
     		   if (dbQuery.setQueryUpdate("update dmt set report=\'"
						+ sBadSign + "\' where id like \'" + sID) == null) {
					BridgeApp.LogError(Logs.Srvcs.SetText("Error: 2.2.1 cannot update dmt.report"));
				}   		  
    	  }
      }
      catch (Exception e) {
        BridgeApp.LogError(Logs.Srvcs.SetText("Decrypt Exception: "));
        BridgeApp.LogError(e);
        
        String sMoveTo = mWaitDirPath + File.separator + oFile.getName();
        if (! oFile.renameTo(new File(sMoveTo)))
          BridgeApp.LogError(Logs.Srvcs.SetText("Can not move file: "+sFileName
                            + " to " + sMoveTo));
        bDecryptedOk = false;
      }

      //
      // Parse decrypted message
      //
      if (bDecryptedOk) {
        FileInputStream mailFile = null;
        try {
          mailFile = new FileInputStream(
              sTargetFilePath);
          //BridgeApp.LogDebug(Logs.Srvcs.SetText(
          //        "create mime message using section and mailFile "));
          MimeMessage savedMailMessage = new MimeMessage(session,
              mailFile);
          //BridgeApp.LogDebug(Logs.Srvcs.SetText(
          //"in target file "+sTargetFilePath+" get name  end split _part "));
          String sID = (new File(sTargetFilePath)).getName().split("_part")[0];
          //BridgeApp.LogDebug(Logs.Srvcs.SetText(
          //        "set header for a saved mail message with ID="+sID));
                  
		  savedMailMessage.setHeader("PARSER_TransID", sID);
		  //BridgeApp.LogDebug(Logs.Srvcs.SetText(
		  //        "Mail parsing starting ..."));
		  // Debug try to save the mimie message
		  //FileOutputStream omime=new FileOutputStream("my_test_mime.txt");
		  //savedMailMessage.writeTo(omime);
          //end Debug
		  mMailParser.parse(savedMailMessage);
          bMailParsedOk = true;
        }
        catch (Exception ex) {
          BridgeApp.LogError(Logs.Srvcs.SetText(
              "Exception while mail parse: " + ex.getMessage()));
          BridgeApp.LogError(ex);
          //ex.printStackTrace(System.out);
        }
        try {
          mailFile.close();
        }
        catch (Exception ex) {}
      }

      // Delete tmp files
      if (! (new File(sTargetFilePath)).delete()) {
        BridgeApp.LogWarning(Logs.Srvcs.SetText("Delete the Message file: " +
                                                sTargetFilePath +
                                                "cannot be deleted"));
      }
      if (bMailParsedOk) {
        if (! ( (new File(sFileName)).delete())) {
          BridgeApp.LogWarning(Logs.Srvcs.SetText("Decryptor try to delete : " +
                                                  sTargetFilePath +
                                                  "cannot be deleted"));
        }
      } else {
        vFailed.add(sFileName);
      }
    }
    BridgeApp.LogInfo(Logs.Srvcs.SetText("Total files failed: " +
                                         vFailed.size()));
  }

/*  public void SetDecryptPassword(String sDecryptPassword) {
    mDecryptPassword = sDecryptPassword;
  }
*/
  //private String mDecryptPassword;
  //private String mPubKeyN;
  private MailParser mMailParser;
  private String mWaitDirPath;
  private dbUser mUser;
}// end class


//*****************************************************************************
// FilesManager which retieves files from Mail servers
//
class CMailFilesReceiver {
  public CMailFilesReceiver(dbUser oUser) {
    mvReceivers = new Vector();

    MailParser oMailParser = new MailParser(new CParseMsgContr(oUser));
    //
    // Initialize Receivers for this user
    //
    //if auto-user then only one checker allowed
    int num_checkers = oUser.getMailTable().getSize();

    for (int i = 0; i < num_checkers; i++) {
      dbMailRecord record = oUser.getMailTable().getRecord(i);
      boolean imapFlag = record.isImap();
      String sServer = record.getDomain();
      String sUser = record.getMail(); // todo change to get UserName
      String sPassword = new String(record.getPassword());
      String sTargetDir = new String(oUser.getExtDirPath());
      long iTimeout = record.getTimeDelay() * 60000;
      String sPort = Integer.toString(record.getPort());
      boolean SSL_flag = record.isSSL();

      Receiver oReceiver = CreateReceiver(oMailParser, sServer, sUser,
                                          sPassword, false, imapFlag,
                                          SSL_flag, sPort, iTimeout);
      mvReceivers.add(oReceiver);
    }
  }

  public void CheckMailForFiles() {
    for (int i = 0; i < mvReceivers.size(); i++) {
      CAutoCheckReceiver oReceiver = (CAutoCheckReceiver) mvReceivers.get(
          i);
      if (System.currentTimeMillis() >= oReceiver.GetNextCheckTime()) {
        BridgeApp.LogInfo(Logs.Srvcs.SetText("Mail Server: " +
                                             oReceiver.getHost() + "\n" +
                                             "User: " +
                                             oReceiver.getUser()));
        oReceiver.getMail();
        oReceiver.SetNextCheckTime(System.currentTimeMillis() +
                                   oReceiver.GetTimeout());
      }
    }
  }

  class CAutoCheckReceiver
      extends Receiver {
    public CAutoCheckReceiver() {
      mNextCheckTime = 0;
      mTimeout = 30000;
    }

    public void SetNextCheckTime(long iTime) {
      mNextCheckTime = iTime;
    }

    public long GetNextCheckTime() {
      return mNextCheckTime;
    }

    public void SetTimeout(long iTime) {
      mTimeout = iTime;
    }

    public long GetTimeout() {
      return mTimeout;
    }

    private long mNextCheckTime;
    private long mTimeout;
  }

  private CAutoCheckReceiver CreateReceiver(MailParser oMailParser,
                                            String host,
                                            String user, String pass,
                                            boolean leaveMessage,
                                            boolean imap_flag,
                                            boolean sslFlag,
                                            String port, long iTimeout) {
    CAutoCheckReceiver mailReceiver = new CAutoCheckReceiver();
    mailReceiver.setHost(host);
    mailReceiver.setUser(user);
    mailReceiver.setPass(pass);
    mailReceiver.setPort(port);
    mailReceiver.setSSL(sslFlag);
//    mailReceiver.setDir("c:\\test\\");
    mailReceiver.setMailParser(oMailParser);
    mailReceiver.setLeaveFlag(leaveMessage);
    mailReceiver.SetTimeout(iTimeout);
    mailReceiver.setIMAP(imap_flag);

    return mailReceiver;
  }

  private Vector mvReceivers;
}// end class

/*
 class CParseMailContr extends IParseController
 {
    CParseMailContr(String encrypted)
    {
        mEncryptedDir = encrypted;
    }

    public void setEncryptedDir(String sparam)
    {
        mEncryptedDir = sparam;
    }

    public void NewEncryptedFile(Part oPart) throws Exception
    {
        String saveTo = FileTools.makeUniqName();
        saveTo += ("_crp.asc");
        saveTo = mEncryptedDir + File.separator + saveTo;

        FileOutputStream fos = new FileOutputStream(saveTo, false);
        oPart.writeTo(fos);
        fos.close();
    }

    private String mEncryptedDir;
 }
 */


//****************************************************************************
//  Files Managers
//public class FilesManagers {
//  public static void main(String[] args) {
//  }
//}

//============================================================================
//                       CExtDoc
//   The class which executes the applications by the extension
//   Bob 30.07.2004
//============================================================================

class CExtDoc {
  private String mExePath;
  private String mOpt;
  private String mArchPath;

  //----------------------------------------------------------------------
  public CExtDoc(Vector vTableData) {
    //The Vecor have to be size 4 String
    //1 = EXT
    //2 = path + name
    //3 = options
    //4 = comments

    mExePath = (String) vTableData.get(1);
    mOpt = (String) vTableData.get(2);
  }

  //------------------------------------------------------------------------
  //Execute the $filename and move it to $sArchPath when closed
  public void execute(String filename, String sArchPath, boolean move_files) {

    int arrSize = 3;
    String sCmd[] = new String[arrSize];
    int sCmdCount = 0;
    sCmd[sCmdCount++] = mExePath;
    sCmd[sCmdCount++] = filename;
    sCmd[sCmdCount++] = mOpt;

    try {
      Process p = Runtime.getRuntime().exec(sCmd);
      // Strange?! but required
      InputStreamReader in = new InputStreamReader(p.getInputStream());
      in.close();

      if (p.waitFor() == 0) {
        //Application normally closed, move the file into archive folder
        if (move_files) {
          File temp = new File(filename);
          File arch_file = new File(sArchPath + File.separator +
                                    temp.getName());
          //Check if the file with the same name is allready exists?
          if (arch_file.exists()) {
            arch_file.delete();
          }
          //Move the file from WAIT to ARCHIV
          temp.renameTo(arch_file);
        }
      }
      else {
        // something wrong
        // do nothing, i.e. don't move the file
      }

    }
    catch (Exception ex) {

    }

  }

  //------------------------------------------------------------------------
}

//==========================================================================
//                     CAppExecutor
//         executes the application by given extension
// Bob 30.07.2004
//==========================================================================
//------------------------------------------------------------------------
//  Thread class to run applications
//-----------------------------------------------------------------------

class AppFileProcessor
    extends Thread {
  private CAppExecutor mFilesProcessor;
  private Vector mFiles;

  public AppFileProcessor(Vector TableData, dbUser oUser) {
    mFilesProcessor = new CAppExecutor(TableData, oUser);
    mFiles = new Vector();
  }

  public void processFile(String filename) {
    mFiles.clear();
    mFiles.add(filename);
    this.start();
  }

  public void run() {
    mFilesProcessor.ProcessFiles(mFiles);
    if (mFilesProcessor.isUnsupportedFile()) {
      JOptionPane.showMessageDialog(null,
                                    "Please define the application for this file",
                                    "Unsupported file type!",
                                    JOptionPane.ERROR_MESSAGE);
    }

  }
}

//----------------------------------------------------------------------------

class CAppExecutor
    extends IFilesManager {

  private HashMap mExtDocMap;
  private boolean isValid;
  private Vector mExts;
  private dbUser mUser;
  private boolean UnsupportedFile;
  private boolean mTerminated;

  public boolean isUnsupportedFile() {
    return UnsupportedFile;
  }

  public void Terminate() {
    mTerminated = true;
  }

  //Constructor converts the XML Table data into HashMap of ExtDoc`s
  public CAppExecutor(Vector TableData, dbUser oUser) {
    mUser = oUser;
    if (TableData.size() > 0) {
      mExtDocMap = new HashMap();
      mExts = new Vector();
      for (int extCount = 0; extCount < TableData.size(); ++extCount) {
        String currExt = (String) ( (Vector) TableData.get(extCount)).
            get(0);
        mExts.add(currExt);
        CExtDoc currDoc = new CExtDoc( (Vector) TableData.get(extCount));
        mExtDocMap.put(currExt, currDoc);
      }
      isValid = true;
      mTerminated = false;
    }
    else {
      isValid = false;
    }
  }

  //-------------------------------------------------------------------------
  public CExtDoc getExtProcessor(String ext) {
    CExtDoc result = null;
    if (isValid) {
      result = (CExtDoc) mExtDocMap.get(ext);
    }
    return result;
  }

  //-------------------------------------------------------------------------
  public boolean isValid() {
    return isValid;
  }

  //-------------------------------------------------------------------------
  public void ProcessFiles(Vector vFiles) {
    //DEBUG
    //System.out.println("Process Files TEST:");
    //System.out.println("---------------------------");
    UnsupportedFile = true;
    if (isValid) {
      for (int i = 0; (i < vFiles.size()) && !mTerminated; i++) {
        String sFileName = (String) vFiles.get(i);

        //DEBUG
        //System.out.println("Loop" + i);
        //System.out.println(sFileName);

        for (int j = 0; j < mExts.size(); j++) {
          String currExt = (String) mExts.get(j);
          if (sFileName.endsWith(currExt)) {  // if extension is as expected

            //DEBUG
            //System.out.println("EXT - " + currExt + " detected");

            CExtDoc currExtDoc = (CExtDoc) mExtDocMap.get(currExt);
            boolean b_move_files = false;

            //DEBUG
            /*System.out.println("--- App Executor DEBUG --------- ");
            System.out.println("User: " + mUser.getName());
            System.out.println("WaitDir: " + mUser.getWaitDirPath());
            System.out.println("ArchiveDir: " + mUser.getArchiveDirPath());
            System.out.println("FilePath: " + sFileName);
            System.out.println("---------------------------------- ");
            */
            if (sFileName.indexOf(mUser.getWaitDirPath()) != -1) {
              b_move_files = true;
            }
            else {
              b_move_files = false;
            }
            //
            // extract last dir name. If this name is not waited, add in to archiveDirPath
            // create a new directory at the Archive, copy data
            //
            int index = sFileName.lastIndexOf(System.getProperty("file.separator"));
            String path=sFileName.substring(0,index);
            int index2=path.lastIndexOf(System.getProperty("file.separator"));
            String subdir_name=path.substring(index2);
            if ( mUser.getWaitDirPath().endsWith(subdir_name) ){
              subdir_name="";
            }
            String archive_path=new String(mUser.getArchiveDirPath() + subdir_name);
            File dir=new File( archive_path);
            if (!dir.exists()){ dir.mkdir();}
            currExtDoc.execute(sFileName, archive_path,
                               b_move_files);
            UnsupportedFile = false;
          }
        }
      }
    }
  }

  //-------------------------------------------------------------------------

}
